@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith8.cc,v 1.34 2004/08/18 21:29:07 sshwarts Exp $
+// $Id: arith8.cc,v 1.36 2005/05/20 20:06:50 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -30,8 +30,7 @@
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
-  void
-BX_CPU_C::ADD_EbGb(bxInstruction_c *i)
+void BX_CPU_C::ADD_EbGb(bxInstruction_c *i)
 {
   Bit8u op2, op1, sum;
 
@@ -41,18 +40,17 @@ BX_CPU_C::ADD_EbGb(bxInstruction_c *i)
     op1 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
     sum = op1 + op2;
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), sum);
-    }
+  }
   else {
     read_RMW_virtual_byte(i->seg(), RMAddr(i), &op1);
     sum = op1 + op2;
     Write_RMW_virtual_byte(sum);
-    }
+  }
 
   SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD8);
 }
 
-  void
-BX_CPU_C::ADD_GbEb(bxInstruction_c *i)
+void BX_CPU_C::ADD_GbEb(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
 
@@ -61,19 +59,18 @@ BX_CPU_C::ADD_GbEb(bxInstruction_c *i)
   if (i->modC0()) {
     op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
     sum = op1 + op2;
-    }
+  }
   else {
     read_virtual_byte(i->seg(), RMAddr(i), &op2);
     sum = op1 + op2;
-    }
+  }
 
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), sum);
 
   SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD8);
 }
 
-  void
-BX_CPU_C::ADD_ALIb(bxInstruction_c *i)
+void BX_CPU_C::ADD_ALIb(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
 
@@ -85,8 +82,7 @@ BX_CPU_C::ADD_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD8);
 }
 
-  void
-BX_CPU_C::ADC_EbGb(bxInstruction_c *i)
+void BX_CPU_C::ADC_EbGb(bxInstruction_c *i)
 {
   Bit8u op2, op1, sum;
   bx_bool temp_CF = getB_CF();
@@ -97,19 +93,18 @@ BX_CPU_C::ADC_EbGb(bxInstruction_c *i)
     op1 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
     sum = op1 + op2 + temp_CF;
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), sum);
-    }
+  }
   else {
     read_RMW_virtual_byte(i->seg(), RMAddr(i), &op1);
     sum = op1 + op2 + temp_CF;
     Write_RMW_virtual_byte(sum);
-    }
+  }
 
   SET_FLAGS_OSZAPC_8(op1, op2, sum,
 	       (temp_CF) ? BX_INSTR_ADC8 : BX_INSTR_ADD8);
 }
 
-  void
-BX_CPU_C::ADC_GbEb(bxInstruction_c *i)
+void BX_CPU_C::ADC_GbEb(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
   bx_bool temp_CF = getB_CF();
@@ -118,10 +113,10 @@ BX_CPU_C::ADC_GbEb(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     read_virtual_byte(i->seg(), RMAddr(i), &op2);
-    }
+  }
 
   sum = op1 + op2 + temp_CF;
 
@@ -131,8 +126,7 @@ BX_CPU_C::ADC_GbEb(bxInstruction_c *i)
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), sum);
 }
 
-  void
-BX_CPU_C::ADC_ALIb(bxInstruction_c *i)
+void BX_CPU_C::ADC_ALIb(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
   bx_bool temp_CF = getB_CF();
@@ -146,8 +140,7 @@ BX_CPU_C::ADC_ALIb(bxInstruction_c *i)
 	       (temp_CF) ? BX_INSTR_ADC8 : BX_INSTR_ADD8);
 }
 
-  void
-BX_CPU_C::SBB_EbGb(bxInstruction_c *i)
+void BX_CPU_C::SBB_EbGb(bxInstruction_c *i)
 {
   Bit8u op2_8, op1_8, diff_8;
   bx_bool temp_CF = getB_CF();
@@ -158,19 +151,18 @@ BX_CPU_C::SBB_EbGb(bxInstruction_c *i)
     op1_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
     diff_8 = op1_8 - (op2_8 + temp_CF);
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), diff_8);
-    }
+  }
   else {
     read_RMW_virtual_byte(i->seg(), RMAddr(i), &op1_8);
     diff_8 = op1_8 - (op2_8 + temp_CF);
     Write_RMW_virtual_byte(diff_8);
-    }
+  }
 
   SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8,
 	       (temp_CF) ? BX_INSTR_SBB8 : BX_INSTR_SUB8);
 }
 
-  void
-BX_CPU_C::SBB_GbEb(bxInstruction_c *i)
+void BX_CPU_C::SBB_GbEb(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
   bx_bool temp_CF = getB_CF();
@@ -179,10 +171,10 @@ BX_CPU_C::SBB_GbEb(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     read_virtual_byte(i->seg(), RMAddr(i), &op2_8);
-    }
+  }
 
   diff_8 = op1_8 - (op2_8 + temp_CF);
 
@@ -192,8 +184,7 @@ BX_CPU_C::SBB_GbEb(bxInstruction_c *i)
 	       (temp_CF) ? BX_INSTR_SBB8 : BX_INSTR_SUB8);
 }
 
-  void
-BX_CPU_C::SBB_ALIb(bxInstruction_c *i)
+void BX_CPU_C::SBB_ALIb(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
   bx_bool temp_CF = getB_CF();
@@ -207,8 +198,7 @@ BX_CPU_C::SBB_ALIb(bxInstruction_c *i)
 	       (temp_CF) ? BX_INSTR_SBB8 : BX_INSTR_SUB8);
 }
 
-  void
-BX_CPU_C::SBB_EbIb(bxInstruction_c *i)
+void BX_CPU_C::SBB_EbIb(bxInstruction_c *i)
 {
   Bit8u op2_8, op1_8, diff_8;
   bx_bool temp_CF = getB_CF();
@@ -219,19 +209,18 @@ BX_CPU_C::SBB_EbIb(bxInstruction_c *i)
     op1_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
     diff_8 = op1_8 - (op2_8 + temp_CF);
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), diff_8);
-    }
+  }
   else {
     read_RMW_virtual_byte(i->seg(), RMAddr(i), &op1_8);
     diff_8 = op1_8 - (op2_8 + temp_CF);
     Write_RMW_virtual_byte(diff_8);
-    }
+  }
 
   SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8,
 	       (temp_CF) ? BX_INSTR_SBB8 : BX_INSTR_SUB8);
 }
 
-  void
-BX_CPU_C::SUB_EbGb(bxInstruction_c *i)
+void BX_CPU_C::SUB_EbGb(bxInstruction_c *i)
 {
   Bit8u op2_8, op1_8, diff_8;
 
@@ -241,18 +230,17 @@ BX_CPU_C::SUB_EbGb(bxInstruction_c *i)
     op1_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
     diff_8 = op1_8 - op2_8;
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), diff_8);
-    }
+  }
   else {
     read_RMW_virtual_byte(i->seg(), RMAddr(i), &op1_8);
     diff_8 = op1_8 - op2_8;
     Write_RMW_virtual_byte(diff_8);
-    }
+  }
 
   SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB8);
 }
 
-  void
-BX_CPU_C::SUB_GbEb(bxInstruction_c *i)
+void BX_CPU_C::SUB_GbEb(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
 
@@ -260,10 +248,10 @@ BX_CPU_C::SUB_GbEb(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     read_virtual_byte(i->seg(), RMAddr(i), &op2_8);
-    }
+  }
 
   diff_8 = op1_8 - op2_8;
 
@@ -272,8 +260,7 @@ BX_CPU_C::SUB_GbEb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB8);
 }
 
-  void
-BX_CPU_C::SUB_ALIb(bxInstruction_c *i)
+void BX_CPU_C::SUB_ALIb(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
 
@@ -285,8 +272,7 @@ BX_CPU_C::SUB_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB8);
 }
 
-  void
-BX_CPU_C::CMP_EbGb(bxInstruction_c *i)
+void BX_CPU_C::CMP_EbGb(bxInstruction_c *i)
 {
   Bit8u op2_8, op1_8;
 
@@ -294,10 +280,10 @@ BX_CPU_C::CMP_EbGb(bxInstruction_c *i)
 
   if (i->modC0()) {
     op1_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     read_virtual_byte(i->seg(), RMAddr(i), &op1_8);
-    }
+  }
 
 #if defined(BX_HostAsm_Cmp8)
   Bit32u flags32;
@@ -309,8 +295,7 @@ BX_CPU_C::CMP_EbGb(bxInstruction_c *i)
 #endif
 }
 
-  void
-BX_CPU_C::CMP_GbEb(bxInstruction_c *i)
+void BX_CPU_C::CMP_GbEb(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8;
 
@@ -318,10 +303,10 @@ BX_CPU_C::CMP_GbEb(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     read_virtual_byte(i->seg(), RMAddr(i), &op2_8);
-    }
+  }
 
 #if defined(BX_HostAsm_Cmp8)
   Bit32u flags32;
@@ -333,13 +318,10 @@ BX_CPU_C::CMP_GbEb(bxInstruction_c *i)
 #endif
 }
 
-  void
-BX_CPU_C::CMP_ALIb(bxInstruction_c *i)
+void BX_CPU_C::CMP_ALIb(bxInstruction_c *i)
 {
-  Bit8u op1_8, op2_8;
-
-  op1_8 = AL;
-  op2_8 = i->Ib();
+  Bit8u op1_8 = AL;
+  Bit8u op2_8 = i->Ib();
 
 #if defined(BX_HostAsm_Cmp8)
   Bit32u flags32;
@@ -351,8 +333,7 @@ BX_CPU_C::CMP_ALIb(bxInstruction_c *i)
 #endif
 }
 
-  void
-BX_CPU_C::XADD_EbGb(bxInstruction_c *i)
+void BX_CPU_C::XADD_EbGb(bxInstruction_c *i)
 {
 #if (BX_CPU_LEVEL >= 4) || (BX_CPU_LEVEL_HACKED >= 4)
   Bit8u op2, op1, sum;
@@ -374,14 +355,14 @@ BX_CPU_C::XADD_EbGb(bxInstruction_c *i)
     //       For example:  XADD AL, AL
     BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), op1);
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), sum);
-    }
+  }
   else {
     read_RMW_virtual_byte(i->seg(), RMAddr(i), &op1);
     sum = op1 + op2;
     Write_RMW_virtual_byte(sum);
     /* and write destination into source */
     BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), op1);
-    }
+  }
 
   SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD8);
 #else
@@ -390,8 +371,7 @@ BX_CPU_C::XADD_EbGb(bxInstruction_c *i)
 #endif
 }
 
-  void
-BX_CPU_C::ADD_EbIb(bxInstruction_c *i)
+void BX_CPU_C::ADD_EbIb(bxInstruction_c *i)
 {
   Bit8u op2, op1, sum;
 
@@ -401,18 +381,17 @@ BX_CPU_C::ADD_EbIb(bxInstruction_c *i)
     op1 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
     sum = op1 + op2;
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), sum);
-    }
+  }
   else {
     read_RMW_virtual_byte(i->seg(), RMAddr(i), &op1);
     sum = op1 + op2;
     Write_RMW_virtual_byte(sum);
-    }
+  }
 
   SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD8);
 }
 
-  void
-BX_CPU_C::ADC_EbIb(bxInstruction_c *i)
+void BX_CPU_C::ADC_EbIb(bxInstruction_c *i)
 {
   Bit8u op2, op1, sum;
   bx_bool temp_CF = getB_CF();
@@ -434,8 +413,7 @@ BX_CPU_C::ADC_EbIb(bxInstruction_c *i)
 	       (temp_CF) ? BX_INSTR_ADC8 : BX_INSTR_ADD8);
 }
 
-  void
-BX_CPU_C::SUB_EbIb(bxInstruction_c *i)
+void BX_CPU_C::SUB_EbIb(bxInstruction_c *i)
 {
   Bit8u op2_8, op1_8, diff_8;
 
@@ -455,8 +433,7 @@ BX_CPU_C::SUB_EbIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB8);
 }
 
-  void
-BX_CPU_C::CMP_EbIb(bxInstruction_c *i)
+void BX_CPU_C::CMP_EbIb(bxInstruction_c *i)
 {
   Bit8u op2_8, op1_8;
 
@@ -464,10 +441,10 @@ BX_CPU_C::CMP_EbIb(bxInstruction_c *i)
 
   if (i->modC0()) {
     op1_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     read_virtual_byte(i->seg(), RMAddr(i), &op1_8);
-    }
+  }
 
 #if defined(BX_HostAsm_Cmp8)
   Bit32u flags32;
@@ -479,27 +456,25 @@ BX_CPU_C::CMP_EbIb(bxInstruction_c *i)
 #endif
 }
 
-  void
-BX_CPU_C::NEG_Eb(bxInstruction_c *i)
+void BX_CPU_C::NEG_Eb(bxInstruction_c *i)
 {
   Bit8u op1_8, diff_8;
 
   if (i->modC0()) {
     op1_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    diff_8 = 0 - op1_8;
+    diff_8 = -op1_8;
     BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), diff_8);
   }
   else {
     read_RMW_virtual_byte(i->seg(), RMAddr(i), &op1_8);
-    diff_8 = 0 - op1_8;
+    diff_8 = -op1_8;
     Write_RMW_virtual_byte(diff_8);
   }
 
   SET_FLAGS_OSZAPC_RESULT_8(diff_8, BX_INSTR_NEG8);
 }
 
-  void
-BX_CPU_C::INC_Eb(bxInstruction_c *i)
+void BX_CPU_C::INC_Eb(bxInstruction_c *i)
 {
   Bit8u op1;
 
@@ -517,8 +492,7 @@ BX_CPU_C::INC_Eb(bxInstruction_c *i)
   SET_FLAGS_OSZAP_RESULT_8(op1, BX_INSTR_INC8);
 }
 
-  void
-BX_CPU_C::DEC_Eb(bxInstruction_c *i)
+void BX_CPU_C::DEC_Eb(bxInstruction_c *i)
 {
   Bit8u op1_8;
 
@@ -536,18 +510,17 @@ BX_CPU_C::DEC_Eb(bxInstruction_c *i)
   SET_FLAGS_OSZAP_RESULT_8(op1_8, BX_INSTR_DEC8);
 }
 
-  void
-BX_CPU_C::CMPXCHG_EbGb(bxInstruction_c *i)
+void BX_CPU_C::CMPXCHG_EbGb(bxInstruction_c *i)
 {
 #if (BX_CPU_LEVEL >= 4) || (BX_CPU_LEVEL_HACKED >= 4)
   Bit8u op2_8, op1_8, diff_8;
 
   if (i->modC0()) {
     op1_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     read_RMW_virtual_byte(i->seg(), RMAddr(i), &op1_8);
-    }
+  }
 
   diff_8 = AL - op1_8;
 
@@ -559,10 +532,10 @@ BX_CPU_C::CMPXCHG_EbGb(bxInstruction_c *i)
 
     if (i->modC0()) {
       BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op2_8);
-      }
+    }
     else {
       Write_RMW_virtual_byte(op2_8);
-      }
+    }
   }
   else {
     // accumulator <-- dest
@@ -570,6 +543,7 @@ BX_CPU_C::CMPXCHG_EbGb(bxInstruction_c *i)
   }
 
 #else
-  BX_PANIC(("CMPXCHG_EbGb: not supported for cpulevel <= 3"));
+  BX_INFO(("CMPXCHG_EbGb: not supported for cpulevel <= 3"));
+  UndefinedOpcode(i);
 #endif
 }
