@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: data_xfer16.cc,v 1.31 2004/08/13 20:00:03 sshwarts Exp $
+// $Id: data_xfer16.cc,v 1.33.2.1 2005/07/07 08:15:25 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,7 +25,6 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #define LOG_THIS BX_CPU_THIS_PTR
@@ -33,14 +32,14 @@
 
 void BX_CPU_C::MOV_RXIw(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR gen_reg[i->opcodeReg()].word.rx = i->Iw();
+  BX_WRITE_16BIT_REG(i->opcodeReg(), i->Iw());
 }
 
 void BX_CPU_C::XCHG_RXAX(bxInstruction_c *i)
 {
   Bit16u temp16 = AX;
-  AX = BX_CPU_THIS_PTR gen_reg[i->opcodeReg()].word.rx;
-  BX_CPU_THIS_PTR gen_reg[i->opcodeReg()].word.rx = temp16;
+  AX = BX_READ_16BIT_REG(i->opcodeReg());
+  BX_WRITE_16BIT_REG(i->opcodeReg(), temp16);
 }
 
 void BX_CPU_C::MOV_EEwGw(bxInstruction_c *i)
@@ -84,14 +83,14 @@ void BX_CPU_C::MOV_EwSw(bxInstruction_c *i)
   if (i->modC0()) {
     if ( i->os32L() ) {
       BX_WRITE_32BIT_REGZ(i->rm(), seg_reg);
-      }
+    }
     else {
       BX_WRITE_16BIT_REG(i->rm(), seg_reg);
-      }
     }
+  }
   else {
     write_virtual_word(i->seg(), RMAddr(i), &seg_reg);
-    }
+  }
 }
 
 void BX_CPU_C::MOV_SwEw(bxInstruction_c *i)
@@ -115,10 +114,10 @@ void BX_CPU_C::MOV_SwEw(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_word(i->seg(), RMAddr(i), &op2_16);
-    }
+  }
 
   load_seg_reg(&BX_CPU_THIS_PTR sregs[i->nnn()], op2_16);
 
@@ -145,122 +144,92 @@ void BX_CPU_C::LEA_GwM(bxInstruction_c *i)
 
 void BX_CPU_C::MOV_AXOw(bxInstruction_c *i)
 {
-  /* read from memory address */
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    read_virtual_word(i->seg(), i->Id(), &AX);
-    }
-  else {
-    read_virtual_word(BX_SEG_REG_DS, i->Id(), &AX);
-    }
+  read_virtual_word(i->seg(), i->Id(), &AX);
 }
 
 void BX_CPU_C::MOV_OwAX(bxInstruction_c *i)
 {
-  /* write to memory address */
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    write_virtual_word(i->seg(), i->Id(), &AX);
-    }
-  else {
-    write_virtual_word(BX_SEG_REG_DS, i->Id(), &AX);
-    }
+  write_virtual_word(i->seg(), i->Id(), &AX);
 }
 
 void BX_CPU_C::MOV_EwIw(bxInstruction_c *i)
 {
-    Bit16u op2_16 = i->Iw();
+  Bit16u op2_16 = i->Iw();
 
-    /* now write sum back to destination */
-    if (i->modC0()) {
-      BX_WRITE_16BIT_REG(i->rm(), op2_16);
-      }
-    else {
-      write_virtual_word(i->seg(), RMAddr(i), &op2_16);
-      }
+  /* now write sum back to destination */
+  if (i->modC0()) {
+    BX_WRITE_16BIT_REG(i->rm(), op2_16);
+  }
+  else {
+    write_virtual_word(i->seg(), RMAddr(i), &op2_16);
+  }
 }
 
+#if BX_CPU_LEVEL >= 3
 void BX_CPU_C::MOVZX_GwEb(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL < 3
-  BX_INFO(("MOVZX_GvEb: not supported on < 386"));
-  UndefinedOpcode(i);
-#else
   Bit8u  op2_8;
 
   if (i->modC0()) {
     op2_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_byte(i->seg(), RMAddr(i), &op2_8);
-    }
+  }
 
-    /* zero extend byte op2 into word op1 */
-    BX_WRITE_16BIT_REG(i->nnn(), (Bit16u) op2_8);
-#endif /* BX_CPU_LEVEL < 3 */
+  /* zero extend byte op2 into word op1 */
+  BX_WRITE_16BIT_REG(i->nnn(), (Bit16u) op2_8);
 }
 
 void BX_CPU_C::MOVZX_GwEw(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL < 3
-  BX_INFO(("MOVZX_GvEw: not supported on < 386"));
-  UndefinedOpcode(i);
-#else
   Bit16u op2_16;
 
   if (i->modC0()) {
     op2_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_word(i->seg(), RMAddr(i), &op2_16);
-    }
+  }
 
-    /* normal move */
-    BX_WRITE_16BIT_REG(i->nnn(), op2_16);
-#endif /* BX_CPU_LEVEL < 3 */
+  /* normal move */
+  BX_WRITE_16BIT_REG(i->nnn(), op2_16);
 }
 
 void BX_CPU_C::MOVSX_GwEb(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL < 3
-  BX_INFO(("MOVSX_GvEb: not supported on < 386"));
-  UndefinedOpcode(i);
-#else
   Bit8u op2_8;
 
   if (i->modC0()) {
     op2_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_byte(i->seg(), RMAddr(i), &op2_8);
-    }
+  }
 
-    /* sign extend byte op2 into word op1 */
-    BX_WRITE_16BIT_REG(i->nnn(), (Bit8s) op2_8);
-#endif /* BX_CPU_LEVEL < 3 */
+  /* sign extend byte op2 into word op1 */
+  BX_WRITE_16BIT_REG(i->nnn(), (Bit8s) op2_8);
 }
 
 void BX_CPU_C::MOVSX_GwEw(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL < 3
-  BX_INFO(("MOVSX_GvEw: not supported on < 386"));
-  UndefinedOpcode(i);
-#else
   Bit16u op2_16;
 
   if (i->modC0()) {
     op2_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_word(i->seg(), RMAddr(i), &op2_16);
-    }
+  }
 
-    /* normal move */
-    BX_WRITE_16BIT_REG(i->nnn(), op2_16);
-#endif /* BX_CPU_LEVEL < 3 */
+  /* normal move */
+  BX_WRITE_16BIT_REG(i->nnn(), op2_16);
 }
+#endif
 
 void BX_CPU_C::XCHG_EwGw(bxInstruction_c *i)
 {
@@ -275,21 +244,20 @@ void BX_CPU_C::XCHG_EwGw(bxInstruction_c *i)
   }
 #endif
 
-    /* op2_16 is a register, op2_addr is an index of a register */
-    op2_16 = BX_READ_16BIT_REG(i->nnn());
+  op2_16 = BX_READ_16BIT_REG(i->nnn());
 
-    /* op1_16 is a register or memory reference */
-    if (i->modC0()) {
-      op1_16 = BX_READ_16BIT_REG(i->rm());
-      BX_WRITE_16BIT_REG(i->rm(), op2_16);
-      }
-    else {
-      /* pointer, segment address pair */
-      read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
-      Write_RMW_virtual_word(op2_16);
-      }
+  /* op1_16 is a register or memory reference */
+  if (i->modC0()) {
+    op1_16 = BX_READ_16BIT_REG(i->rm());
+    BX_WRITE_16BIT_REG(i->rm(), op2_16);
+  }
+  else {
+    /* pointer, segment address pair */
+    read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
+    Write_RMW_virtual_word(op2_16);
+  }
 
-    BX_WRITE_16BIT_REG(i->nnn(), op1_16);
+  BX_WRITE_16BIT_REG(i->nnn(), op1_16);
 }
 
 void BX_CPU_C::CMOV_GwEw(bxInstruction_c *i)
@@ -322,19 +290,19 @@ void BX_CPU_C::CMOV_GwEw(bxInstruction_c *i)
     case 0x14F: condition = !get_ZF() && (getB_SF() == getB_OF()); break;
     default:
       BX_PANIC(("CMOV_GwEw: default case"));
-    }
+  }
 
   if (i->modC0()) {
     op2_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_word(i->seg(), RMAddr(i), &op2_16);
-    }
+  }
 
   if (condition) {
     BX_WRITE_16BIT_REG(i->nnn(), op2_16);
-    }
+  }
 #else
   BX_INFO(("CMOV_GwEw: required P6 support, use --enable-cpu-level=6 option"));
   UndefinedOpcode(i);
