@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: data_xfer32.cc,v 1.32 2004/11/02 20:39:45 sshwarts Exp $
+// $Id: data_xfer32.cc,v 1.34.2.1 2005/07/07 08:14:09 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,15 +25,12 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
   
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
-
-  void
-BX_CPU_C::XCHG_ERXEAX(bxInstruction_c *i)
+void BX_CPU_C::XCHG_ERXEAX(bxInstruction_c *i)
 {
   Bit32u temp32 = EAX;
 
@@ -48,44 +45,35 @@ BX_CPU_C::XCHG_ERXEAX(bxInstruction_c *i)
 
 void BX_CPU_C::MOV_ERXId(bxInstruction_c *i)
 {
-#if BX_SUPPORT_X86_64
-  BX_CPU_THIS_PTR gen_reg[i->opcodeReg()].rrx = i->Id();
-#else
-  BX_CPU_THIS_PTR gen_reg[i->opcodeReg()].dword.erx = i->Id();
-#endif
+  BX_WRITE_32BIT_REGZ(i->opcodeReg(), i->Id());
 }
 
-  void
-BX_CPU_C::MOV_EEdGd(bxInstruction_c *i)
+void BX_CPU_C::MOV_EEdGd(bxInstruction_c *i)
 {
   write_virtual_dword(i->seg(), RMAddr(i), &BX_READ_32BIT_REG(i->nnn()));
 }
 
-  void
-BX_CPU_C::MOV_EGdGd(bxInstruction_c *i)
+void BX_CPU_C::MOV_EGdGd(bxInstruction_c *i)
 {
   Bit32u op2_32 = BX_READ_32BIT_REG(i->nnn());
   BX_WRITE_32BIT_REGZ(i->rm(), op2_32);
 }
 
-  void
-BX_CPU_C::MOV_GdEGd(bxInstruction_c *i)
+void BX_CPU_C::MOV_GdEGd(bxInstruction_c *i)
 {
   // 2nd modRM operand Ex, is known to be a general register Gd.
   Bit32u op2_32 = BX_READ_32BIT_REG(i->rm());
   BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
 }
 
-  void
-BX_CPU_C::MOV_GdEEd(bxInstruction_c *i)
+void BX_CPU_C::MOV_GdEEd(bxInstruction_c *i)
 {
   // 2nd modRM operand Ex, is known to be a memory operand, Ed.
   read_virtual_dword(i->seg(), RMAddr(i), &BX_READ_32BIT_REG(i->nnn()));
   BX_CLEAR_64BIT_HIGH(i->nnn());
 }
 
-  void
-BX_CPU_C::LEA_GdM(bxInstruction_c *i)
+void BX_CPU_C::LEA_GdM(bxInstruction_c *i)
 {
   if (i->modC0()) {
     BX_INFO(("LEA_GdM: op2 is a register"));
@@ -96,161 +84,117 @@ BX_CPU_C::LEA_GdM(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->nnn(), RMAddr(i));
 }
 
-
-  void
-BX_CPU_C::MOV_EAXOd(bxInstruction_c *i)
+void BX_CPU_C::MOV_EAXOd(bxInstruction_c *i)
 {
-  /* read from memory address */
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    read_virtual_dword(i->seg(), i->Id(), &EAX);
-  }
-  else {
-    read_virtual_dword(BX_SEG_REG_DS, i->Id(), &EAX);
-  }
-
+  read_virtual_dword(i->seg(), i->Id(), &EAX);
   BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RAX);
 }
 
-  void
-BX_CPU_C::MOV_OdEAX(bxInstruction_c *i)
+void BX_CPU_C::MOV_OdEAX(bxInstruction_c *i)
 {
-  /* write to memory address */
-  if (!BX_NULL_SEG_REG(i->seg())) {
-    write_virtual_dword(i->seg(), i->Id(), &EAX);
-    }
+  write_virtual_dword(i->seg(), i->Id(), &EAX);
+}
+
+void BX_CPU_C::MOV_EdId(bxInstruction_c *i)
+{
+  Bit32u op2_32 = i->Id();
+
+  /* now write sum back to destination */
+  if (i->modC0()) {
+    BX_WRITE_32BIT_REGZ(i->rm(), op2_32);
+  }
   else {
-    write_virtual_dword(BX_SEG_REG_DS, i->Id(), &EAX);
-    }
+    write_virtual_dword(i->seg(), RMAddr(i), &op2_32);
+  }
 }
 
-  void
-BX_CPU_C::MOV_EdId(bxInstruction_c *i)
+#if BX_CPU_LEVEL >= 3
+void BX_CPU_C::MOVZX_GdEb(bxInstruction_c *i)
 {
-    Bit32u op2_32 = i->Id();
-
-    /* now write sum back to destination */
-    if (i->modC0()) {
-      BX_WRITE_32BIT_REGZ(i->rm(), op2_32);
-      }
-    else {
-      write_virtual_dword(i->seg(), RMAddr(i), &op2_32);
-      }
-}
-
-  void
-BX_CPU_C::MOVZX_GdEb(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL < 3
-  BX_INFO(("MOVZX_GvEb: not supported on < 386"));
-  UndefinedOpcode(i);
-#else
   Bit8u  op2_8;
 
   if (i->modC0()) {
     op2_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_byte(i->seg(), RMAddr(i), &op2_8);
-    }
+  }
 
-    /* zero extend byte op2 into dword op1 */
-    BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_8);
-#endif /* BX_CPU_LEVEL < 3 */
+  /* zero extend byte op2 into dword op1 */
+  BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_8);
 }
 
-  void
-BX_CPU_C::MOVZX_GdEw(bxInstruction_c *i)
+void BX_CPU_C::MOVZX_GdEw(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL < 3
-  BX_INFO(("MOVZX_GvEw: not supported on < 386"));
-  UndefinedOpcode(i);
-#else
   Bit16u op2_16;
 
   if (i->modC0()) {
     op2_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_word(i->seg(), RMAddr(i), &op2_16);
-    }
+  }
 
-    /* zero extend word op2 into dword op1 */
-    BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_16);
-#endif /* BX_CPU_LEVEL < 3 */
+  /* zero extend word op2 into dword op1 */
+  BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_16);
 }
 
-  void
-BX_CPU_C::MOVSX_GdEb(bxInstruction_c *i)
+void BX_CPU_C::MOVSX_GdEb(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL < 3
-  BX_INFO(("MOVSX_GvEb: not supported on < 386"));
-  UndefinedOpcode(i);
-#else
   Bit8u op2_8;
 
   if (i->modC0()) {
     op2_8 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_byte(i->seg(), RMAddr(i), &op2_8);
-    }
+  }
 
-    /* sign extend byte op2 into dword op1 */
-    BX_WRITE_32BIT_REGZ(i->nnn(), (Bit8s) op2_8);
-#endif /* BX_CPU_LEVEL < 3 */
+  /* sign extend byte op2 into dword op1 */
+  BX_WRITE_32BIT_REGZ(i->nnn(), (Bit8s) op2_8);
 }
 
-  void
-BX_CPU_C::MOVSX_GdEw(bxInstruction_c *i)
+void BX_CPU_C::MOVSX_GdEw(bxInstruction_c *i)
 {
-#if BX_CPU_LEVEL < 3
-  BX_INFO(("MOVSX_GvEw: not supported on < 386"));
-  UndefinedOpcode(i);
-#else
   Bit16u op2_16;
 
   if (i->modC0()) {
     op2_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_word(i->seg(), RMAddr(i), &op2_16);
-    }
+  }
 
-    /* sign extend word op2 into dword op1 */
-    BX_WRITE_32BIT_REGZ(i->nnn(), (Bit16s) op2_16);
-#endif /* BX_CPU_LEVEL < 3 */
+  /* sign extend word op2 into dword op1 */
+  BX_WRITE_32BIT_REGZ(i->nnn(), (Bit16s) op2_16);
 }
+#endif
 
-
-  void
-BX_CPU_C::XCHG_EdGd(bxInstruction_c *i)
+void BX_CPU_C::XCHG_EdGd(bxInstruction_c *i)
 {
-    Bit32u op2_32, op1_32;
+  Bit32u op2_32, op1_32;
 
-    /* op2_32 is a register, op2_addr is an index of a register */
-    op2_32 = BX_READ_32BIT_REG(i->nnn());
+  op2_32 = BX_READ_32BIT_REG(i->nnn());
 
-    /* op1_32 is a register or memory reference */
-    if (i->modC0()) {
-      op1_32 = BX_READ_32BIT_REG(i->rm());
-      BX_WRITE_32BIT_REGZ(i->rm(), op2_32);
-      }
-    else {
-      /* pointer, segment address pair */
-      read_RMW_virtual_dword(i->seg(), RMAddr(i), &op1_32);
-      Write_RMW_virtual_dword(op2_32);
-      }
+  /* op1_32 is a register or memory reference */
+  if (i->modC0()) {
+    op1_32 = BX_READ_32BIT_REG(i->rm());
+    BX_WRITE_32BIT_REGZ(i->rm(), op2_32);
+  }
+  else {
+    /* pointer, segment address pair */
+    read_RMW_virtual_dword(i->seg(), RMAddr(i), &op1_32);
+    Write_RMW_virtual_dword(op2_32);
+  }
 
-    BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
+  BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
 }
 
-
-  void
-BX_CPU_C::CMOV_GdEd(bxInstruction_c *i)
+void BX_CPU_C::CMOV_GdEd(bxInstruction_c *i)
 {
 #if (BX_CPU_LEVEL >= 6) || (BX_CPU_LEVEL_HACKED >= 6)
   // Note: CMOV accesses a memory source operand (read), regardless
@@ -280,19 +224,19 @@ BX_CPU_C::CMOV_GdEd(bxInstruction_c *i)
     case 0x14F: condition = !get_ZF() && (getB_SF() == getB_OF()); break;
     default:
       BX_PANIC(("CMOV_GdEd: default case"));
-    }
+  }
 
   if (i->modC0()) {
     op2_32 = BX_READ_32BIT_REG(i->rm());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_dword(i->seg(), RMAddr(i), &op2_32);
-    }
+  }
 
   if (condition) {
     BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-    }
+  }
 #else
   BX_INFO(("CMOV_GdEd: -enable-cpu-level=6 required"));
   UndefinedOpcode(i);
