@@ -1,7 +1,9 @@
 /////////////////////////////////////////////////////////////////////////
+// $Id: fetchdecode.h,v 1.17 2005/05/12 18:07:41 sshwarts Exp $
+/////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
-//          Written by Stanislav Shwartsman <gate at fidonet.org.il>
+//          Written by Stanislav Shwartsman <stl at fidonet.org.il>
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -16,14 +18,17 @@
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
-
+//
+/////////////////////////////////////////////////////////////////////////
 
 
 #ifndef BX_COMMON_FETCHDECODE_TABLES_H
 #define BX_COMMON_FETCHDECODE_TABLES_H
 
 //
-// Common FetchDecode Opcode Tables
+// This file contains common IA-32/IA-64 opcode tables, like FPU opcode
+// table, 3DNow! opcode table or SSE opcode groups (choose the opcode
+// according to instruction prefixes)
 //
 
 BX_CPP_INLINE Bit16u FetchWORD(Bit8u *iptr)
@@ -40,19 +45,70 @@ BX_CPP_INLINE Bit32u FetchDWORD(Bit8u *iptr)
    return data;
 }
 
+#if BX_SUPPORT_X86_64
 BX_CPP_INLINE Bit64u FetchQWORD(Bit8u *iptr)
 {
    Bit64u data;
    ReadHostQWordFromLittleEndian(iptr, data);
    return data;
 }
+#endif
 
 //
-// This file contains common IA-32/IA-64 opcode tables, like FPU opcode
-// table, 3DNow! opcode table or SSE opcode groups (choose the opcode
-// according to instruction prefixes)
+// Common FetchDecode Opcode Tables
 //
 
+static BxOpcodeInfo_t opcodesADD_EwIw[2] = {
+  { BxLockable, &BX_CPU_C::ADD_EEwIw },
+  { 0,          &BX_CPU_C::ADD_EGwIw }
+  };
+
+static BxOpcodeInfo_t opcodesADD_EdId[2] = {
+  { BxLockable, &BX_CPU_C::ADD_EEdId },
+  { 0,          &BX_CPU_C::ADD_EGdId }
+  };
+
+static BxOpcodeInfo_t opcodesADD_GwEw[2] = {
+  { 0,          &BX_CPU_C::ADD_GwEEw },
+  { 0,          &BX_CPU_C::ADD_GwEGw }
+  };
+
+static BxOpcodeInfo_t opcodesADD_GdEd[2] = {
+  { 0,          &BX_CPU_C::ADD_GdEEd },
+  { 0,          &BX_CPU_C::ADD_GdEGd }
+  };
+
+static BxOpcodeInfo_t opcodesMOV_GbEb[2] = {
+  { 0,          &BX_CPU_C::MOV_GbEEb },
+  { 0,          &BX_CPU_C::MOV_GbEGb }
+  };
+
+static BxOpcodeInfo_t opcodesMOV_GwEw[2] = {
+  { 0,          &BX_CPU_C::MOV_GwEEw },
+  { 0,          &BX_CPU_C::MOV_GwEGw }
+  };
+
+static BxOpcodeInfo_t opcodesMOV_GdEd[2] = {
+  { 0,          &BX_CPU_C::MOV_GdEEd },
+  { 0,          &BX_CPU_C::MOV_GdEGd }
+  };
+
+static BxOpcodeInfo_t opcodesMOV_EbGb[2] = {
+  { 0,          &BX_CPU_C::MOV_EEbGb },
+  { 0,          &BX_CPU_C::MOV_EGbGb }
+  };
+
+static BxOpcodeInfo_t opcodesMOV_EwGw[2] = {
+  { 0,          &BX_CPU_C::MOV_EEwGw },
+  { 0,          &BX_CPU_C::MOV_EGwGw }
+  };
+
+static BxOpcodeInfo_t opcodesMOV_EdGd[2] = {
+  { 0,          &BX_CPU_C::MOV_EEdGd },
+  { 0,          &BX_CPU_C::MOV_EGdGd }
+  };
+
+
 #if BX_SUPPORT_FPU
 
 /* ************************************************************************ */
@@ -960,45 +1016,6 @@ static BxOpcodeInfo_t Bx3DNowOpcodeInfo[256] = {
 #endif
 
 
-/* ************************************************************************ */
-/* SSE Opcode Groups */
-
-
-/*** Duplicate SSE/SSE2 instructions ***/
-
-#define MOVUPD_VpdWpd  /* 66 0f 10 */ MOVUPS_VpsWps    /*    0f 10 */
-#define MOVUPD_WpdVpd  /* 66 0f 11 */ MOVUPS_WpsVps    /*    0f 11 */
-#define MOVAPD_VpdWpd  /* 66 0f 28 */ MOVAPS_VpsWps    /*    0f 28 */
-#define MOVAPD_WpdVpd  /* 66 0f 29 */ MOVAPS_WpsVps    /*    0f 29 */
-#define MOVDQU_VdqWdq  /* f3 0f 6f */ MOVUPS_VpsWps    /*    0f 10 */
-#define MOVDQU_WdqVdq  /* f3 0f 7f */ MOVUPS_WpsVps    /*    0f 11 */
-#define MOVDQA_VdqWdq  /* 66 0f 6f */ MOVAPS_VpsWps    /*    0f 28 */
-#define MOVDQA_WdqVdq  /* 66 0f 7f */ MOVAPS_WpsVps    /*    0f 29 */
-
-#define UNPCKHPS_VpsWq /*    0f 15 */ PUNPCKHDQ_VdqWq  /* 66 0f 6a */
-#define UNPCKLPS_VpsWq /*    0f 14 */ PUNPCKLDQ_VdqWq  /* 66 0f 62 */
-
-#define ANDPS_VpsWps   /*    0f 54 */ PAND_VdqWdq      /* 66 0f db */
-#define ANDPD_VpdWpd   /* 66 0f 54 */ PAND_VdqWdq      /* 66 0f db */
-#define ANDNPS_VpsWps  /*    0f 55 */ PANDN_VdqWdq     /* 66 0f df */
-#define ANDNPD_VpdWpd  /* 66 0f 55 */ PANDN_VdqWdq     /* 66 0f df */
-#define ORPS_VpsWps    /*    0f 56 */ POR_VdqWdq       /* 66 0f eb */
-#define ORPD_VpdWpd    /* 66 0f 56 */ POR_VdqWdq       /* 66 0f eb */
-#define XORPS_VpsWps   /*    0f 57 */ PXOR_VdqWdq      /* 66 0f ef */
-#define XORPD_VpdWpd   /* 66 0f 57 */ PXOR_VdqWdq      /* 66 0f ef */
-
-#define UNPCKHPD_VpdWq /* 66 0f 15 */ PUNPCKHQDQ_VdqWq /* 66 0f 6d */
-#define UNPCKLPD_VpdWq /* 66 0f 14 */ PUNPCKLQDQ_VdqWq /* 66 0f 6c */
-
-#define MOVLPD_VsdMq   /* 66 0f 12 */ MOVLPS_VpsMq     /*    0f 12 */
-#define MOVLPD_MqVsd   /* 66 0f 13 */ MOVLPS_MqVps     /*    0f 13 */
-#define MOVHPD_VsdMq   /* 66 0f 16 */ MOVHPS_VpsMq     /*    0f 16 */
-#define MOVHPD_MqVsd   /* 66 0f 17 */ MOVHPS_MqVps     /*    0f 17 */
-
-#define MOVNTPD_MdqVpd /* 66 0f 2b */ MOVNTPS_MdqVps   /*    0f 2b */
-#define MOVNTDQ_MdqVdq /* 66 0f e7 */ MOVNTPD_MdqVpd   /* 66 0f 2b */
-
-
 /* ********** */
 /* SSE Groups */
 /* ********** */
