@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical16.cc,v 1.18 2002/10/25 18:26:28 sshwarts Exp $
+// $Id: logical16.cc,v 1.23 2005/05/20 20:06:50 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,19 +25,12 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
-
-
-
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
-
-
-  void
-BX_CPU_C::XOR_EwGw(bxInstruction_c *i)
+void BX_CPU_C::XOR_EwGw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16, result_16;
 
@@ -46,37 +39,33 @@ BX_CPU_C::XOR_EwGw(bxInstruction_c *i)
   if (i->modC0()) {
     op1_16 = BX_READ_16BIT_REG(i->rm());
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Xor16)
     Bit32u flags32;
-
     asmXor16(result_16, op1_16, op2_16, flags32);
     setEFlagsOSZAPC(flags32);
 #else
     result_16 = op1_16 ^ op2_16;
 #endif
     BX_WRITE_16BIT_REG(i->rm(), result_16);
-    }
+  }
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Xor16)
     Bit32u flags32;
-
     asmXor16(result_16, op1_16, op2_16, flags32);
     setEFlagsOSZAPC(flags32);
 #else
     result_16 = op1_16 ^ op2_16;
 #endif
     Write_RMW_virtual_word(result_16);
-    }
+  }
 
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_XOR16);
+#if !defined(BX_HostAsm_Xor16)
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 #endif
 }
 
-
-  void
-BX_CPU_C::XOR_GwEw(bxInstruction_c *i)
+void BX_CPU_C::XOR_GwEw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, result_16;
   unsigned nnn = i->nnn();
@@ -85,82 +74,71 @@ BX_CPU_C::XOR_GwEw(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_word(i->seg(), RMAddr(i), &op2_16);
-    }
+  }
 
   result_16 = op1_16 ^ op2_16;
 
   BX_WRITE_16BIT_REG(nnn, result_16);
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_XOR16);
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 }
 
-
-  void
-BX_CPU_C::XOR_AXIw(bxInstruction_c *i)
+void BX_CPU_C::XOR_AXIw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, sum_16;
 
   op1_16 = AX;
   op2_16 = i->Iw();
-
   sum_16 = op1_16 ^ op2_16;
-
   AX = sum_16;
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_INSTR_XOR16);
+  SET_FLAGS_OSZAPC_RESULT_16(sum_16, BX_INSTR_LOGIC16);
 }
 
-  void
-BX_CPU_C::XOR_EwIw(bxInstruction_c *i)
+void BX_CPU_C::XOR_EwIw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16, result_16;
 
-
   op2_16 = i->Iw();
 
   if (i->modC0()) {
     op1_16 = BX_READ_16BIT_REG(i->rm());
     result_16 = op1_16 ^ op2_16;
     BX_WRITE_16BIT_REG(i->rm(), result_16);
-    }
+  }
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
     result_16 = op1_16 ^ op2_16;
     Write_RMW_virtual_word(result_16);
-    }
+  }
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_XOR16);
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 }
 
-
-  void
-BX_CPU_C::OR_EwIw(bxInstruction_c *i)
+void BX_CPU_C::OR_EwIw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16, result_16;
 
-
   op2_16 = i->Iw();
 
   if (i->modC0()) {
     op1_16 = BX_READ_16BIT_REG(i->rm());
     result_16 = op1_16 | op2_16;
     BX_WRITE_16BIT_REG(i->rm(), result_16);
-    }
+  }
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
     result_16 = op1_16 | op2_16;
     Write_RMW_virtual_word(result_16);
-    }
+  }
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_OR16);
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 }
 
-
-  void
-BX_CPU_C::NOT_Ew(bxInstruction_c *i)
+void BX_CPU_C::NOT_Ew(bxInstruction_c *i)
 {
   Bit16u op1_16, result_16;
 
@@ -168,17 +146,15 @@ BX_CPU_C::NOT_Ew(bxInstruction_c *i)
     op1_16 = BX_READ_16BIT_REG(i->rm());
     result_16 = ~op1_16;
     BX_WRITE_16BIT_REG(i->rm(), result_16);
-    }
+  }
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
     result_16 = ~op1_16;
     Write_RMW_virtual_word(result_16);
-    }
+  }
 }
 
-
-  void
-BX_CPU_C::OR_EwGw(bxInstruction_c *i)
+void BX_CPU_C::OR_EwGw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16, result_16;
 
@@ -188,19 +164,17 @@ BX_CPU_C::OR_EwGw(bxInstruction_c *i)
     op1_16 = BX_READ_16BIT_REG(i->rm());
     result_16 = op1_16 | op2_16;
     BX_WRITE_16BIT_REG(i->rm(), result_16);
-    }
+  }
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
     result_16 = op1_16 | op2_16;
     Write_RMW_virtual_word(result_16);
-    }
+  }
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_OR16);
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 }
 
-
-  void
-BX_CPU_C::OR_GwEw(bxInstruction_c *i)
+void BX_CPU_C::OR_GwEw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, result_16;
 
@@ -208,47 +182,36 @@ BX_CPU_C::OR_GwEw(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_word(i->seg(), RMAddr(i), &op2_16);
-    }
+  }
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Or16)
   Bit32u flags32;
-
   asmOr16(result_16, op1_16, op2_16, flags32);
   setEFlagsOSZAPC(flags32);
 #else
   result_16 = op1_16 | op2_16;
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 #endif
 
   BX_WRITE_16BIT_REG(i->nnn(), result_16);
-
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_OR16);
-#endif
 }
 
-
-  void
-BX_CPU_C::OR_AXIw(bxInstruction_c *i)
+void BX_CPU_C::OR_AXIw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, sum_16;
 
   op1_16 = AX;
   op2_16 = i->Iw();
-
   sum_16 = op1_16 | op2_16;
-
   AX = sum_16;
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_INSTR_OR16);
+  SET_FLAGS_OSZAPC_RESULT_16(sum_16, BX_INSTR_LOGIC16);
 }
 
-
-
-  void
-BX_CPU_C::AND_EwGw(bxInstruction_c *i)
+void BX_CPU_C::AND_EwGw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16, result_16;
 
@@ -257,9 +220,8 @@ BX_CPU_C::AND_EwGw(bxInstruction_c *i)
   if (i->modC0()) {
     op1_16 = BX_READ_16BIT_REG(i->rm());
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And16)
     Bit32u flags32;
-
     asmAnd16(result_16, op1_16, op2_16, flags32);
     setEFlagsOSZAPC(flags32);
 #else
@@ -267,13 +229,12 @@ BX_CPU_C::AND_EwGw(bxInstruction_c *i)
 #endif
 
     BX_WRITE_16BIT_REG(i->rm(), result_16);
-    }
+  }
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And16)
     Bit32u flags32;
-
     asmAnd16(result_16, op1_16, op2_16, flags32);
     setEFlagsOSZAPC(flags32);
 #else
@@ -281,16 +242,14 @@ BX_CPU_C::AND_EwGw(bxInstruction_c *i)
 #endif
 
     Write_RMW_virtual_word(result_16);
-    }
+  }
 
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_AND16);
+#if !defined(BX_HostAsm_And16)
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 #endif
 }
 
-
-  void
-BX_CPU_C::AND_GwEw(bxInstruction_c *i)
+void BX_CPU_C::AND_GwEw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, result_16;
 
@@ -298,54 +257,43 @@ BX_CPU_C::AND_GwEw(bxInstruction_c *i)
 
   if (i->modC0()) {
     op2_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_word(i->seg(), RMAddr(i), &op2_16);
-    }
+  }
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And16)
   Bit32u flags32;
-
   asmAnd16(result_16, op1_16, op2_16, flags32);
   setEFlagsOSZAPC(flags32);
 #else
   result_16 = op1_16 & op2_16;
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 #endif
 
   BX_WRITE_16BIT_REG(i->nnn(), result_16);
-
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_AND16);
-#endif
 }
 
-
-  void
-BX_CPU_C::AND_AXIw(bxInstruction_c *i)
+void BX_CPU_C::AND_AXIw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, result_16;
 
   op1_16 = AX;
   op2_16 = i->Iw();
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And16)
   Bit32u flags32;
-
   asmAnd16(result_16, op1_16, op2_16, flags32);
   setEFlagsOSZAPC(flags32);
 #else
   result_16 = op1_16 & op2_16;
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 #endif
 
   AX = result_16;
-
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_AND16);
-#endif
 }
 
-  void
-BX_CPU_C::AND_EwIw(bxInstruction_c *i)
+void BX_CPU_C::AND_EwIw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16, result_16;
 
@@ -354,9 +302,8 @@ BX_CPU_C::AND_EwIw(bxInstruction_c *i)
   if (i->modC0()) {
     op1_16 = BX_READ_16BIT_REG(i->rm());
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And16)
     Bit32u flags32;
-
     asmAnd16(result_16, op1_16, op2_16, flags32);
     setEFlagsOSZAPC(flags32);
 #else
@@ -364,13 +311,12 @@ BX_CPU_C::AND_EwIw(bxInstruction_c *i)
 #endif
 
     BX_WRITE_16BIT_REG(i->rm(), result_16);
-    }
+  }
   else {
     read_RMW_virtual_word(i->seg(), RMAddr(i), &op1_16);
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_And16)
     Bit32u flags32;
-
     asmAnd16(result_16, op1_16, op2_16, flags32);
     setEFlagsOSZAPC(flags32);
 #else
@@ -378,16 +324,14 @@ BX_CPU_C::AND_EwIw(bxInstruction_c *i)
 #endif
 
     Write_RMW_virtual_word(result_16);
-    }
+  }
 
-#if !(defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_AND16);
+#if !defined(BX_HostAsm_And16)
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 #endif
 }
 
-
-  void
-BX_CPU_C::TEST_EwGw(bxInstruction_c *i)
+void BX_CPU_C::TEST_EwGw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16;
 
@@ -395,50 +339,39 @@ BX_CPU_C::TEST_EwGw(bxInstruction_c *i)
 
   if (i->modC0()) {
     op1_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_word(i->seg(), RMAddr(i), &op1_16);
-    }
+  }
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Test16)
   Bit32u flags32;
-
   asmTest16(op1_16, op2_16, flags32);
   setEFlagsOSZAPC(flags32);
 #else
-  Bit16u result_16;
-  result_16 = op1_16 & op2_16;
-
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_TEST16);
+  Bit16u result_16 = op1_16 & op2_16;
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 #endif
 }
 
-
-
-  void
-BX_CPU_C::TEST_AXIw(bxInstruction_c *i)
+void BX_CPU_C::TEST_AXIw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16;
 
   op1_16 = AX;
   op2_16 = i->Iw();
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Test16)
   Bit32u flags32;
-
   asmTest16(op1_16, op2_16, flags32);
   setEFlagsOSZAPC(flags32);
 #else
-  Bit16u result_16;
-  result_16 = op1_16 & op2_16;
-
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_TEST16);
+  Bit16u result_16 = op1_16 & op2_16;
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 #endif
 }
 
-
-  void
-BX_CPU_C::TEST_EwIw(bxInstruction_c *i)
+void BX_CPU_C::TEST_EwIw(bxInstruction_c *i)
 {
   Bit16u op2_16, op1_16;
 
@@ -446,20 +379,17 @@ BX_CPU_C::TEST_EwIw(bxInstruction_c *i)
 
   if (i->modC0()) {
     op1_16 = BX_READ_16BIT_REG(i->rm());
-    }
+  }
   else {
     read_virtual_word(i->seg(), RMAddr(i), &op1_16);
-    }
+  }
 
-#if (defined(__i386__) && defined(__GNUC__) && BX_SupportHostAsms)
+#if defined(BX_HostAsm_Test16)
   Bit32u flags32;
-
   asmTest16(op1_16, op2_16, flags32);
   setEFlagsOSZAPC(flags32);
 #else
-  Bit16u result_16;
-  result_16 = op1_16 & op2_16;
-
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, result_16, BX_INSTR_TEST16);
+  Bit16u result_16 = op1_16 & op2_16;
+  SET_FLAGS_OSZAPC_RESULT_16(result_16, BX_INSTR_LOGIC16);
 #endif
 }
