@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: mult8.cc,v 1.11 2002/10/25 11:44:35 bdenney Exp $
+// $Id: mult8.cc,v 1.19 2005/05/20 20:06:50 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -25,180 +25,149 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
-
-
-
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 
-
-  void
-BX_CPU_C::MUL_ALEb(bxInstruction_c *i)
+void BX_CPU_C::MUL_ALEb(bxInstruction_c *i)
 {
   Bit8u op2, op1;
-  Bit16u product_16;
-  bx_bool temp_flag;
 
   op1 = AL;
 
   /* op2 is a register or memory reference */
   if (i->modC0()) {
     op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_byte(i->seg(), RMAddr(i), &op2);
-    }
+  }
 
-  product_16 = op1 * op2;
+  Bit32u product_16  = ((Bit16u) op1) * ((Bit16u) op2);
 
-  /* set EFLAGS:
-   * MUL affects the following flags: C,O
-   */
+  Bit8u product_8l = (product_16 & 0xFF);
+  Bit8u product_8h =  product_16 >> 8;
 
-  temp_flag = ((product_16 & 0xFF00) != 0);
-  SET_FLAGS_OxxxxC(temp_flag, temp_flag);
+  /* set EFLAGS */
+  SET_FLAGS_OSZAPC_S1S2_8(product_8l, product_8h, BX_INSTR_MUL8);
 
   /* now write product back to destination */
-
   AX = product_16;
 }
 
-
-  void
-BX_CPU_C::IMUL_ALEb(bxInstruction_c *i)
+void BX_CPU_C::IMUL_ALEb(bxInstruction_c *i)
 {
   Bit8s op2, op1;
-  Bit16s product_16;
-  Bit16u upper_bits;
-
 
   op1 = AL;
 
   /* op2 is a register or memory reference */
   if (i->modC0()) {
     op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_byte(i->seg(), RMAddr(i), (Bit8u *) &op2);
-    }
+  }
 
-  product_16 = op1 * op2;
+  Bit16s product_16 = op1 * op2;
 
-  /* now write product back to destination */
-
-  AX = product_16;
+  Bit8u product_8l = (product_16 & 0xFF);
+  Bit8u product_8h =  product_16 >> 8;
 
   /* set EFLAGS:
-   * IMUL affects the following flags: C,O
    * IMUL r/m8: condition for clearing CF & OF:
-   *   AL = sign-extend of AL to 16 bits
+   *   AX = sign-extend of AL to 16 bits
    */
-  upper_bits = AX & 0xff80;
-  if (upper_bits==0xff80  ||  upper_bits==0x0000) {
-    SET_FLAGS_OxxxxC(0, 0);
-    }
-  else {
-    SET_FLAGS_OxxxxC(1, 1);
-    }
+  SET_FLAGS_OSZAPC_S1S2_8(product_8l, product_8h, BX_INSTR_IMUL8);
+
+  /* now write product back to destination */
+  AX = product_16;
 }
 
-
-
-  void
-BX_CPU_C::DIV_ALEb(bxInstruction_c *i)
+void BX_CPU_C::DIV_ALEb(bxInstruction_c *i)
 {
   Bit8u op2, quotient_8l, remainder_8;
   Bit16u quotient_16, op1;
 
-
   op1 = AX;
 
   /* op2 is a register or memory reference */
   if (i->modC0()) {
     op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_byte(i->seg(), RMAddr(i), &op2);
-    }
+  }
 
   if (op2 == 0) {
     exception(BX_DE_EXCEPTION, 0, 0);
-    }
+  }
+
   quotient_16 = op1 / op2;
   remainder_8 = op1 % op2;
   quotient_8l = quotient_16 & 0xFF;
 
-  if (quotient_16 != quotient_8l) {
+  if (quotient_16 != quotient_8l)
+  {
     exception(BX_DE_EXCEPTION, 0, 0);
-    }
+  }
 
   /* set EFLAGS:
    * DIV affects the following flags: O,S,Z,A,P,C are undefined
    */
 
 #if INTEL_DIV_FLAG_BUG == 1
-    set_CF(1);
+  set_CF(1);
 #endif
 
   /* now write quotient back to destination */
-
   AL = quotient_8l;
   AH = remainder_8;
 }
 
-
-  void
-BX_CPU_C::IDIV_ALEb(bxInstruction_c *i)
+void BX_CPU_C::IDIV_ALEb(bxInstruction_c *i)
 {
   Bit8s op2, quotient_8l, remainder_8;
   Bit16s quotient_16, op1;
 
-
   op1 = AX;
 
-
   /* op2 is a register or memory reference */
   if (i->modC0()) {
     op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-    }
+  }
   else {
     /* pointer, segment address pair */
     read_virtual_byte(i->seg(), RMAddr(i), (Bit8u *) &op2);
-    }
+  }
 
-  if (op2 == 0) {
+  if (op2 == 0)
+    exception(BX_DE_EXCEPTION, 0, 0);
+
+  /* check MIN_INT divided by -1 case */
+  if ((op1 == ((Bit16s)0x8000)) && (op2 == -1))
     exception(BX_DE_EXCEPTION, 0, 0);
-    }
 
   quotient_16 = op1 / op2;
   remainder_8 = op1 % op2;
   quotient_8l = quotient_16 & 0xFF;
 
-  if (quotient_16 != quotient_8l) {
-BX_INFO(("quotient_16: %04x, remainder_8: %02x, quotient_8l: %02x",
-  (unsigned) quotient_16, (unsigned) remainder_8, (unsigned) quotient_8l));
-AL = quotient_8l;
-AH = remainder_8;
-BX_INFO(("AH: %02x, AL: %02x", (unsigned) AH, (unsigned) AL));
+  if (quotient_16 != quotient_8l)
     exception(BX_DE_EXCEPTION, 0, 0);
-    }
 
   /* set EFLAGS:
    * DIV affects the following flags: O,S,Z,A,P,C are undefined
    */
 
 #if INTEL_DIV_FLAG_BUG == 1
-    set_CF(1);
+  set_CF(1);
 #endif
 
   /* now write quotient back to destination */
-
   AL = quotient_8l;
   AH = remainder_8;
 }
