@@ -1,7 +1,9 @@
 /////////////////////////////////////////////////////////////////////////
+// $Id: xmm.h,v 1.18.2.1 2005/07/07 07:34:15 vruppert Exp $
+/////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2002 Stanislav Shwartsman
-//          Written by Stanislav Shwartsman <gate@fidonet.org.il>
+//   Copyright (c) 2003 Stanislav Shwartsman
+//          Written by Stanislav Shwartsman <stl at fidonet.org.il>
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -152,7 +154,7 @@ typedef union bx_xmm_reg_t {
 
 #define MXCSR_RESET 0x00001F80  /* reset value of the MXCSR register */
 
-struct bx_mxcsr_t 
+struct BOCHSAPI bx_mxcsr_t 
 {
   Bit32u mxcsr;
 
