@@ -37,15 +37,15 @@ static const unsigned char instruction_has_modrm[512] = {
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F 40 */
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F 50 */
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F 60 */
-           1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1, /* 0F 70 */
+           1,1,1,1,1,1,1,0,0,0,0,0,1,1,1,1, /* 0F 70 */
            0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 0F 80 */
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F 90 */
            0,0,0,1,1,1,0,0,0,0,0,1,1,1,1,1, /* 0F A0 */
            1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,1, /* 0F B0 */
            1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0, /* 0F C0 */
-           0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F D0 */
+           1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F D0 */
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F E0 */
-           0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0  /* 0F F0 */
+           1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0  /* 0F F0 */
   /*       -------------------------------           */
   /*       0 1 2 3 4 5 6 7 8 9 a b c d e f           */
 };
@@ -75,20 +75,16 @@ static const unsigned char instruction_has_modrm[512] = {
  *
  *      66h - operand size override prefix
  *      67h - address size override prefix
- *
- *  For each instruction, one prefix may be used from each of these groups
- *  and  be  placed  in any order. Using redundant prefixes (more than one
- *  prefix from a group) is reserved and will cause undefined behaviour.
  */
 
 unsigned disassembler::disasm(bx_bool is_32, 
-        Bit32u base, Bit32u ip, Bit8u *instr, char *disbuf)
+        bx_address base, bx_address ip, Bit8u *instr, char *disbuf)
 {
   i32bit_opsize = is_32;
   i32bit_addrsize = is_32;
   db_eip = ip;
   db_base = base; // cs linear base (base for PM & cs<<4 for RM & VM)
-  instruction_begin = instruction = instr;
+  Bit8u *instruction_begin = instruction = instr;
   displacement.displ32 = 0;
 
   resolve_modrm = NULL;
@@ -109,53 +105,59 @@ unsigned disassembler::disasm(bx_bool is_32,
 
   for(;;)
   {
-          b1 = fetch_byte();
-          entry = &BxDisasmOpcodes[b1];
+    b1 = fetch_byte();
+    entry = &BxDisasmOpcodes[b1];
 
-          if (entry->Attr == _PREFIX)
-          {
-              switch(b1) {
-              case 0xf3:
-                  sse_prefix |= SSE_PREFIX_F3;
-                  break;
-              case 0xf2:
-                  sse_prefix |= SSE_PREFIX_F2;
-                  break;
-              case 0x2e:
-              case 0x36:
-              case 0x3e:
-              case 0x26:
-              case 0x64:
-              case 0x65:
-                  seg_override = entry->Opcode;
-                  break;
-              case 0x66:
-                  i32bit_opsize = !i32bit_opsize;
-                  sse_prefix |= SSE_PREFIX_66;
-                  break;
-              case 0x67:
-                  i32bit_addrsize = !i32bit_addrsize;
-                  break;
-              case 0xf0:      // lock
-                  break;
-              default:
-                  printf("Internal disassembler error !");
-                  return 0;
-            }
+    if (entry->Attr == _PREFIX)
+    {
+      switch(b1) {
+        case 0xf3:
+          sse_prefix |= SSE_PREFIX_F3;
+          break;
 
-            n_prefixes++;
-        }
-        else break;
+        case 0xf2:
+          sse_prefix |= SSE_PREFIX_F2;
+          break;
+
+        case 0x2e:
+        case 0x36:
+        case 0x3e:
+        case 0x26:
+        case 0x64:
+        case 0x65:
+          seg_override = entry->Opcode;
+          break;
+
+        case 0x66:
+          i32bit_opsize = !is_32;
+          sse_prefix |= SSE_PREFIX_66;
+          break;
+
+        case 0x67:
+          i32bit_addrsize = !is_32;
+          break;
+
+        case 0xf0:      // lock
+          break;
+
+         default:
+          printf("Internal disassembler error !\n");
+          return 0;
+      }
+
+      n_prefixes++;
+    }
+    else break;
   }
 
   if (b1 == 0x0f)
   {
-        b1 = 0x100 | fetch_byte();
-        entry = &BxDisasmOpcodes[b1];
+    b1 = 0x100 | fetch_byte();
+    entry = &BxDisasmOpcodes[b1];
   }
 
   if (instruction_has_modrm[b1])
-        decode_modrm();
+    decode_modrm();
 
   int attr = entry->Attr;
   while(attr) 
@@ -184,23 +186,18 @@ unsigned disassembler::disasm(bx_bool is_32,
          if(mod != 3)
          {
              entry = &(entry->AnotherArray[nnn]);
-         }
-         else
-         {
+         } else {
              int index = (b1-0xD8)*64 + (0x3f & modrm);
              entry = &(BxDisasmOpcodeInfoFP[index]);
          }
          break;
 
        case _GRP3DNOW:
-         {
-             int suffix = peek_byte();
-             entry = &(BxDisasm3DNowGroup[suffix]);
-         }
+         entry = &(BxDisasm3DNowGroup[peek_byte()]);
          break;
 
        default:
-         printf("Internal disassembler error !");
+         printf("Internal disassembler error !\n");
          return 0;
     }
 
@@ -210,20 +207,25 @@ unsigned disassembler::disasm(bx_bool is_32,
 
   // print prefixes
   for(unsigned i=0;i<n_prefixes;i++)
-        if (*(instr+i) == 0xF3 || *(instr+i) == 0xF2 || *(instr+i) == 0xF0) 
-             dis_sprintf("%s ", BxDisasmOpcodes[*(instr+i)].Opcode);
+  {
+    if (*(instr+i) == 0xF3 || *(instr+i) == 0xF2 || *(instr+i) == 0xF0) 
+      dis_sprintf("%s ", BxDisasmOpcodes[*(instr+i)].Opcode);
 
-  // print opcode
-  dis_sprintf("%s ", entry->Opcode);
-
-  (this->*entry->Operand1)(entry->Op1Attr);
-  if (entry->Operand2 != &disassembler::XX)
-        dis_sprintf(", ");
-  (this->*entry->Operand2)(entry->Op2Attr);
-  if (entry->Operand3 != &disassembler::XX)
-        dis_sprintf(", ");
-  (this->*entry->Operand3)(entry->Op3Attr);
+    if (entry->Op3Attr == BRANCH_HINT)
+    {
+      if (*(instr+i) == 0x2E) 
+        dis_sprintf("not taken ");
+      if (*(instr+i) == 0x3E) 
+        dis_sprintf("taken ");
+    }
+  }
 
+  // print instruction disassembly
+  if (intel_mode)
+    print_disassembly_intel(entry);
+  else
+    print_disassembly_att  (entry);
+ 
   return(instruction - instruction_begin);
 }
 
@@ -237,3 +239,9 @@ void disassembler::dis_sprintf(char *fmt, ...)
 
   disbufptr += strlen(disbufptr);
 }
+
+void disassembler::dis_putc(char symbol)
+{
+  *disbufptr++ = symbol;
+  *disbufptr = 0;
+}
