@@ -1,11 +1,9 @@
 /////////////////////////////////////////////////////////////////////////
-//  Copyright (C) 2004  MandrakeSoft S.A.
+// $Id: fpu.cc,v 1.13.2.1 2005/07/07 07:23:40 vruppert Exp $
+/////////////////////////////////////////////////////////////////////////
 //
-//    MandrakeSoft S.A.
-//    43, rue d'Aboukir
-//    75002 Paris - France
-//    http://www.linux-mandrake.com/
-//    http://www.mandrakesoft.com/
+//   Copyright (c) 2003 Stanislav Shwartsman
+//          Written by Stanislav Shwartsman <stl at fidonet.org.il>
 //
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
@@ -20,6 +18,7 @@
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//
 /////////////////////////////////////////////////////////////////////////
 
 
@@ -492,11 +491,11 @@ void BX_CPU_C::print_state_FPU()
   static double scale_factor = pow(2.0, -63.0);
 
   Bit32u reg;
-  reg = BX_CPU_THIS_PTR the_i387.cwd;
+  reg = BX_CPU_THIS_PTR the_i387.get_control_word();
   fprintf(stderr, "control word: 0x%04x\n", reg);
-  reg = BX_CPU_THIS_PTR the_i387.swd;
+  reg = BX_CPU_THIS_PTR the_i387.get_status_word();
   fprintf(stderr, "status word:  0x%04x\n", reg);
-  reg = BX_CPU_THIS_PTR the_i387.twd;
+  reg = BX_CPU_THIS_PTR the_i387.get_tag_word();
   fprintf(stderr, "tag word:     0x%04x\n", reg);
   reg = BX_CPU_THIS_PTR the_i387.foo;
   fprintf(stderr, "operand:      0x%04x\n", reg);
@@ -514,7 +513,11 @@ void BX_CPU_C::print_state_FPU()
     const floatx80 &fp = BX_FPU_REG(i);
     double f = pow(2.0, ((0x7fff & fp.exp) - 0x3fff));
     if (fp.exp & 0x8000) f = -f;
+#ifdef _MSC_VER
+    f *= (double)(signed __int64)(fp.fraction>>1) * scale_factor * 2;
+#else
     f *= fp.fraction*scale_factor;
+#endif
     fprintf(stderr, "st(%d):        %.10f (raw 0x%04x:%08x%08x)\n", i, 
           f, fp.exp & 0xffff, fp.fraction >> 32, fp.fraction & 0xffffffff);
   }
