@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: keymap.cc,v 1.16 2003/10/11 10:43:24 vruppert Exp $
+// $Id: keymap.cc,v 1.17 2005/03/27 14:50:07 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002 MandrakeSoft S.A.
@@ -248,7 +248,7 @@ bx_keymap_c::loadKeymap(Bit32u stringToSymbol(const char*), const char* filename
         }
 
       if( hostKey==BX_KEYMAP_UNKNOWN ) {
-        BX_PANIC (("line %d: unknown host key name '%s'",lineCount,hostSym));
+        BX_PANIC (("line %d: unknown host key name '%s' (wrong keymap ?)",lineCount,hostSym));
         continue;
         }
 
