@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: siminterface.h,v 1.127 2004/10/24 20:04:51 vruppert Exp $
+// $Id: siminterface.h,v 1.135 2005/02/01 19:16:19 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 // Before I can describe what this file is for, I have to make the
@@ -87,15 +87,15 @@
 // kind of parameter to ask the user to edit the value.
 //
 // I have been considering whether to use the same strategy for the
-// wxWindows interface, but I'm not sure if I like it.  One problem is
+// wxWidgets interface, but I'm not sure if I like it.  One problem is
 // that in order to declare member functions that are useful for
-// wxWindows, the wxWindows header files would have to be included
+// wxWidgets, the wxWidgets header files would have to be included
 // before the param object definitions.  That means that all the
-// wxwindows headers would have be included when compiling every
+// wxWidgets headers would have be included when compiling every
 // single bochs file.  One of the things I like about the separation
 // between the simulator and CI is that the two parts can be
 // compiled without any knowledge of the other.  Bochs doesn't include
-// <wx.h>, and the wxwindows CI (wxmain.cc) doesn't need to include <bochs.h>.
+// <wx.h>, and the wxWidgets CI (wxmain.cc) doesn't need to include <bochs.h>.
 // Aside from making compiles faster, this enforces the use of the siminterface
 // so it keeps the interface clean (important when we may have multiple UI
 // implementations for example).  This argues for keeping UI-specific
@@ -122,8 +122,10 @@ typedef enum {
   BXP_IPS,
   BXP_REALTIME_PIT,
   BXP_TEXT_SNAPSHOT_CHECK,
+  BXP_VGA_EXTENSION,
   BXP_VGA_UPDATE_INTERVAL,
   BXP_MOUSE_ENABLED,
+  BXP_MOUSE_TYPE,
   BXP_MEM_SIZE,
   BXP_ROM_PATH,
   BXP_ROM_ADDRESS,
@@ -328,10 +330,11 @@ typedef enum {
   BXP_COM4_ENABLED,
   BXP_COM4_MODE,
   BXP_COM4_PATH,
-#define BXP_PARAMS_PER_USB_HUB 3
+#define BXP_PARAMS_PER_USB_HUB 4
   BXP_USB1_ENABLED,
   BXP_USB1_IOADDR,
-  BXP_USB1_IRQ,
+  BXP_USB1_PORT1,
+  BXP_USB1_PORT2,
   BXP_PRIVATE_COLORMAP,
   BXP_FULLSCREEN,
   BXP_SCREENMODE,
@@ -367,7 +370,7 @@ typedef enum {
   BXP_MENU_MISC_2,
   BXP_MENU_RUNTIME,
   BXP_MAX_IPS,
-  BXP_NE2K_PRESENT,
+  BXP_NE2K_ENABLED,
   BXP_NE2K_IOADDR,
   BXP_NE2K_IRQ,
   BXP_NE2K_MACADDR,
@@ -387,7 +390,7 @@ typedef enum {
   BXP_PCIDEV_VENDOR,
   BXP_PCIDEV_DEVICE,
   BXP_PCIDEV,
-  BXP_SB16_PRESENT,
+  BXP_SB16_ENABLED,
   BXP_SB16_MIDIFILE,
   BXP_SB16_WAVEFILE,
   BXP_SB16_LOGFILE,
@@ -512,8 +515,10 @@ typedef enum {
    (bx_id)(BXP_USB1_ENABLED + (((x)-1)*BXP_PARAMS_PER_USB_HUB))
 #define BXP_USBx_IOADDR(x) \
    (bx_id)(BXP_USB1_IOADDR + (((x)-1)*BXP_PARAMS_PER_USB_HUB))
-#define BXP_USBx_IRQ(x) \
-   (bx_id)(BXP_USB1_IRQ + (((x)-1)*BXP_PARAMS_PER_USB_HUB))
+#define BXP_USBx_PORT1(x) \
+   (bx_id)(BXP_USB1_PORT1 + (((x)-1)*BXP_PARAMS_PER_USB_HUB))
+#define BXP_USBx_PORT2(x) \
+   (bx_id)(BXP_USB1_PORT2 + (((x)-1)*BXP_PARAMS_PER_USB_HUB))
 
 // use x=1,2
 #define BXP_PARPORTx_ENABLED(x) \
@@ -601,13 +606,13 @@ typedef enum {
 //
 // Examples:
 //
-// async event: In the wxWindows implementation, both the CI and the
-// VGAW operate in the wxWindows GUI thread.  When the user presses a
-// key, wxWindows sends a wxKeyEvent to the VGAW event handler code in
+// async event: In the wxWidgets implementation, both the CI and the
+// VGAW operate in the wxWidgets GUI thread.  When the user presses a
+// key, wxWidgets sends a wxKeyEvent to the VGAW event handler code in
 // wx.cc.  The VGAW handler then builds a BxEvent with
 // type=BX_ASYNC_EVT_KEY, and fills in the bx_key and raw_scancode
 // fields.  The asynchronous event is placed on the event_queue for
-// the simulator, then the VGAW handler returns.  (With wxWindows and
+// the simulator, then the VGAW handler returns.  (With wxWidgets and
 // many other graphical libaries, the event handler must return
 // quickly because the window will not be updated until it's done.)
 // Some time later, the simulator reaches the point where it checks
@@ -688,7 +693,7 @@ typedef struct {
 //
 // A mouse event can be sent from the VGA window to the Bochs
 // simulator.  It is asynchronous.  Currently unused because mouse
-// events aren't implemented in our wxWindows code yet.
+// events aren't implemented in our wxWidgets code yet.
 typedef struct {
   // type is BX_EVT_MOUSE
   Bit16s dx, dy;           // mouse motion delta
@@ -1221,6 +1226,20 @@ enum {
   BX_RUN_START
 };
 
+enum {
+  BX_MOUSE_TYPE_NONE,
+  BX_MOUSE_TYPE_PS2,
+  BX_MOUSE_TYPE_IMPS2,
+#if BX_SUPPORT_BUSMOUSE
+  BX_MOUSE_TYPE_BUS,
+#endif
+#if BX_SUPPORT_PCIUSB
+  BX_MOUSE_TYPE_USB,
+#endif
+  BX_MOUSE_TYPE_SERIAL,
+  BX_MOUSE_TYPE_SERIAL_WHEEL
+};
+
 #define BX_FLOPPY_NONE   10 // floppy not present
 #define BX_FLOPPY_1_2    11 // 1.2M  5.25"
 #define BX_FLOPPY_1_44   12 // 1.44M 3.5"
@@ -1330,7 +1349,8 @@ typedef struct {
 typedef struct {
   bx_param_bool_c *Oenabled;
   bx_param_num_c *Oioaddr;
-  bx_param_num_c *Oirq;
+  bx_param_string_c *Oport1;
+  bx_param_string_c *Oport2;
   } bx_usb_options;
 
 typedef struct {
@@ -1510,7 +1530,7 @@ typedef struct BOCHSAPI {
   char initial_dir[MAX_PATH];
 #endif
 #ifdef __WXMSW__
-  // these are only used when compiling with wxWindows.  This gives us a
+  // these are only used when compiling with wxWidgets.  This gives us a
   // place to store the data that was passed to WinMain.
   HINSTANCE hInstance;
   HINSTANCE hPrevInstance;
