@@ -1,7 +1,7 @@
 /////////////////////////////////////////////////////////////////
-// $Id: wxmain.h,v 1.44 2003/09/13 16:49:09 vruppert Exp $
+// $Id: wxmain.h,v 1.46 2004/10/03 09:11:28 vruppert Exp $
 /////////////////////////////////////////////////////////////////
-// This file defines variables and classes that the wxWindows .cc files 
+// This file defines variables and classes that the wxWidgets .cc files 
 // share.  It should be included only by wx.cc and wxmain.cc.  
 
 // forward class declaration so that each class can have a pointer to 
@@ -41,6 +41,7 @@ enum
   ID_Edit_Cdrom,  // for toolbar. FIXME: toolbar can't handle >1 cdrom
   ID_Edit_Boot,
   ID_Edit_Memory,
+  ID_Edit_PCI,
   ID_Edit_Sound,
   ID_Edit_Timing,
   ID_Edit_Network,
@@ -139,7 +140,7 @@ public:
   ~MyPanel();
   void OnKeyDown(wxKeyEvent& event);
   void OnKeyUp(wxKeyEvent& event);
-  void OnTimer(wxCommandEvent& event);
+  void OnTimer(wxTimerEvent& event);
   void OnPaint(wxPaintEvent& event);
   void OnMouse(wxMouseEvent& event);
   void MyRefresh ();
@@ -175,7 +176,7 @@ public:
   void OnLogMsg(BxEvent *logMsgEvent);
   void OnEditBoot(wxCommandEvent& event);
   void OnEditMemory(wxCommandEvent& event);
-  void OnEditSpeed(wxCommandEvent& event);
+  void OnEditPCI(wxCommandEvent& event);
   void OnEditSound(wxCommandEvent& event);
   void OnEditTiming(wxCommandEvent& event);
   void OnEditNet(wxCommandEvent& event);
