@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: x.cc,v 1.86 2004/11/06 17:03:43 vruppert Exp $
+// $Id: x.cc,v 1.89.2.1 2005/07/07 07:17:02 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -104,7 +104,7 @@ static unsigned imDepth, imWide, imBPP;
 
 // current cursor coordinates
 static int prev_x=-1, prev_y=-1;
-static int current_x=-1, current_y=-1;
+static int current_x=-1, current_y=-1, current_z=0;
 static unsigned mouse_button_state = 0;
 static bx_bool CTRL_pressed = 0;
 
@@ -155,7 +155,7 @@ static unsigned bx_statusitem_pos[12] = {
   };
 static bx_bool bx_statusitem_active[12];
 static long bx_status_led_green, bx_status_graytext;
-static char bx_status_info_text[32];
+static char bx_status_info_text[34];
 
 static void headerbar_click(int x, int y);
 static void send_keyboard_mouse_status(void);
@@ -347,17 +347,7 @@ bx_x_gui_c::specific_init(int argc, char **argv, unsigned tilewidth, unsigned ti
   unsigned i;
   int x, y;   /* window position */
   unsigned int border_width = 4;  /* four pixels */
-#if BX_CPU_LEVEL < 2
-  char *window_name = "Bochs 8086 emulator, http://bochs.sourceforge.net/";
-#elif BX_CPU_LEVEL == 2
-  char *window_name = "Bochs 80286 emulator, http://bochs.sourceforge.net/";
-#elif BX_CPU_LEVEL == 3
-  char *window_name = "Bochs 80386 emulator, http://bochs.sourceforge.net/";
-#elif BX_CPU_LEVEL == 4
-  char *window_name = "Bochs 80486 emulator, http://bochs.sourceforge.net/";
-#else
-  char *window_name = "Bochs Pentium emulator, http://bochs.sourceforge.net/";
-#endif
+  char *window_name = BOCHS_WINDOW_NAME;
   char *icon_name = "Bochs";
   Pixmap icon_pixmap;
 #if BX_HAVE_XPM_H
@@ -753,6 +743,7 @@ bx_x_gui_c::handle_events(void)
 
   while (XPending(bx_x_display) > 0)  {
     XNextEvent(bx_x_display, &report);
+    current_z = 0;
     switch  (report.type) {
 
     case Expose:
@@ -860,6 +851,16 @@ bx_x_gui_c::handle_events(void)
           send_keyboard_mouse_status();
           mouse_update = 0;
           break;
+        case Button4:
+          current_z = 1;
+          send_keyboard_mouse_status();
+          mouse_update = 0;
+          break;
+        case Button5:
+          current_z = -1;
+          send_keyboard_mouse_status();
+          mouse_update = 0;
+          break;
         }
       break;
 
@@ -923,16 +924,17 @@ send_keyboard_mouse_status(void)
 	BX_DEBUG(("XXX: prev=(%d,%d) curr=(%d,%d)",
 			prev_x, prev_y, current_x, current_y));
 
-  if ( (prev_x!=-1) && (current_x!=-1) && (prev_y!=-1) && (current_y!=-1)) {
-    int dx, dy;
+  if (((prev_x!=-1) && (current_x!=-1) && (prev_y!=-1) && (current_y!=-1)) ||
+     (current_z != 0)) {
+    int dx, dy, dz;
 
     // (mch) consider warping here
     dx = current_x - prev_x - warp_dx;
     dy = -(current_y - prev_y - warp_dy);
+    dz = current_z;
     warp_cursor(warp_home_x-current_x, warp_home_y-current_y);
 
-//BX_INFO(("xxx: MOUSE_MOTION: dx=%d, dy=%d", (int) dx, (int) dy));
-    DEV_mouse_motion (dx, dy, mouse_button_state);
+    DEV_mouse_motion_ext (dx, dy, dz, mouse_button_state);
     //if (warped) {
     //  prev_x = current_x = -1;
     //  prev_y = current_y = -1;
@@ -1788,11 +1790,11 @@ static void disable_cursor ()
 	     mask_width = 16,
 	     mask_height = 16;
 
-      static uint32 shape_bits[(16*16)/32] = {
+      static Bit32u shape_bits[(16*16)/32] = {
 	    0x00000000, 0x00000000, 0x00000000, 0x00000000,
 	    0x00000000, 0x00000000, 0x00000000, 0x00000000,
       };
-      static uint32 mask_bits[(16*16)/32] = {
+      static Bit32u mask_bits[(16*16)/32] = {
 	    0x00000000, 0x00000000, 0x00000000, 0x00000000,
 	    0x00000000, 0x00000000, 0x00000000, 0x00000000,
       };
@@ -1897,8 +1899,11 @@ Bool XPeekEventTimeout( Display *display, XEvent *event_return, struct timeval *
       switch(res)
       {
         case -1: /* select() error - should not happen */ 
-            perror("XPeekEventTimeout: select() failure"); 
-            return(False);
+          if (errno == EINTR) 
+             break; // caused e.g. by alarm(3)
+          perror("XPeekEventTimeout: select() failure"); 
+          return(False);
+
         case  0: /* timeout */
           return(False);
       }
@@ -1908,7 +1913,6 @@ Bool XPeekEventTimeout( Display *display, XEvent *event_return, struct timeval *
     return(True);
 }
 
-#if BX_USE_IDLE_HACK
 void bx_x_gui_c::sim_is_idle () {
   XEvent dummy;
   struct timeval   timeout;   
@@ -1916,7 +1920,6 @@ void bx_x_gui_c::sim_is_idle () {
   timeout.tv_usec = 1000; /* 1/1000 s */  
   XPeekEventTimeout(bx_x_display, &dummy, &timeout);
 }
-#endif
 #endif /* BX_USE_IDLE_HACK */  
 
 
