@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.cc,v 1.11 2003/10/09 19:05:13 sshwarts Exp $
+// $Id: instrument.cc,v 1.13.2.1 2005/07/08 07:17:06 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -31,6 +31,7 @@
 void bx_instr_init(unsigned cpu) {}
 void bx_instr_shutdown(unsigned cpu) {}
 void bx_instr_reset(unsigned cpu) {}
+void bx_instr_hlt(unsigned cpu) {}
 void bx_instr_new_instruction(unsigned cpu) {}
 
 void bx_instr_debug_promt() {}
@@ -46,18 +47,7 @@ void bx_instr_far_branch(unsigned cpu, unsigned what, Bit16u new_cs, bx_address
 void bx_instr_opcode(unsigned cpu, Bit8u *opcode, unsigned len, bx_bool is32) {}
 void bx_instr_fetch_decode_completed(unsigned cpu, const bxInstruction_c *i) {}
 
-void bx_instr_prefix_as(unsigned cpu) {}
-void bx_instr_prefix_os(unsigned cpu) {}
-void bx_instr_prefix_rep(unsigned cpu) {}
-void bx_instr_prefix_repne(unsigned cpu) {}
-void bx_instr_prefix_lock(unsigned cpu) {}
-void bx_instr_prefix_cs(unsigned cpu) {}
-void bx_instr_prefix_ss(unsigned cpu) {}
-void bx_instr_prefix_ds(unsigned cpu) {}
-void bx_instr_prefix_es(unsigned cpu) {}
-void bx_instr_prefix_fs(unsigned cpu) {}
-void bx_instr_prefix_gs(unsigned cpu) {}
-void bx_instr_prefix_extend8b(unsigned cpu) {}
+void bx_instr_prefix(unsigned cpu, Bit8u prefix) {}
 
 void bx_instr_interrupt(unsigned cpu, unsigned vector) {}
 void bx_instr_exception(unsigned cpu, unsigned vector) {}
@@ -67,9 +57,9 @@ void bx_instr_tlb_cntrl(unsigned cpu, unsigned what, Bit32u newval) {}
 void bx_instr_cache_cntrl(unsigned cpu, unsigned what) {}
 void bx_instr_prefetch_hint(unsigned cpu, unsigned what, unsigned seg, bx_address offset) {}
 
-void bx_instr_before_execution(unsigned cpu) {}
-void bx_instr_after_execution(unsigned cpu) {}
-void bx_instr_repeat_iteration(unsigned cpu) {}
+void bx_instr_before_execution(unsigned cpu, const bxInstruction_c *i) {}
+void bx_instr_after_execution(unsigned cpu, const bxInstruction_c *i) {}
+void bx_instr_repeat_iteration(unsigned cpu, const bxInstruction_c *i) {}
 
 void bx_instr_inp(Bit16u addr, unsigned len) {}
 void bx_instr_outp(Bit16u addr, unsigned len) {}
@@ -84,3 +74,5 @@ void bx_instr_lin_write(unsigned cpu, bx_address lin, bx_address phy, unsigned l
 
 void bx_instr_phy_write(unsigned cpu, bx_address addr, unsigned len) {}
 void bx_instr_phy_read(unsigned cpu, bx_address addr, unsigned len) {}
+
+void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value) {}
