@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.h,v 1.14 2003/10/09 19:05:13 sshwarts Exp $
+// $Id: instrument.h,v 1.17.2.2 2005/07/08 07:17:06 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -40,13 +40,10 @@
 #define BX_INSTR_IS_JMP   13
 #define BX_INSTR_IS_INT   14
 
-#define BX_INSTR_PREFETCH_NTA 00
-#define BX_INSTR_PREFETCH_T0  01
-#define BX_INSTR_PREFETCH_T1  02
-#define BX_INSTR_PREFETCH_T2  03
-
-
-
+#define BX_INSTR_PREFETCH_NTA 0
+#define BX_INSTR_PREFETCH_T0  1
+#define BX_INSTR_PREFETCH_T1  2
+#define BX_INSTR_PREFETCH_T2  3
 
 
 #if BX_INSTRUMENTATION
@@ -58,6 +55,7 @@ class bxInstruction_c;
 void bx_instr_init(unsigned cpu);
 void bx_instr_shutdown(unsigned cpu);
 void bx_instr_reset(unsigned cpu);
+void bx_instr_hlt(unsigned cpu);
 void bx_instr_new_instruction(unsigned cpu);
 
 void bx_instr_debug_promt();
@@ -73,18 +71,7 @@ void bx_instr_far_branch(unsigned cpu, unsigned what, Bit16u new_cs, bx_address
 void bx_instr_opcode(unsigned cpu, Bit8u *opcode, unsigned len, bx_bool is32);
 void bx_instr_fetch_decode_completed(unsigned cpu, const bxInstruction_c *i);
 
-void bx_instr_prefix_as(unsigned cpu);
-void bx_instr_prefix_os(unsigned cpu);
-void bx_instr_prefix_rep(unsigned cpu);
-void bx_instr_prefix_repne(unsigned cpu);
-void bx_instr_prefix_lock(unsigned cpu);
-void bx_instr_prefix_cs(unsigned cpu);
-void bx_instr_prefix_ss(unsigned cpu);
-void bx_instr_prefix_ds(unsigned cpu);
-void bx_instr_prefix_es(unsigned cpu);
-void bx_instr_prefix_fs(unsigned cpu);
-void bx_instr_prefix_gs(unsigned cpu);
-void bx_instr_prefix_extend8b(unsigned cpu);
+void bx_instr_prefix(unsigned cpu, Bit8u prefix);
 
 void bx_instr_interrupt(unsigned cpu, unsigned vector);
 void bx_instr_exception(unsigned cpu, unsigned vector);
@@ -94,9 +81,9 @@ void bx_instr_tlb_cntrl(unsigned cpu, unsigned what, Bit32u newval);
 void bx_instr_cache_cntrl(unsigned cpu, unsigned what);
 void bx_instr_prefetch_hint(unsigned cpu, unsigned what, unsigned seg, bx_address offset);
 
-void bx_instr_before_execution(unsigned cpu);
-void bx_instr_after_execution(unsigned cpu);
-void bx_instr_repeat_iteration(unsigned cpu);
+void bx_instr_before_execution(unsigned cpu, const bxInstruction_c *i);
+void bx_instr_after_execution(unsigned cpu, const bxInstruction_c *i);
+void bx_instr_repeat_iteration(unsigned cpu, const bxInstruction_c *i);
 
 void bx_instr_inp(Bit16u addr, unsigned len);
 void bx_instr_outp(Bit16u addr, unsigned len);
@@ -112,10 +99,13 @@ void bx_instr_lin_write(unsigned cpu, bx_address lin, bx_address phy, unsigned l
 void bx_instr_phy_write(unsigned cpu, bx_address addr, unsigned len);
 void bx_instr_phy_read(unsigned cpu, bx_address addr, unsigned len);
 
+void bx_instr_wrmsr(unsigned cpu, unsigned addr, Bit64u value);
+
 /* simulation init, shutdown, reset */
 #  define BX_INSTR_INIT(cpu_id)            bx_instr_init(cpu_id)
 #  define BX_INSTR_SHUTDOWN(cpu_id)        bx_instr_shutdown(cpu_id)
 #  define BX_INSTR_RESET(cpu_id)           bx_instr_reset(cpu_id)
+#  define BX_INSTR_HLT(cpu_id)             bx_instr_hlt(cpu_id)
 #  define BX_INSTR_NEW_INSTRUCTION(cpu_id) bx_instr_new_instruction(cpu_id)
 
 /* called from command line debugger */
@@ -136,19 +126,8 @@ void bx_instr_phy_read(unsigned cpu, bx_address addr, unsigned len);
 #  define BX_INSTR_FETCH_DECODE_COMPLETED(cpu_id, i) \
                        bx_instr_fetch_decode_completed(cpu_id, i)
      
-/* prefix decoded */
-#  define BX_INSTR_PREFIX_AS(cpu_id)       bx_instr_prefix_as(cpu_id)
-#  define BX_INSTR_PREFIX_OS(cpu_id)       bx_instr_prefix_os(cpu_id)
-#  define BX_INSTR_PREFIX_REP(cpu_id)      bx_instr_prefix_rep(cpu_id)
-#  define BX_INSTR_PREFIX_REPNE(cpu_id)    bx_instr_prefix_repne(cpu_id)
-#  define BX_INSTR_PREFIX_LOCK(cpu_id)     bx_instr_prefix_lock(cpu_id)
-#  define BX_INSTR_PREFIX_CS(cpu_id)       bx_instr_prefix_cs(cpu_id)
-#  define BX_INSTR_PREFIX_SS(cpu_id)       bx_instr_prefix_ss(cpu_id)
-#  define BX_INSTR_PREFIX_DS(cpu_id)       bx_instr_prefix_ds(cpu_id)
-#  define BX_INSTR_PREFIX_ES(cpu_id)       bx_instr_prefix_es(cpu_id)
-#  define BX_INSTR_PREFIX_FS(cpu_id)       bx_instr_prefix_fs(cpu_id)
-#  define BX_INSTR_PREFIX_GS(cpu_id)       bx_instr_prefix_gs(cpu_id)
-#  define BX_INSTR_PREFIX_EXTEND8B(cpu_id) bx_instr_prefix_extend8b(cpu_id)
+/* prefix byte decoded */
+#  define BX_INSTR_PREFIX(cpu_id, prefix)  bx_instr_prefix(cpu_id, prefix)
 
 /* exceptional case and interrupt */
 #  define BX_INSTR_EXCEPTION(cpu_id, vector)            bx_instr_exception(cpu_id, vector)
@@ -162,9 +141,9 @@ void bx_instr_phy_read(unsigned cpu, bx_address addr, unsigned len);
                        bx_instr_prefetch_hint(cpu_id, what, seg, offset)
 
 /* execution */
-#  define BX_INSTR_BEFORE_EXECUTION(cpu_id)             bx_instr_before_execution(cpu_id)
-#  define BX_INSTR_AFTER_EXECUTION(cpu_id)              bx_instr_after_execution(cpu_id)
-#  define BX_INSTR_REPEAT_ITERATION(cpu_id)             bx_instr_repeat_iteration(cpu_id)
+#  define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)          bx_instr_before_execution(cpu_id, i)
+#  define BX_INSTR_AFTER_EXECUTION(cpu_id, i)           bx_instr_after_execution(cpu_id, i)
+#  define BX_INSTR_REPEAT_ITERATION(cpu_id, i)          bx_instr_repeat_iteration(cpu_id, i)
 
 /* memory access */
 #  define BX_INSTR_LIN_READ(cpu_id, lin, phy, len)      bx_instr_lin_read(cpu_id, lin, phy, len)
@@ -183,12 +162,16 @@ void bx_instr_phy_read(unsigned cpu, bx_address addr, unsigned len);
 #  define BX_INSTR_OUTP(addr, len)              bx_instr_outp(addr, len)
 #  define BX_INSTR_OUTP2(addr, len, val)        bx_instr_outp2(addr, len, val)
 
+/* wrmsr callback */
+#  define BX_INSTR_WRMSR(cpu_id, addr, value)           bx_instr_wrmsr(cpu_id, addr, value)
+
 #else   
 
 /* simulation init, shutdown, reset */
 #  define BX_INSTR_INIT(cpu_id)
 #  define BX_INSTR_SHUTDOWN(cpu_id)
 #  define BX_INSTR_RESET(cpu_id)
+#  define BX_INSTR_HLT(cpu_id)
 #  define BX_INSTR_NEW_INSTRUCTION(cpu_id)
 
 /* called from command line debugger */
@@ -207,19 +190,8 @@ void bx_instr_phy_read(unsigned cpu, bx_address addr, unsigned len);
 #  define BX_INSTR_OPCODE(cpu_id, opcode, len, is32) 
 #  define BX_INSTR_FETCH_DECODE_COMPLETED(cpu_id, i)
      
-/* prefix decoded */
-#  define BX_INSTR_PREFIX_AS(cpu_id)
-#  define BX_INSTR_PREFIX_OS(cpu_id)
-#  define BX_INSTR_PREFIX_REP(cpu_id)
-#  define BX_INSTR_PREFIX_REPNE(cpu_id)
-#  define BX_INSTR_PREFIX_LOCK(cpu_id)
-#  define BX_INSTR_PREFIX_CS(cpu_id)
-#  define BX_INSTR_PREFIX_SS(cpu_id)
-#  define BX_INSTR_PREFIX_DS(cpu_id)
-#  define BX_INSTR_PREFIX_ES(cpu_id)
-#  define BX_INSTR_PREFIX_FS(cpu_id)
-#  define BX_INSTR_PREFIX_GS(cpu_id)
-#  define BX_INSTR_PREFIX_EXTEND8B(cpu_id)
+/* prefix byte decoded */
+#  define BX_INSTR_PREFIX(cpu_id, prefix)
 
 /* exceptional case and interrupt */
 #  define BX_INSTR_EXCEPTION(cpu_id, vector)
@@ -232,9 +204,9 @@ void bx_instr_phy_read(unsigned cpu, bx_address addr, unsigned len);
 #  define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
 
 /* execution */
-#  define BX_INSTR_BEFORE_EXECUTION(cpu_id)
-#  define BX_INSTR_AFTER_EXECUTION(cpu_id)
-#  define BX_INSTR_REPEAT_ITERATION(cpu_id)
+#  define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)
+#  define BX_INSTR_AFTER_EXECUTION(cpu_id, i)
+#  define BX_INSTR_REPEAT_ITERATION(cpu_id, i)
 
 /* memory access */
 #  define BX_INSTR_LIN_READ(cpu_id, lin, phy, len)
@@ -253,4 +225,7 @@ void bx_instr_phy_read(unsigned cpu, bx_address addr, unsigned len);
 #  define BX_INSTR_OUTP(addr, len)
 #  define BX_INSTR_OUTP2(addr, len, val)
 
+/* wrmsr callback */
+#  define BX_INSTR_WRMSR(cpu_id, addr, value)
+
 #endif  
