@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cdrom.cc,v 1.75 2004/11/04 22:41:24 sshwarts Exp $
+// $Id: cdrom.cc,v 1.77.2.1 2005/07/07 07:12:08 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -523,7 +523,7 @@ cdrom_interface::cdrom_interface(char *dev)
 
 void
 cdrom_interface::init(void) {
-  BX_DEBUG(("Init $Id: cdrom.cc,v 1.75 2004/11/04 22:41:24 sshwarts Exp $"));
+  BX_DEBUG(("Init $Id: cdrom.cc,v 1.77.2.1 2005/07/07 07:12:08 vruppert Exp $"));
   BX_INFO(("file = '%s'",path));
 }
 
@@ -772,8 +772,9 @@ if (using_file == 0)
 
 
   bx_bool
-cdrom_interface::read_toc(uint8* buf, int* length, bx_bool msf, int start_track, int format)
+cdrom_interface::read_toc(Bit8u* buf, int* length, bx_bool msf, int start_track, int format)
 {
+  unsigned i;
   // Read CD TOC. Returns false if start track is out of bounds.
 
   if (fd < 0) {
@@ -829,9 +830,9 @@ cdrom_interface::read_toc(uint8* buf, int* length, bx_bool msf, int start_track,
         // Start address
         if (msf) {
           buf[len++] = 0; // reserved
-          buf[len++] = (uint8)(((blocks + 150) / 75) / 60); // minute
-          buf[len++] = (uint8)(((blocks + 150) / 75) % 60); // second
-          buf[len++] = (uint8)((blocks + 150) % 75); // frame;
+          buf[len++] = (Bit8u)(((blocks + 150) / 75) / 60); // minute
+          buf[len++] = (Bit8u)(((blocks + 150) / 75) % 60); // second
+          buf[len++] = (Bit8u)((blocks + 150) % 75); // frame;
         } else {
           buf[len++] = (blocks >> 24) & 0xff;
           buf[len++] = (blocks >> 16) & 0xff;
@@ -850,7 +851,7 @@ cdrom_interface::read_toc(uint8* buf, int* length, bx_bool msf, int start_track,
         buf[1] = 0x0a;
         buf[2] = 1;
         buf[3] = 1;
-        for (unsigned i = 0; i < 8; i++)
+        for (i = 0; i < 8; i++)
           buf[4+i] = 0;
         len = 12;
         break;
@@ -887,6 +888,8 @@ cdrom_interface::read_toc(uint8* buf, int* length, bx_bool msf, int start_track,
     *length = iBytesReturned;
 
     return true;
+  } else {
+    return false;
   }
 #elif __linux__ || defined(__sun)
   {
@@ -1090,14 +1093,14 @@ cdrom_interface::read_toc(uint8* buf, int* length, bx_bool msf, int start_track,
   buf[len++] = 0xaa; // Track number
   buf[len++] = 0; // Reserved
 
-  uint32 blocks = capacity();
+  Bit32u blocks = capacity();
 
   // Start address
   if (msf) {
     buf[len++] = 0; // reserved
-    buf[len++] = (uint8)(((blocks + 150) / 75) / 60); // minute
-    buf[len++] = (uint8)(((blocks + 150) / 75) % 60); // second
-    buf[len++] = (uint8)((blocks + 150) % 75); // frame;
+    buf[len++] = (Bit8u)(((blocks + 150) / 75) / 60); // minute
+    buf[len++] = (Bit8u)(((blocks + 150) / 75) % 60); // second
+    buf[len++] = (Bit8u)((blocks + 150) % 75); // frame;
   } else {
     buf[len++] = (blocks >> 24) & 0xff;
     buf[len++] = (blocks >> 16) & 0xff;
@@ -1123,7 +1126,7 @@ cdrom_interface::read_toc(uint8* buf, int* length, bx_bool msf, int start_track,
 }
 
 
-  uint32
+  Bit32u
 cdrom_interface::capacity()
 {
   // Return CD-ROM capacity.  I believe you want to return
@@ -1275,18 +1278,19 @@ cdrom_interface::capacity()
   }
 #elif defined WIN32
   {
-	  if(bUseASPI) {
-		  return (GetCDCapacity(hid, tid, lun) / 2352);
-	  } else if(using_file) {
-	    ULARGE_INTEGER FileSize;
-	    FileSize.LowPart = GetFileSize(hFile, &FileSize.HighPart);
-		return (FileSize.QuadPart / 2048);
-	  } else {  /* direct device access */
-	    DWORD SectorsPerCluster;
-	    DWORD TotalNumOfClusters;
-	    GetDiskFreeSpace( path, &SectorsPerCluster, NULL, NULL, &TotalNumOfClusters);
-		return (TotalNumOfClusters * SectorsPerCluster);
-	  }
+    if(bUseASPI) {
+      return (GetCDCapacity(hid, tid, lun) / 2352);
+    } else if(using_file) {
+      ULARGE_INTEGER FileSize;
+      FileSize.LowPart = GetFileSize(hFile, &FileSize.HighPart);
+      return (Bit32u)(FileSize.QuadPart / 2048);
+    } else {  /* direct device access */
+      ULARGE_INTEGER FreeBytesForCaller;
+      ULARGE_INTEGER TotalNumOfBytes;
+      ULARGE_INTEGER TotalFreeBytes;
+      GetDiskFreeSpaceEx( path, &FreeBytesForCaller, &TotalNumOfBytes, &TotalFreeBytes);
+      return (Bit32u)(TotalNumOfBytes.QuadPart / 2048);
+    }
   }
 #elif defined __APPLE__
 // Find the size of the first data track on the cd.  This has produced
@@ -1357,7 +1361,7 @@ cdrom_interface::capacity()
 }
 
   void BX_CPP_AttrRegparmN(2)
-cdrom_interface::read_block(uint8* buf, int lba)
+cdrom_interface::read_block(Bit8u* buf, int lba)
 {
   // Read a single block from the CD
 
