@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cdrom_amigaos.cc,v 1.5 2003/02/21 14:51:12 cisc Exp $
+// $Id: cdrom_amigaos.cc,v 1.6.4.1 2005/07/07 07:13:34 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2000  MandrakeSoft S.A.
@@ -32,8 +32,9 @@
 // for your OS if it is not supported yet.
 
 
-#include "bochs.h"
+#include "iodev.h"
 #include "scsi_commands.h"
+#include "cdrom.h"
 
 #include <exec/types.h>
 #include <exec/memory.h>
@@ -117,9 +118,9 @@ cdrom_interface::~cdrom_interface(void)
   bx_bool
 cdrom_interface::insert_cdrom(char *dev)
 {
-  uint8 cdb[6];
-  uint8 buf[2*BX_CD_FRAMESIZE];
-  uint8 i = 0;
+  Bit8u cdb[6];
+  Bit8u buf[2*BX_CD_FRAMESIZE];
+  Bit8u i = 0;
   
   memset(cdb,0,sizeof(cdb));
 
@@ -155,7 +156,7 @@ cdrom_interface::insert_cdrom(char *dev)
   void
 cdrom_interface::eject_cdrom()
 {
-  uint8 cdb[6];
+  Bit8u cdb[6];
 
   memset(cdb,0,sizeof(cdb));
 
@@ -168,9 +169,9 @@ cdrom_interface::eject_cdrom()
 
 
   bx_bool
-cdrom_interface::read_toc(uint8* buf, int* length, bx_bool msf, int start_track)
+cdrom_interface::read_toc(Bit8u* buf, int* length, bx_bool msf, int start_track)
 {
-  uint8 cdb[10];
+  Bit8u cdb[10];
   TOC *toc;
   toc = (TOC*) buf;
 
@@ -195,11 +196,11 @@ cdrom_interface::read_toc(uint8* buf, int* length, bx_bool msf, int start_track)
 }
 
 
-  uint32
+  Bit32u
 cdrom_interface::capacity()
 {
   CAPACITY cap;
-  uint8 cdb[10];
+  Bit8u cdb[10];
 
 
   memset(cdb,0,sizeof(cdb));
@@ -218,7 +219,7 @@ cdrom_interface::capacity()
 }
 
   void
-cdrom_interface::read_block(uint8* buf, int lba)
+cdrom_interface::read_block(Bit8u* buf, int lba)
 {
   CDIO->iotd_Req.io_Data    = buf;
   CDIO->iotd_Req.io_Command = CMD_READ;
@@ -231,7 +232,20 @@ cdrom_interface::read_block(uint8* buf, int lba)
   }
 }
 
-int DoSCSI(UBYTE *data, int datasize, uint8 *cmd,int cmdsize, UBYTE direction)
+  int
+cdrom_interface::start_cdrom()
+{
+  // Spin up the cdrom drive.
+
+  if (fd >= 0) {
+    BX_INFO(("start_cdrom: your OS is not supported yet."));
+    return(false); // OS not supported yet, return false always.
+  }
+  return(false);
+}
+
+
+int DoSCSI(UBYTE *data, int datasize, Bit8u *cmd,int cmdsize, UBYTE direction)
 {
   struct SCSICmd scmd;
 
