@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: devices.cc,v 1.76 2004/10/03 20:02:09 vruppert Exp $
+// $Id: devices.cc,v 1.82 2005/02/16 17:53:40 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -55,7 +55,7 @@ bx_devices_c::bx_devices_c(void)
 #if BX_SUPPORT_PCI
   pluginPciBridge = &stubPci;
   pluginPci2IsaBridge = &stubPci2Isa;
-  pluginPciIdeController = NULL;
+  pluginPciIdeController = &stubPciIde;
 #if BX_SUPPORT_PCIVGA
     pluginPciVgaAdapter = NULL;
 #endif
@@ -63,7 +63,7 @@ bx_devices_c::bx_devices_c(void)
     pluginPciDevAdapter = NULL;
 #endif
 #if BX_SUPPORT_PCIUSB
-    pluginPciUSBAdapter = NULL;
+    pluginPciUSBAdapter = &stubUsbAdapter;
 #endif        
 #if BX_SUPPORT_PCIPNIC
     pluginPciPNicAdapter = NULL;
@@ -71,11 +71,14 @@ bx_devices_c::bx_devices_c(void)
 #endif
   pit = NULL;
   pluginKeyboard = &stubKeyboard;
+#if BX_SUPPORT_BUSMOUSE
+  pluginBusMouse = &stubBusMouse;
+#endif
   pluginDmaDevice = &stubDma;
   pluginFloppyDevice = &stubFloppy;
   pluginBiosDevice = NULL;
   pluginCmosDevice = &stubCmos;
-  pluginSerialDevice = NULL;
+  pluginSerialDevice = &stubSerial;
   pluginParallelDevice = NULL;
   pluginUnmapped = NULL;
   pluginVgaDevice = &stubVga;
@@ -106,7 +109,7 @@ bx_devices_c::init(BX_MEM_C *newmem)
 {
   unsigned i;
 
-  BX_DEBUG(("Init $Id: devices.cc,v 1.76 2004/10/03 20:02:09 vruppert Exp $"));
+  BX_DEBUG(("Init $Id: devices.cc,v 1.82 2005/02/16 17:53:40 vruppert Exp $"));
   mem = newmem;
 
   /* set no-default handlers, will be overwritten by the real default handler */
@@ -159,6 +162,11 @@ bx_devices_c::init(BX_MEM_C *newmem)
   PLUG_load_plugin(floppy, PLUGTYPE_CORE);
   PLUG_load_plugin(harddrv, PLUGTYPE_OPTIONAL);
   PLUG_load_plugin(keyboard, PLUGTYPE_OPTIONAL);
+#if BX_SUPPORT_BUSMOUSE
+  if (bx_options.Omouse_type->get () == BX_MOUSE_TYPE_BUS) {
+    PLUG_load_plugin(busmouse, PLUGTYPE_OPTIONAL);
+  }
+#endif
   if (is_serial_enabled ())
     PLUG_load_plugin(serial, PLUGTYPE_OPTIONAL);
   if (is_parallel_enabled ()) 
@@ -179,7 +187,9 @@ bx_devices_c::init(BX_MEM_C *newmem)
     PLUG_load_plugin(pci2isa, PLUGTYPE_CORE);
     PLUG_load_plugin(pci_ide, PLUGTYPE_OPTIONAL);
 #if BX_SUPPORT_PCIVGA
-    PLUG_load_plugin(pcivga, PLUGTYPE_OPTIONAL);
+    if (!strcmp(bx_options.Ovga_extension->getptr (), "vbe")) {
+      PLUG_load_plugin(pcivga, PLUGTYPE_OPTIONAL);
+    }
 #endif
 #if BX_SUPPORT_PCIUSB
     PLUG_load_plugin(pciusb, PLUGTYPE_OPTIONAL);
@@ -198,7 +208,7 @@ bx_devices_c::init(BX_MEM_C *newmem)
   }
 
   // NE2000 NIC
-  if (bx_options.ne2k.Opresent->get ()) {
+  if (bx_options.ne2k.Oenabled->get ()) {
 #if BX_SUPPORT_NE2K
     PLUG_load_plugin(ne2k, PLUGTYPE_OPTIONAL);
 #else
@@ -225,7 +235,7 @@ bx_devices_c::init(BX_MEM_C *newmem)
   pluginFloppyDevice->init();
 
   //--- SOUND ---
-  if (bx_options.sb16.Opresent->get ()) {
+  if (bx_options.sb16.Oenabled->get ()) {
 #if BX_SUPPORT_SB16
     PLUG_load_plugin(sb16, PLUGTYPE_OPTIONAL);
 #else
@@ -276,7 +286,7 @@ bx_devices_c::init(BX_MEM_C *newmem)
 
   // misc. CMOS
   Bit32u extended_memory_in_k = mem->get_memory_in_k() > 1024 ? (mem->get_memory_in_k() - 1024) : 0;
-  if (extended_memory_in_k > 0xffff) extended_memory_in_k = 0xffff;
+  if (extended_memory_in_k > 0xfc00) extended_memory_in_k = 0xfc00;
 
   DEV_cmos_set_reg(0x15, (Bit8u) BASE_MEMORY_IN_K);
   DEV_cmos_set_reg(0x16, (Bit8u) (BASE_MEMORY_IN_K >> 8));
