@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: dma.h,v 1.15 2003/05/03 07:41:27 vruppert Exp $
+// $Id: dma.h,v 1.17 2005/04/06 21:09:25 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -80,8 +80,7 @@ private:
     bx_bool flip_flop;
     Bit8u   status_reg;
     Bit8u   command_reg;
-    Bit8u   request_reg;
-    Bit8u   temporary_reg;
+    bx_bool ctrl_disabled;
     struct {
       struct {
         Bit8u mode_type;
@@ -101,6 +100,8 @@ private:
   bx_bool HLDA;    // Hold Acknowlege
   bx_bool TC;      // Terminal Count
 
+  Bit8u   ext_page_reg[16]; // Extra page registers (unused)
+
   struct {
     void (* dmaRead8)(Bit8u *data_byte);
     void (* dmaWrite8)(Bit8u *data_byte);
