@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: eth_tuntap.cc,v 1.19 2004/10/07 17:38:03 vruppert Exp $
+// $Id: eth_tuntap.cc,v 1.21 2005/05/21 07:38:29 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -84,6 +84,7 @@ private:
   int rx_timer_index;
   static void rx_timer_handler(void *);
   void rx_timer ();
+  Bit8u guest_macaddr[6];
 #if BX_ETH_TUNTAP_LOGGING
   FILE *txlog, *txlog_txt, *rxlog, *rxlog_txt;
 #endif
@@ -192,6 +193,7 @@ bx_tuntap_pktmover_c::bx_tuntap_pktmover_c(const char *netif,
 				1, 1, "eth_tuntap"); // continuous, active
   this->rxh   = rxh;
   this->rxarg = rxarg;
+  memcpy(&guest_macaddr[0], macaddr, 6);
 #if BX_ETH_TUNTAP_LOGGING
   // eventually Bryce wants txlog to dump in pcap format so that
   // tcpdump -r FILE can read it and interpret packets.
@@ -306,8 +308,8 @@ void bx_tuntap_pktmover_c::rx_timer ()
   // hack: TUN/TAP device likes to create an ethernet header which has
   // the same source and destination address FE:FD:00:00:00:00.
   // Change the dest address to FE:FD:00:00:00:01.
-  if (memcmp(&rxbuf[0], broadcast_macaddr, 6)) {
-    rxbuf[5] = 1;
+  if (!memcmp(&rxbuf[0], &rxbuf[6], 6)) {
+    rxbuf[5] = guest_macaddr[5];
   }
 
 #ifdef __APPLE__	//FIXME:hack
@@ -345,7 +347,7 @@ void bx_tuntap_pktmover_c::rx_timer ()
     fflush (rxlog_txt);
   }
 #endif
-  BX_DEBUG(("eth_tuntap: got packet: %d bytes, dst=%x:%x:%x:%x:%x:%x, src=%x:%x:%x:%x:%x:%x\n", nbytes, rxbuf[0], rxbuf[1], rxbuf[2], rxbuf[3], rxbuf[4], rxbuf[5], rxbuf[6], rxbuf[7], rxbuf[8], rxbuf[9], rxbuf[10], rxbuf[11]));
+  BX_DEBUG(("eth_tuntap: got packet: %d bytes, dst=%02x:%02x:%02x:%02x:%02x:%02x, src=%02x:%02x:%02x:%02x:%02x:%02x", nbytes, rxbuf[0], rxbuf[1], rxbuf[2], rxbuf[3], rxbuf[4], rxbuf[5], rxbuf[6], rxbuf[7], rxbuf[8], rxbuf[9], rxbuf[10], rxbuf[11]));
   if (nbytes < 60) {
     BX_INFO (("packet too short (%d), padding to 60", nbytes));
     nbytes = 60;
@@ -370,12 +372,13 @@ int tun_alloc(char *dev)
    *        IFF_NO_PI - Do not provide packet information  
    */ 
   ifr.ifr_flags = IFF_TAP | IFF_NO_PI; 
-  if (*dev)
-    strncpy(ifr.ifr_name, dev, IFNAMSIZ);
+  ifr.ifr_name[0]=0;
   if ((err = ioctl(fd, TUNSETIFF, (void *) &ifr)) < 0) {
     close(fd);
     return err;
   }
+  strncpy(dev, ifr.ifr_name, IFNAMSIZ);
+  dev[IFNAMSIZ-1]=0;
 
   ioctl( fd, TUNSETNOCSUM, 1 );
 #endif
