@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: eth_vnet.cc,v 1.12 2004/10/24 12:49:04 vruppert Exp $
+// $Id: eth_vnet.cc,v 1.15.2.1 2005/07/07 07:06:22 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 // virtual Ethernet locator
 //
-// An implementation of ARP, ping(ICMP-echo) and DHCP.
+// An implementation of ARP, ping(ICMP-echo), DHCP and read/write TFTP.
 // Virtual host acts as a DHCP server for guest.
 // There are no connections between the virtual host and real ethernets.
 //
@@ -13,7 +13,7 @@
 // Guest IP: 192.168.10.2
 // Guest netmask: 255.255.255.0
 // Guest broadcast: 192.168.10.255
-//
+// TFTP server uses ethdev value for the root directory and doesn't overwrite files
 
 #define BX_PLUGGABLE
 
@@ -67,8 +67,21 @@ typedef void (*layer4_handler_t)(
 #define INET_PORT_HTTP 80
 #define INET_PORT_NTP 123
 
+// TFTP server support by EaseWay <easeway@123.com>
+
+#define INET_PORT_TFTP_SERVER 69
+
+#define TFTP_RRQ    1
+#define TFTP_WRQ    2
+#define TFTP_DATA   3
+#define TFTP_ACK    4
+#define TFTP_ERROR  5
+
+#define TFTP_BUFFER_SIZE 512
+
 #define BOOTREQUEST 1
 #define BOOTREPLY 2
+
 #define BOOTPOPT_PADDING 0
 #define BOOTPOPT_END 255
 #define BOOTPOPT_SUBNETMASK 1
@@ -98,12 +111,13 @@ typedef void (*layer4_handler_t)(
 #define BOOTPOPT_CLIENT_IDENTIFIER 61
 
 #define DHCPDISCOVER 1
-#define DHCPOFFER 2
-#define DHCPREQUEST 3
-#define DHCPDECLINE 4
-#define DHCPACK 5
-#define DHCPNAK 6
-#define DHCPRELEASE 7
+#define DHCPOFFER    2
+#define DHCPREQUEST  3
+#define DHCPDECLINE  4
+#define DHCPACK      5
+#define DHCPNAK      6
+#define DHCPRELEASE  7
+#define DHCPINFORM   8
 
 class bx_vnet_pktmover_c : public eth_pktmover_c {
 public:
@@ -153,6 +167,32 @@ private:
     const Bit8u *ipheader, unsigned ipheader_len,
     unsigned sourceport, unsigned targetport,
     const Bit8u *data, unsigned data_len);
+  static void udpipv4_tftp_handler(
+    void *this_ptr,
+    const Bit8u *ipheader, unsigned ipheader_len,
+    unsigned sourceport, unsigned targetport,
+    const Bit8u *data, unsigned data_len);
+  void udpipv4_tftp_handler_ns(
+    const Bit8u *ipheader, unsigned ipheader_len,
+    unsigned sourceport, unsigned targetport,
+    const Bit8u *data, unsigned data_len);
+  void tftp_send_error(
+    Bit8u *buffer,
+    unsigned sourceport, unsigned targetport,
+    unsigned code, const char *msg);
+  void tftp_send_data(
+    Bit8u *buffer,
+    unsigned sourceport, unsigned targetport,
+    unsigned block_nr);  
+  void tftp_send_ack(
+    Bit8u *buffer,
+    unsigned sourceport, unsigned targetport,
+    unsigned block_nr);
+    
+  char tftp_filename[BX_PATHNAME_LEN];
+  char tftp_rootdir[BX_PATHNAME_LEN];
+  bx_bool tftp_write;
+  Bit16u tftp_tid;
 
   Bit8u host_macaddr[6];
   Bit8u guest_macaddr[6];
@@ -251,6 +291,9 @@ bx_vnet_pktmover_c::pktmover_init(
   BX_INFO(("ne2k vnet driver"));
   this->rxh   = rxh;
   this->rxarg = rxarg;
+  strcpy(this->tftp_rootdir, netif);
+  this->tftp_tid = 0;
+  this->tftp_write = 0;
 
   memcpy(&host_macaddr[0], macaddr, 6);
   memcpy(&guest_macaddr[0], macaddr, 6);
@@ -262,6 +305,7 @@ bx_vnet_pktmover_c::pktmover_init(
   l4data_used = 0;
 
   register_layer4_handler(0x11,INET_PORT_BOOTP_SERVER,udpipv4_dhcp_handler);
+  register_layer4_handler(0x11,INET_PORT_TFTP_SERVER,udpipv4_tftp_handler);
 
   this->rx_timer_index = 
     bx_pc_system.register_timer(this, this->rx_timer_handler, 1000,
@@ -271,12 +315,13 @@ bx_vnet_pktmover_c::pktmover_init(
   pktlog_txt = fopen ("ne2k-pktlog.txt", "wb");
   if (!pktlog_txt) BX_PANIC (("ne2k-pktlog.txt failed"));
   fprintf (pktlog_txt, "vnet packetmover readable log file\n");
-  fprintf (pktlog_txt, "net IF = %s\n", netif);
+  fprintf (pktlog_txt, "TFTP root = %s\n", netif);
   fprintf (pktlog_txt, "host MAC address = ");
-  for (int i=0; i<6; i++) 
+  int i;
+  for (i=0; i<6; i++) 
     fprintf (pktlog_txt, "%02x%s", 0xff & host_macaddr[i], i<5?":" : "\n");
   fprintf (pktlog_txt, "guest MAC address = ");
-  for (int i=0; i<6; i++) 
+  for (i=0; i<6; i++) 
     fprintf (pktlog_txt, "%02x%s", 0xff & guest_macaddr[i], i<5?":" : "\n");
   fprintf (pktlog_txt, "--\n");
   fflush (pktlog_txt);
@@ -414,13 +459,13 @@ bx_vnet_pktmover_c::process_arp(const Bit8u *buf, unsigned io_len)
         }
         break;
       case 0x0002: // ARP REPLY
-        BX_INFO(("unexpected ARP REPLY\n"));
+        BX_INFO(("unexpected ARP REPLY"));
         break;
       case 0x0003: // RARP REQUEST
-        BX_ERROR(("RARP is not implemented\n"));
+        BX_ERROR(("RARP is not implemented"));
         break;
       case 0x0004: // RARP REPLY
-        BX_INFO(("unexpected RARP REPLY\n"));
+        BX_INFO(("unexpected RARP REPLY"));
         break;
       default:
         BX_INFO(("arp: unknown ARP opcode %04x",opcode));
@@ -465,11 +510,11 @@ bx_vnet_pktmover_c::process_ipv4(const Bit8u *buf, unsigned io_len)
   unsigned l4pkt_len;
 
   if (io_len < (14U+20U)) {
-    BX_INFO(("ip packet - too small packet\n"));
+    BX_INFO(("ip packet - too small packet"));
     return;
   }
   if ((buf[14+0] & 0xf0) != 0x40) {
-    BX_INFO(("ipv%u packet - not implemented\n",((unsigned)buf[14+0] >> 4)));
+    BX_INFO(("ipv%u packet - not implemented",((unsigned)buf[14+0] >> 4)));
     return;
   }
   l3header_len = ((unsigned)(buf[14+0] & 0x0f) << 2);
@@ -484,7 +529,9 @@ bx_vnet_pktmover_c::process_ipv4(const Bit8u *buf, unsigned io_len)
   }
 
   total_len = get_net2(&buf[14+2]);
-  if (io_len > (14U+total_len)) return;
+  // FIXED By EaseWay
+  // Ignore this check to tolerant some cases
+  //if (io_len > (14U+total_len)) return;
 
   if (memcmp(&buf[14+16],host_ipv4addr,4) &&
       memcmp(&buf[14+16],broadcast_ipv4addr[0],4) &&
@@ -503,7 +550,7 @@ bx_vnet_pktmover_c::process_ipv4(const Bit8u *buf, unsigned io_len)
   ipproto = buf[14+9];
 
   if ((fragment_flags & 0x1) || (fragment_offset != 0)) {
-    BX_INFO(("ignore fragmented packet!\n"));
+    BX_INFO(("ignore fragmented packet!"));
     return;
   } else {
     l4pkt = &buf[14 + l3header_len];
@@ -521,7 +568,7 @@ bx_vnet_pktmover_c::process_ipv4(const Bit8u *buf, unsigned io_len)
     process_udpipv4(&buf[14],l3header_len,l4pkt,l4pkt_len);
     break;
   default:
-    BX_INFO(("unknown IP protocol %02x\n",ipproto));
+    BX_INFO(("unknown IP protocol %02x",ipproto));
     break;
   }
 }
@@ -1055,6 +1102,201 @@ bx_vnet_pktmover_c::udpipv4_dhcp_handler_ns(
     replybuf, opts_len);
 }
 
+void 
+bx_vnet_pktmover_c::udpipv4_tftp_handler(
+  void *this_ptr,
+  const Bit8u *ipheader, unsigned ipheader_len,
+  unsigned sourceport, unsigned targetport,
+  const Bit8u *data, unsigned data_len)
+{
+  ((bx_vnet_pktmover_c *)this_ptr)->udpipv4_tftp_handler_ns(
+    ipheader,ipheader_len,sourceport,targetport,data,data_len);
+}
+
+void 
+bx_vnet_pktmover_c::udpipv4_tftp_handler_ns(
+  const Bit8u *ipheader, unsigned ipheader_len,
+  unsigned sourceport, unsigned targetport,
+  const Bit8u *data, unsigned data_len)
+{
+  Bit8u buffer[TFTP_BUFFER_SIZE + 4];
+  char path[BX_PATHNAME_LEN];
+  FILE *fp;
+  unsigned block_nr;
+  unsigned tftp_len;
+
+  switch (get_net2(data)) {
+    case TFTP_RRQ:
+      if (tftp_tid == 0) {
+        strncpy((char*)buffer, (const char*)data + 2, data_len - 2);
+        buffer[data_len - 4] = 0;
+
+        // transfer mode
+        if (strlen((char*)buffer) < data_len - 2) {
+          const char *mode = (const char*)data + 2 + strlen((char*)buffer) + 1;
+          if (memcmp(mode, "octet\0", 6) != 0) {
+            tftp_send_error(buffer, sourceport, targetport, 4, "Unsupported transfer mode");
+            return;
+          }
+        }
+
+        strcpy(tftp_filename, (char*)buffer);
+        tftp_tid = sourceport;
+        tftp_write = 0;
+        tftp_send_data(buffer, sourceport, targetport, 1);
+      } else {
+        tftp_send_error(buffer, sourceport, targetport, 4, "Illegal request");
+      }
+      break;
+    case TFTP_WRQ:
+      if (tftp_tid == 0) {
+        strncpy((char*)buffer, (const char*)data + 2, data_len - 2);
+        buffer[data_len - 4] = 0;
+
+        // transfer mode
+        if (strlen((char*)buffer) < data_len - 2) {
+          const char *mode = (const char*)data + 2 + strlen((char*)buffer) + 1;
+          if (memcmp(mode, "octet\0", 6) != 0) {
+            tftp_send_error(buffer, sourceport, targetport, 4, "Unsupported transfer mode");
+            return;
+          }
+        }
+
+        strcpy(tftp_filename, (char*)buffer);
+        sprintf(path, "%s/%s", tftp_rootdir, tftp_filename);
+        fp = fopen(path, "rb");
+        if (fp) {
+          tftp_send_error(buffer, sourceport, targetport, 6, "File exists");
+          fclose(fp);
+          return;  	
+        }
+        fp = fopen(path, "wb");
+        if (!fp) {
+          tftp_send_error(buffer, sourceport, targetport, 2, "Access violation");
+          return;  	
+        }
+        fclose(fp);
+        tftp_tid = sourceport;
+        tftp_write = 1;
+
+        tftp_send_ack(buffer, sourceport, targetport, 0);
+      } else {
+        tftp_send_error(buffer, sourceport, targetport, 4, "Illegal request");
+      }
+      break;
+    case TFTP_DATA:
+      if ((tftp_tid == sourceport) && (tftp_write == 1)) {
+        block_nr = get_net2(data + 2);
+        strncpy((char*)buffer, (const char*)data + 4, data_len - 4);
+        tftp_len = data_len - 4;
+        buffer[tftp_len] = 0;
+        if (tftp_len <= 512) {
+          sprintf(path, "%s/%s", tftp_rootdir, tftp_filename);
+          fp = fopen(path, "ab");
+          if (!fp) {
+            tftp_send_error(buffer, sourceport, targetport, 2, "Access violation");
+            return;  	
+          }
+          if (fseek(fp, (block_nr - 1) * TFTP_BUFFER_SIZE, SEEK_SET) < 0) {
+            tftp_send_error(buffer, sourceport, targetport, 3, "Block not seekable");
+            return;  	
+          }
+          fwrite(buffer, 1, tftp_len, fp);
+          fclose(fp);
+          tftp_send_ack(buffer, sourceport, targetport, block_nr);
+          if (tftp_len < 512) {
+            tftp_tid = 0;
+          }
+        } else {
+          tftp_send_error(buffer, sourceport, targetport, 4, "Illegal request");
+        }
+      } else {
+        tftp_send_error(buffer, sourceport, targetport, 4, "Illegal request");
+      }
+      break;
+    case TFTP_ACK:
+      tftp_send_data(buffer, sourceport, targetport, get_net2(data + 2) + 1);
+      break;
+    case TFTP_ERROR:
+      // silently ignore error packets
+      break;
+    default:
+      BX_ERROR(("TFTP unknown opt %d", get_net2(data)));
+  } 
+}
+
+void 
+bx_vnet_pktmover_c::tftp_send_error(
+  Bit8u *buffer,
+  unsigned sourceport, unsigned targetport,
+  unsigned code, const char *msg)
+{
+  put_net2(buffer, TFTP_ERROR);
+  put_net2(buffer + 2, code);
+  strcpy((char*)buffer + 4, msg);
+  host_to_guest_udpipv4_packet(sourceport, targetport, buffer, strlen(msg) + 5);  
+  tftp_tid = 0;
+}
+
+void 
+bx_vnet_pktmover_c::tftp_send_data(
+  Bit8u *buffer,
+  unsigned sourceport, unsigned targetport,
+  unsigned block_nr)
+{
+  char path[BX_PATHNAME_LEN];
+  char msg[BX_PATHNAME_LEN];
+  int rd;
+
+  if (strlen(tftp_filename) == 0) {
+    tftp_send_error(buffer, sourceport, targetport, 1, "File not found");
+    return;
+  }
+
+  if ((strlen(tftp_rootdir) + strlen(tftp_filename)) > BX_PATHNAME_LEN) {
+    tftp_send_error(buffer, sourceport, targetport, 1, "Path name too long");
+    return;
+  }
+
+  sprintf(path, "%s/%s", tftp_rootdir, tftp_filename);
+  FILE *fp = fopen(path, "rb");
+  if (!fp) {
+    sprintf(msg, "File not found: %s", tftp_filename);
+    tftp_send_error(buffer, sourceport, targetport, 1, msg);
+    return;  	
+  }
+  
+  if (fseek(fp, (block_nr - 1) * TFTP_BUFFER_SIZE, SEEK_SET) < 0) {
+    tftp_send_error(buffer, sourceport, targetport, 3, "Block not seekable");
+    return;  	
+  }
+  
+  rd = fread(buffer + 4, 1, TFTP_BUFFER_SIZE, fp);
+  fclose(fp);
+  
+  if (rd < 0) {
+    tftp_send_error(buffer, sourceport, targetport, 3, "Block not readable");
+    return;  	  	
+  }
+  
+  put_net2(buffer, TFTP_DATA);
+  put_net2(buffer + 2, block_nr);
+  host_to_guest_udpipv4_packet(sourceport, targetport, buffer, rd + 4);
+  if (rd < TFTP_BUFFER_SIZE) {
+    tftp_tid = 0;
+  }
+}
+
+void 
+bx_vnet_pktmover_c::tftp_send_ack(
+  Bit8u *buffer,
+  unsigned sourceport, unsigned targetport,
+  unsigned block_nr)
+{
+  put_net2(buffer, TFTP_ACK);
+  put_net2(buffer + 2, block_nr);
+  host_to_guest_udpipv4_packet(sourceport, targetport, buffer, 4);
+}
 
 
 #endif /* if BX_NETWORKING */
