@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: harddrv.h,v 1.22.2.1 2004/02/06 22:14:36 danielg4 Exp $
+// $Id: harddrv.h,v 1.29.2.1 2005/07/06 21:02:21 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -32,12 +32,12 @@
 
  typedef struct
  {
-   uint32  magic;
-   uint32  version;
-   uint32  pagesize;
-   uint32  numpages;
+   Bit32u  magic;
+   Bit32u  version;
+   Bit32u  pagesize;
+   Bit32u  numpages;
 
-   uint32  padding[60];
+   Bit32u  padding[60];
  } sparse_header_t;
 
 #define STANDARD_HEADER_MAGIC     "Bochs Virtual HD Image"
@@ -257,7 +257,7 @@ class sparse_image_t : public device_image_t
  size_t  mmap_length;
  size_t  system_pagesize_mask;
 #endif
- uint32 *  pagetable;
+ Bit32u *  pagetable;
 
  // Header is written to disk in little-endian (x86) format
  // Thus needs to be converted on big-endian systems before read
@@ -265,9 +265,9 @@ class sparse_image_t : public device_image_t
 
  sparse_header_t header;
 
- uint32  pagesize;
+ Bit32u  pagesize;
  int     pagesize_shift;
- uint32  pagesize_mask;
+ Bit32u  pagesize_mask;
 
  off_t   data_start;
  off_t   underlying_filesize;
@@ -276,9 +276,9 @@ class sparse_image_t : public device_image_t
 
  off_t position;
 
- uint32 position_virtual_page;
- uint32 position_physical_page;
- uint32 position_page_offset;
+ Bit32u position_virtual_page;
+ Bit32u position_physical_page;
+ Bit32u position_page_offset;
 
  off_t underlying_current_filepos;
 
@@ -294,9 +294,9 @@ class sparse_image_t : public device_image_t
 #ifndef PARANOID
        sparse_image_t::
 #endif
-                       set_virtual_page(uint32 new_virtual_page);
+                       set_virtual_page(Bit32u new_virtual_page);
  void read_header();
- ssize_t read_page_fragment(uint32 read_virtual_page, uint32 read_page_offset, size_t read_size, void * buf);
+ ssize_t read_page_fragment(Bit32u read_virtual_page, Bit32u read_page_offset, size_t read_size, void * buf);
 
  sparse_image_t *  parent_image;
 };
@@ -635,13 +635,8 @@ struct error_recovery_t {
   error_recovery_t ();
 };
 
-uint16 read_16bit(const uint8* buf) BX_CPP_AttrRegparmN(1);
-uint32 read_32bit(const uint8* buf) BX_CPP_AttrRegparmN(1);
-
-
-#ifdef LOWLEVEL_CDROM
-#  include "cdrom.h"
-#endif
+Bit16u read_16bit(const Bit8u* buf) BX_CPP_AttrRegparmN(1);
+Bit32u read_32bit(const Bit8u* buf) BX_CPP_AttrRegparmN(1);
 
 
 struct cdrom_t
@@ -651,7 +646,7 @@ struct cdrom_t
 #ifdef LOWLEVEL_CDROM
   LOWLEVEL_CDROM* cd;
 #endif
-  uint32 capacity;
+  Bit32u capacity;
   int next_lba;
   int remaining_blocks;
   struct currentStruct {
@@ -661,7 +656,7 @@ struct cdrom_t
 
 struct atapi_t
 {
-  uint8 command;
+  Bit8u command;
   int drq_bytes;
   int total_bytes_remaining;
 };
@@ -690,6 +685,11 @@ public:
   virtual Bit32u   get_first_cd_handle(void);
   virtual unsigned get_cd_media_status(Bit32u handle);
   virtual unsigned set_cd_media_status(Bit32u handle, unsigned status);
+#if BX_SUPPORT_PCI
+  virtual bx_bool  bmdma_read_sector(Bit8u channel, Bit8u *buffer);
+  virtual bx_bool  bmdma_write_sector(Bit8u channel, Bit8u *buffer);
+  virtual void     bmdma_complete(Bit8u channel);
+#endif
 
   virtual Bit32u virt_read_handler(Bit32u address, unsigned io_len) {
     return read_handler (this, address, io_len);
@@ -707,6 +707,9 @@ public:
   static Bit32u read_handler(void *this_ptr, Bit32u address, unsigned io_len);
   static void   write_handler(void *this_ptr, Bit32u address, Bit32u value, unsigned io_len);
 
+  static void iolight_timer_handler(void *);
+  BX_HD_SMF void iolight_timer(void);
+
 private:
 
   BX_HD_SMF bx_bool calculate_logical_address(Bit8u channel, off_t *sector) BX_CPP_AttrRegparmN(2);
@@ -718,9 +721,10 @@ private:
   BX_HD_SMF void init_send_atapi_command(Bit8u channel, Bit8u command, int req_length, int alloc_length, bool lazy = false) BX_CPP_AttrRegparmN(3);
   BX_HD_SMF void ready_to_send_atapi(Bit8u channel) BX_CPP_AttrRegparmN(1);
   BX_HD_SMF void raise_interrupt(Bit8u channel) BX_CPP_AttrRegparmN(1);
-  BX_HD_SMF void atapi_cmd_error(Bit8u channel, sense_t sense_key, asc_t asc);
+  BX_HD_SMF void atapi_cmd_error(Bit8u channel, sense_t sense_key, asc_t asc, bx_bool show);
   BX_HD_SMF void init_mode_sense_single(Bit8u channel, const void* src, int size);
   BX_HD_SMF void atapi_cmd_nop(Bit8u channel) BX_CPP_AttrRegparmN(1);
+  BX_HD_SMF bx_bool bmdma_present(void);
 
   // FIXME:
   // For each ATA channel we should have one controller struct
@@ -741,6 +745,8 @@ private:
       atapi_t atapi;
 
       Bit8u model_no[41];
+      int statusbar_id;
+      int iolight_counter;
       } drives[2];
     unsigned drive_select;
 
@@ -750,15 +756,7 @@ private:
 
     } channels[BX_MAX_ATA_CHANNEL];
 
-#if BX_PDC20230C_VLBIDE_SUPPORT
-// pdc20630c is only available for 1st ata channel
-  struct pdc20630c_t {
-    bx_bool prog_mode;
-    Bit8u   prog_count;
-    Bit32u  p1f3_value;
-    Bit32u  p1f4_value;
-    } pdc20230c;
-#endif
+  int iolight_timer_index;
 
   };
 #endif // INCLUDE_ONLY_SPARSE_HEADER
