@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ioapic.cc,v 1.14 2004/09/15 21:48:57 sshwarts Exp $
+// $Id: ioapic.cc,v 1.18.2.1 2005/07/06 20:58:53 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 #include <stdio.h>
@@ -12,16 +12,16 @@ class bx_ioapic_c bx_ioapic;
 
 void bx_io_redirect_entry_t::parse_value ()
 {
-  dest = (value >> 56) & APIC_ID_MASK;
-  masked = (value >> 16) & 1;
-  trig_mode = (value >> 15) & 1;
-  remote_irr = (value >> 14) & 1;
-  polarity = (value >> 13) & 1;
+  dest = (Bit8u)((value >> 56) & APIC_ID_MASK);
+  masked = (Bit8u)((value >> 16) & 1);
+  trig_mode = (Bit8u)((value >> 15) & 1);
+  remote_irr = (Bit8u)((value >> 14) & 1);
+  polarity = (Bit8u)((value >> 13) & 1);
   //delivery_status = (value >> 12) & 1;
   delivery_status = 0;  // we'll change this later...
-  dest_mode = (value >> 11) & 1;
-  delivery_mode = (value >> 8) & 7;
-  vector = value & 0xff;
+  dest_mode = (Bit8u)((value >> 11) & 1);
+  delivery_mode = (Bit8u)((value >> 8) & 7);
+  vector = (Bit8u)(value & 0xff);
 }
 
 void bx_io_redirect_entry_t::sprintf_self (char *buf)
@@ -41,7 +41,7 @@ bx_ioapic_c::~bx_ioapic_c () {}
 void bx_ioapic_c::init () 
 {
   bx_generic_apic_c::init ();
-  BX_DEBUG(("initializing I/O APIC"));
+  BX_INFO(("initializing I/O APIC"));
   base_addr = 0xfec00000;
   set_id(BX_IOAPIC_DEFAULT_ID);
   ioregsel = 0;
@@ -165,7 +165,6 @@ void bx_ioapic_c::service_ioapic ()
       if (!entry->masked) {
 	// clear irr bit and deliver
 	bx_bool done = deliver (entry->dest, entry->dest_mode, entry->delivery_mode, entry->vector, entry->polarity, entry->trig_mode);
-	if (done) irr &= ~(1<<bit);
 	if (done) {
 	  irr &= ~(1<<bit);
 	  entry->delivery_status = 0;
