@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: iodev.h,v 1.37 2003/08/04 16:03:09 akrisak Exp $
+// $Id: iodev.h,v 1.61 2005/04/02 11:30:08 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -10,6 +10,8 @@
 //    http://www.linux-mandrake.com/
 //    http://www.mandrakesoft.com/
 //
+//  I/O port handlers API Copyright (C) 2003 by Frank Cornelis
+//
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
 //  License as published by the Free Software Foundation; either
@@ -24,7 +26,10 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
+#ifndef IODEV_H
+#define IODEV_H
 
+#include "bochs.h"
 
 /* maximum number of emulated devices allowed.  floppy, vga, etc...
    you can increase this to anything below 256 since an 8-bit handle
@@ -43,9 +48,12 @@
 
 class bx_pit_c;
 class bx_keyb_c;
+#if BX_SUPPORT_BUSMOUSE
+class bx_busm_c;
+#endif
 class bx_ioapic_c;
 class bx_g2h_c;
-#if BX_IODEBUG_SUPPORT
+#if BX_SUPPORT_IODEBUG
 class bx_iodebug_c;
 #endif
 
@@ -92,7 +100,7 @@ class BOCHSAPI bx_keyb_stub_c : public bx_devmodel_c {
   public:
   virtual ~bx_keyb_stub_c () {}
   // stubs for bx_keyb_c methods
-  virtual void mouse_motion(int delta_x, int delta_y, unsigned button_state) {
+  virtual void mouse_motion(int delta_x, int delta_y, int delta_z, unsigned button_state) {
     STUBFUNC(keyboard, mouse_motion);
   }
   virtual void gen_scancode(Bit32u key) {
@@ -141,6 +149,15 @@ class BOCHSAPI bx_hard_drive_stub_c : public bx_devmodel_c {
   {
     STUBFUNC(HD, virt_write_handler);
   }
+  virtual bx_bool bmdma_read_sector(Bit8u channel, Bit8u *buffer) {
+    STUBFUNC(HD, bmdma_read_sector); return 0;
+  }
+  virtual bx_bool bmdma_write_sector(Bit8u channel, Bit8u *buffer) {
+    STUBFUNC(HD, bmdma_write_sector); return 0;
+  }
+  virtual void bmdma_complete(Bit8u channel) {
+    STUBFUNC(HD, bmdma_complete);
+  }
 };
 
 class BOCHSAPI bx_floppy_stub_c : public bx_devmodel_c {
@@ -210,6 +227,9 @@ class BOCHSAPI bx_pic_stub_c : public bx_devmodel_c {
   virtual void lower_irq(unsigned irq_no) {
     STUBFUNC(pic, lower_irq); 
   }
+  virtual void set_mode(bx_bool ma_sl, Bit8u mode) {
+    STUBFUNC(pic, set_mode); 
+  }
   virtual Bit8u IAC(void) {
     STUBFUNC(pic, IAC); return 0;
   }
@@ -243,6 +263,7 @@ class BOCHSAPI bx_vga_stub_c : public bx_devmodel_c {
   virtual Bit8u get_actl_palette_idx(Bit8u index) {
     return 0;
   }
+  virtual void dump_status(void) {}
 };
 
 class BOCHSAPI bx_pci_stub_c : public bx_devmodel_c {
@@ -250,9 +271,24 @@ class BOCHSAPI bx_pci_stub_c : public bx_devmodel_c {
   virtual bx_bool register_pci_handlers(void *this_ptr,
                                         Bit32u (*bx_pci_read_handler)(void *, Bit8u, unsigned),
                                         void(*bx_pci_write_handler)(void *, Bit8u, Bit32u, unsigned),
-                                        Bit8u devfunc, const char *name) {
+                                        Bit8u *devfunc, const char *name,
+                                        const char *descr) {
     STUBFUNC(pci, register_pci_handlers); return 0;
   }
+  virtual bx_bool is_pci_device (const char *name) {
+    return 0;
+  }
+  virtual void pci_set_base_mem(void *this_ptr, memory_handler_t f1, memory_handler_t f2,
+                                Bit32u *addr, Bit8u *pci_conf, unsigned size) {
+    STUBFUNC(pci, pci_set_base_mem);
+  }
+
+  virtual void pci_set_base_io(void *this_ptr, bx_read_handler_t f1, bx_write_handler_t f2,
+                              Bit32u *addr, Bit8u *pci_conf, unsigned size,
+                              const Bit8u *iomask, const char *name) {
+    STUBFUNC(pci, pci_set_base_io);
+  }
+
   virtual Bit8u rd_memType (Bit32u addr) {
     return 0;
   }
@@ -262,11 +298,69 @@ class BOCHSAPI bx_pci_stub_c : public bx_devmodel_c {
   virtual void print_i440fx_state(void) {}
 };
 
+class BOCHSAPI bx_pci2isa_stub_c : public bx_devmodel_c {
+  public:
+  virtual void pci_set_irq (Bit8u devfunc, unsigned line, bx_bool level) {
+    STUBFUNC(pci2isa, pci_set_irq);
+  }
+};
+
+class BOCHSAPI bx_pci_ide_stub_c : public bx_devmodel_c {
+  public:
+  virtual bx_bool bmdma_present (void) {
+    return 0;
+  }
+};
+
 class BOCHSAPI bx_ne2k_stub_c : public bx_devmodel_c {
   public:
   virtual void print_info(FILE *file, int page, int reg, int nodups) {}
 };
 
+class BOCHSAPI bx_speaker_stub_c : public bx_devmodel_c {
+  public:
+  virtual void beep_on(float frequency) {}
+  virtual void beep_off() {}
+};
+
+class BOCHSAPI bx_serial_stub_c : public bx_devmodel_c {
+  public:
+  virtual void serial_mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state) {
+    STUBFUNC(serial, serial_mouse_enq);
+  }
+};
+
+#if BX_SUPPORT_PCIUSB
+class BOCHSAPI bx_usb_stub_c : public bx_devmodel_c {
+  public:
+  virtual void usb_mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state) {
+    STUBFUNC(pciusb, usb_mouse_enq);
+  }
+  virtual void usb_mouse_enable(bx_bool enable) {
+    STUBFUNC(pciusb, usb_mouse_enable);
+  }
+  virtual bx_bool usb_key_enq(Bit8u *scan_code) {
+    STUBFUNC(pciusb, usb_key_enq);
+    return 0;
+  }
+  virtual bx_bool usb_keyboard_connected() {
+    return 0;
+  }
+  virtual bx_bool usb_mouse_connected() {
+    return 0;
+  }
+};
+#endif
+
+#if BX_SUPPORT_BUSMOUSE
+class BOCHSAPI bx_busm_stub_c : public bx_devmodel_c {
+  public:
+  virtual void bus_mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state) {
+    STUBFUNC(busmouse, bus_mouse_enq);
+  }
+};
+#endif
+
 class BOCHSAPI bx_devices_c : public logfunctions {
 public:
   bx_devices_c(void);
@@ -280,8 +374,23 @@ public:
   // power-on, hardware, or software.
   void reset(unsigned type);
   BX_MEM_C *mem;  // address space associated with these devices
-  bx_bool register_io_read_handler(void *this_ptr, bx_read_handler_t f, Bit32u addr, const char *name, Bit8u mask );
+  bx_bool register_io_read_handler(void *this_ptr, bx_read_handler_t f, 
+		  Bit32u addr, const char *name, Bit8u mask );
+  bx_bool unregister_io_read_handler( void *this_ptr, bx_read_handler_t f,
+                                        Bit32u addr, Bit8u mask );
   bx_bool register_io_write_handler(void *this_ptr, bx_write_handler_t f, Bit32u addr, const char *name, Bit8u mask );
+  bx_bool unregister_io_write_handler( void *this_ptr, bx_write_handler_t f,
+                                        Bit32u addr, Bit8u mask );
+  bx_bool register_io_read_handler_range( void *this_ptr, bx_read_handler_t f,
+		  Bit32u begin_addr, Bit32u end_addr, 
+		  const char *name, Bit8u mask );
+  bx_bool register_io_write_handler_range( void *this_ptr, bx_write_handler_t f,
+		  Bit32u begin_addr, Bit32u end_addr, 
+		  const char *name, Bit8u mask );
+  bx_bool unregister_io_read_handler_range( void *this_ptr, bx_read_handler_t f,
+                                        Bit32u begin, Bit32u end, Bit8u mask );
+  bx_bool unregister_io_write_handler_range( void *this_ptr, bx_write_handler_t f,
+                                        Bit32u begin, Bit32u end, Bit8u mask );
   bx_bool register_default_io_read_handler(void *this_ptr, bx_read_handler_t f, const char *name, Bit8u mask );
   bx_bool register_default_io_write_handler(void *this_ptr, bx_write_handler_t f, const char *name, Bit8u mask );
   bx_bool register_irq(unsigned irq, const char *name);
@@ -293,43 +402,62 @@ public:
   static void timer_handler(void *);
   void timer(void);
 
-  bx_devmodel_c    *pluginBiosDevice;
-  bx_ioapic_c      *ioapic;
-  bx_pci_stub_c    *pluginPciBridge;
-  bx_devmodel_c    *pluginPci2IsaBridge;
-  bx_devmodel_c    *pluginPciVgaAdapter;
-  bx_devmodel_c    *pluginPciUSBAdapter;
-  bx_pit_c         *pit;
-  bx_keyb_stub_c   *pluginKeyboard;
-  bx_dma_stub_c    *pluginDmaDevice;
-  bx_floppy_stub_c *pluginFloppyDevice;
-  bx_cmos_stub_c   *pluginCmosDevice;
-  bx_devmodel_c    *pluginSerialDevice;
-  bx_devmodel_c    *pluginParallelDevice;
-  bx_devmodel_c    *pluginUnmapped;
-  bx_vga_stub_c    *pluginVgaDevice;
-  bx_pic_stub_c    *pluginPicDevice;
+  bx_devmodel_c     *pluginBiosDevice;
+  bx_ioapic_c       *ioapic;
+  bx_pci_stub_c     *pluginPciBridge;
+  bx_pci2isa_stub_c *pluginPci2IsaBridge;
+  bx_pci_ide_stub_c *pluginPciIdeController;
+  bx_devmodel_c     *pluginPciVgaAdapter;
+  bx_devmodel_c     *pluginPciDevAdapter;
+  bx_devmodel_c     *pluginPciPNicAdapter;
+  bx_pit_c          *pit;
+  bx_keyb_stub_c    *pluginKeyboard;
+#if BX_SUPPORT_BUSMOUSE
+  bx_busm_stub_c    *pluginBusMouse;
+#endif
+  bx_dma_stub_c     *pluginDmaDevice;
+  bx_floppy_stub_c  *pluginFloppyDevice;
+  bx_cmos_stub_c    *pluginCmosDevice;
+  bx_serial_stub_c  *pluginSerialDevice;
+#if BX_SUPPORT_PCIUSB
+  bx_usb_stub_c     *pluginPciUSBAdapter;
+#endif
+  bx_devmodel_c     *pluginParallelDevice;
+  bx_devmodel_c     *pluginUnmapped;
+  bx_vga_stub_c     *pluginVgaDevice;
+  bx_pic_stub_c     *pluginPicDevice;
   bx_hard_drive_stub_c *pluginHardDrive;
-  bx_devmodel_c    *pluginSB16Device;
-  bx_ne2k_stub_c   *pluginNE2kDevice;
-  bx_g2h_c         *g2h;
-  bx_devmodel_c    *pluginExtFpuIrq;
-  bx_devmodel_c    *pluginGameport;
-#if BX_IODEBUG_SUPPORT
-  bx_iodebug_c	   *iodebug;
+  bx_devmodel_c     *pluginSB16Device;
+  bx_ne2k_stub_c    *pluginNE2kDevice;
+  bx_g2h_c          *g2h;
+  bx_devmodel_c     *pluginExtFpuIrq;
+  bx_devmodel_c     *pluginGameport;
+  bx_speaker_stub_c *pluginSpeaker;
+#if BX_SUPPORT_IODEBUG
+  bx_iodebug_c	    *iodebug;
 #endif
 
   // stub classes that the pointers (above) can point to until a plugin is
   // loaded
   bx_cmos_stub_c stubCmos;
   bx_keyb_stub_c stubKeyboard;
+#if BX_SUPPORT_BUSMOUSE
+  bx_busm_stub_c stubBusMouse;
+#endif
   bx_hard_drive_stub_c stubHardDrive;
   bx_dma_stub_c  stubDma;
   bx_pic_stub_c  stubPic;
   bx_floppy_stub_c  stubFloppy;
   bx_vga_stub_c  stubVga;
   bx_pci_stub_c  stubPci;
-  bx_ne2k_stub_c stubNE2k;
+  bx_pci2isa_stub_c stubPci2Isa;
+  bx_pci_ide_stub_c stubPciIde;
+  bx_ne2k_stub_c    stubNE2k;
+  bx_speaker_stub_c stubSpeaker;
+  bx_serial_stub_c  stubSerial;
+#if BX_SUPPORT_PCIUSB
+  bx_usb_stub_c     stubUsbAdapter;
+#endif
 
   // Some info to pass to devices which can handled bulk IO.  This allows
   // the interface to remain the same for IO devices which can't handle
@@ -337,30 +465,27 @@ public:
   // functions which stick these values in the bx_devices_c class, and
   // then call the normal functions rather than having gross globals
   // variables.
-  Bit32u   bulkIOHostAddr;
+  Bit8u*   bulkIOHostAddr;
   unsigned bulkIOQuantumsRequested;
   unsigned bulkIOQuantumsTransferred;
 
 private:
 
-  Bit8u                 read_handler_id[0x10000];  // 64K
-  struct {
-    bx_read_handler_t funct;
-    void             *this_ptr;
-    const char       *handler_name;  // name of device
-    Bit8u             mask;          // io_len mask
-    } io_read_handler[BX_MAX_IO_DEVICES];
-  unsigned              num_read_handles;
-
-  Bit8u                 write_handler_id[0x10000]; // 64K
-  struct {
-    bx_write_handler_t funct;
-    void              *this_ptr;
-    const char        *handler_name;  // name of device
-    Bit8u              mask;          // io_len mask
-    } io_write_handler[BX_MAX_IO_DEVICES];
-  unsigned              num_write_handles;
-
+  struct io_handler_struct {
+	struct io_handler_struct *next;
+	struct io_handler_struct *prev;	
+	void *funct; // C++ type checking is great, but annoying
+	void *this_ptr;
+	const char *handler_name;  // name of device
+	int usage_count;
+	Bit8u mask;          // io_len mask
+  };
+  struct io_handler_struct io_read_handlers;
+  struct io_handler_struct io_write_handlers;
+#define PORTS 0x10000
+  struct io_handler_struct **read_port_to_handler;
+  struct io_handler_struct **write_port_to_handler;
+  
   // more for informative purposes, the names of the devices which
   // are use each of the IRQ 0..15 lines are stored here
   const char *irq_handler_name[BX_MAX_IRQS];
@@ -381,13 +506,17 @@ private:
 
 
 
-#if BX_PCI_SUPPORT
+#if BX_SUPPORT_PCI
 #include "iodev/pci.h"
 #include "iodev/pci2isa.h"
-#if BX_PCI_VGA_SUPPORT
+#include "iodev/pci_ide.h"
+#if BX_SUPPORT_PCIVGA
 #include "iodev/pcivga.h"
 #endif
-#if BX_PCI_USB_SUPPORT
+#if BX_SUPPORT_PCIDEV
+#include "iodev/pcidev.h"
+#endif
+#if BX_SUPPORT_PCIUSB
 #include "iodev/pciusb.h"
 #endif
 #endif
@@ -400,11 +529,13 @@ private:
 #include "iodev/dma.h"
 #include "iodev/floppy.h"
 #include "iodev/harddrv.h"
-#include "iodev/vmware3.h"
-#if BX_IODEBUG_SUPPORT
+#if BX_SUPPORT_IODEBUG
 #   include "iodev/iodebug.h"
 #endif
 #include "iodev/keyboard.h"
+#if BX_SUPPORT_BUSMOUSE
+#   include "iodev/busmouse.h"
+#endif
 #include "iodev/parallel.h"
 #include "iodev/pic.h"
 #include "iodev/pit.h"
@@ -415,9 +546,17 @@ private:
 #  include "iodev/sb16.h"
 #endif
 #include "iodev/unmapped.h"
-#include "iodev/eth.h"
 #include "iodev/ne2k.h"
+#if BX_SUPPORT_PCIPNIC
+#include "iodev/pcipnic.h"
+#endif
 #include "iodev/guest2host.h"
 #include "iodev/slowdown_timer.h"
 #include "iodev/extfpuirq.h"
 #include "iodev/gameport.h"
+
+#if ( BX_PROVIDE_DEVICE_MODELS==1 )
+BOCHSAPI extern bx_devices_c   bx_devices;
+#endif
+
+#endif /* IODEV_H */
