@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: slowdown_timer.cc,v 1.17.2.1 2004/02/06 22:14:36 danielg4 Exp $
+// $Id: slowdown_timer.cc,v 1.19.4.1 2005/07/06 20:51:38 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -25,7 +25,7 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 //
 
-#include "bochs.h"
+#include "iodev.h"
 #include <errno.h>
 
 //These need to stay printfs because they are useless in the log file.
@@ -123,7 +123,7 @@ bx_slowdown_timer_c::handle_timer() {
 #endif
   } else {
     bx_pc_system.deactivate_timer(s.timer_handle);
-    bx_pc_system.activate_timer(s.timer_handle,s.Q,0);
+    bx_pc_system.activate_timer(s.timer_handle,(Bit32u)s.Q,0);
 #if BX_SLOWDOWN_PRINTF_FEEDBACK
     printf("running at NORMAL speed\n");
 #endif
@@ -146,7 +146,7 @@ bx_slowdown_timer_c::handle_timer() {
 #if BX_HAVE_USLEEP
     usleep(s.Q);
 #elif BX_HAVE_MSLEEP
-    msleep(usectomsec(s.Q));
+    msleep(usectomsec((Bit32u)s.Q));
 #elif BX_HAVE_SLEEP
     sleep(usectosec(s.Q));
 #else
