@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: speaker.cc,v 1.3 2004/09/05 10:30:19 vruppert Exp $
+// $Id: speaker.cc,v 1.6 2005/02/12 08:59:47 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright 2003 by David N. Welton <davidw@dedasys.com>.
@@ -103,12 +103,34 @@ void bx_speaker_c::beep_on(float frequency)
     this->info("pc speaker on with frequency %f", frequency);
     ioctl(consolefd, KIOCSOUND, (int)(clock_tick_rate/frequency));
   }
+#elif defined(WIN32)
+  usec_start = bx_pc_system.time_usec();
 #endif
    
-  // give the gui a chance to sognal beep off
+  // give the gui a chance to signal beep off
   bx_gui->beep_on(frequency);
 }
 
+#if defined(WIN32)
+
+struct {
+  DWORD frequency;
+  DWORD msec;
+} beep_info;
+
+DWORD WINAPI BeepThread(LPVOID)
+{
+  static BOOL threadActive = FALSE;
+
+  while (threadActive) Sleep(10);
+  threadActive = TRUE;
+  Beep(beep_info.frequency, beep_info.msec);
+  threadActive = FALSE;
+  return 0;
+}
+
+#endif
+
 void bx_speaker_c::beep_off()
 {
   if (beep_frequency != 0.0) {
@@ -116,9 +138,15 @@ void bx_speaker_c::beep_off()
     if (consolefd != -1) {
       ioctl(consolefd, KIOCSOUND, 0);
     }
+#elif defined(WIN32)
+    // FIXME: sound should start at beep_on() and end here
+    DWORD threadID;
+    beep_info.msec = (DWORD)((bx_pc_system.time_usec() - usec_start) / 1000);
+    beep_info.frequency = (DWORD)beep_frequency;
+    CreateThread(NULL, 0, BeepThread, NULL, 0, &threadID);
 #endif
 
-    // give the gui a chance to sognal beep off
+    // give the gui a chance to signal beep off
     bx_gui->beep_off();
     
     beep_frequency = 0.0;
