@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: vga.h,v 1.44 2004/11/06 17:03:44 vruppert Exp $
+// $Id: vga.h,v 1.46 2005/03/18 14:52:29 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -271,7 +271,7 @@ protected:
     unsigned vertical_display_end;
     bx_bool  vga_tile_updated[BX_NUM_X_TILES][BX_NUM_Y_TILES];
     Bit8u vga_memory[256 * 1024];
-    Bit8u text_snapshot[32 * 1024]; // current text snapshot
+    Bit8u text_snapshot[128 * 1024]; // current text snapshot
     Bit8u rgb[3 * 256];
     Bit8u tile[X_TILESIZE * Y_TILESIZE * 4]; /**< Currently allocates the tile as large as needed. */
     Bit16u charmap_address;
@@ -322,13 +322,15 @@ protected:
 #endif
 
   int timer_id;
+  bx_bool extension_init;
+  bx_bool extension_checked;
 
   public:
-  static void   timer_handler(void *);
-  BX_VGA_SMF void   timer(void);
+  static void     timer_handler(void *);
+  BX_VGA_SMF void timer(void);
 
   protected:
-  BX_VGA_SMF void   update(void);
+  BX_VGA_SMF void update(void);
   BX_VGA_SMF void determine_screen_dimensions(unsigned *piHeight,
                                               unsigned *piWidth);
   };
