@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: virt_timer.cc,v 1.19.2.1 2004/02/06 22:14:36 danielg4 Exp $
+// $Id: virt_timer.cc,v 1.21.4.1 2005/07/06 20:46:34 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -62,7 +62,7 @@
 //
 /////////////////////////////////////////////////////////////////////////
 
-#include "bochs.h"
+#include "iodev.h"
 
 #define BX_USE_VIRTUAL_TIMERS 1
 #define BX_VIRTUAL_TIMERS_REALTIME 1
@@ -409,7 +409,8 @@ bx_virt_timer_c::init(void) {
 
   register_timer(this, nullTimer, (Bit32u)NullTimerInterval, 1, 1, "Null Timer");
 
-  system_timer_id = bx_pc_system.register_timer(this, pc_system_timer_handler,virtual_next_event_time , 0, 1, "Virtual Timer");
+  system_timer_id = bx_pc_system.register_timer(this, pc_system_timer_handler,
+                                                (Bit32u)virtual_next_event_time, 0, 1, "Virtual Timer");
 
   //Real time variables:
 #if BX_HAVE_REALTIME_USEC
