@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: main.cc,v 1.282 2004/11/06 10:50:02 vruppert Exp $
+// $Id: main.cc,v 1.289.2.1 2005/07/06 20:26:50 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -25,7 +25,6 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 #include "bochs.h"
-#include "state_file.h"
 #include "iodev/iodev.h"
 
 #ifdef HAVE_LOCALE_H
@@ -98,7 +97,6 @@ bx_bool bx_user_quit;
 
 #if ( BX_PROVIDE_DEVICE_MODELS==1 )
 bx_pc_system_c bx_pc_system;
-class state_file state_stuff("state_file.out", "options");
 #endif
 
 bx_debug_t bx_dbg;
@@ -364,9 +362,9 @@ void RedirectIOToConsole ()
 #endif  /* if defined(__WXMSW__) || (BX_WITH_SDL && defined(WIN32)) */
 
 #if defined(__WXMSW__)
-// only used for wxWindows/win32.
-// This works ok in Cygwin with a standard wxWindows compile.  In
-// VC++ wxWindows must be compiled with -DNOMAIN=1.
+// only used for wxWidgets/win32.
+// This works ok in Cygwin with a standard wxWidgets compile.  In
+// VC++ wxWidgets must be compiled with -DNOMAIN=1.
 int WINAPI WinMain(
   HINSTANCE hInstance,
   HINSTANCE hPrevInstance,
@@ -386,7 +384,7 @@ int WINAPI WinMain(
 
 #if !defined(__WXMSW__)
 // normal main function, presently in for all cases except for
-// wxWindows under win32.
+// wxWidgets under win32.
 int main (int argc, char *argv[])
 {
   bx_startup_flags.argc = argc;
@@ -399,8 +397,7 @@ int main (int argc, char *argv[])
 }
 #endif
 
-void
-print_usage ()
+void print_usage ()
 {
   fprintf(stderr, 
     "Usage: bochs [flags] [bochsrc options]\n\n"
@@ -416,8 +413,7 @@ print_usage ()
 #endif
 }
 
-int
-bx_init_main (int argc, char *argv[])
+int bx_init_main (int argc, char *argv[])
 {
   // To deal with initialization order problems inherent in C++, use the macros
   // SAFE_GET_IOFUNC and SAFE_GET_GENLOG to retrieve "io" and "genlog" in all
@@ -629,12 +625,13 @@ bx_init_main (int argc, char *argv[])
   return 0;
 }
 
-bx_bool load_and_init_display_lib () {
+bx_bool load_and_init_display_lib ()
+{
   if (bx_gui != NULL) {
     // bx_gui has already been filled in.  This happens when you start
     // the simulation for the second time.
-    // Also, if you load wxWindows as the configuration interface.  Its
-    // plugin_init will install wxWindows as the bx_gui.
+    // Also, if you load wxWidgets as the configuration interface.  Its
+    // plugin_init will install wxWidgets as the bx_gui.
     return true;
   }
   BX_ASSERT (bx_gui == NULL);
@@ -648,7 +645,7 @@ bx_bool load_and_init_display_lib () {
   if (!strcmp (gui_name, "wx")) {
     // they must not have used wx as the configuration interface, or bx_gui
     // would already be initialized.  Sorry, it doesn't work that way.
-    BX_ERROR (("wxWindows was not used as the configuration interface, so it cannot be used as the display library"));
+    BX_ERROR (("wxWidgets was not used as the configuration interface, so it cannot be used as the display library"));
     // choose another, hopefully different!
     gui_param->set (0);
     gui_name = gui_param->get_choice (gui_param->get ());
@@ -737,7 +734,6 @@ bx_begin_simulation (int argc, char *argv[])
   else
 #endif
   {
-
     bx_init_hardware();
 
     if (bx_options.load32bitOSImage.OwhichOS->get ()) {
@@ -787,9 +783,7 @@ bx_begin_simulation (int argc, char *argv[])
   return(0);
 }
 
-
-  int
-bx_init_hardware()
+int bx_init_hardware()
 {
   // all configuration has been read, now initialize everything.
 
@@ -838,6 +832,7 @@ bx_init_hardware()
   BX_INFO(("  RepeatSpeedups support: %s",BX_SupportRepeatSpeedups?"yes":"no"));
   BX_INFO(("  Icache support: %s",BX_SUPPORT_ICACHE?"yes":"no"));
   BX_INFO(("  Host Asm support: %s",BX_SupportHostAsms?"yes":"no"));
+  BX_INFO(("  Fast function calls: %s",BX_FAST_FUNC_CALL?"yes":"no"));
 
   // set up memory and CPU objects
 #if BX_SUPPORT_APIC
@@ -849,8 +844,14 @@ bx_init_hardware()
     BX_ERROR(("No romimage to load. Is your bochsrc file loaded/valid ?"));
   }
 
+  Bit32u memSize = bx_options.memory.Osize->get ()*1024*1024;
+
+#if BX_SUPPORT_ICACHE
+  pageWriteStampTable.alloc(memSize);
+#endif
+
 #if BX_SMP_PROCESSORS==1
-  BX_MEM(0)->init_memory(bx_options.memory.Osize->get () * 1024*1024);
+  BX_MEM(0)->init_memory(memSize);
 
   // First load the BIOS and VGABIOS
   BX_MEM(0)->load_ROM(bx_options.rom.Opath->getptr (), bx_options.rom.Oaddress->get (), 0);
@@ -877,7 +878,7 @@ bx_init_hardware()
 #else
   // SMP initialization
   bx_mem_array[0] = new BX_MEM_C ();
-  bx_mem_array[0]->init_memory(bx_options.memory.Osize->get () * 1024*1024);
+  bx_mem_array[0]->init_memory(memSize);
 
   // First load the BIOS and VGABIOS
   bx_mem_array[0]->load_ROM(bx_options.rom.Opath->getptr (), bx_options.rom.Oaddress->get (), 0);
@@ -928,10 +929,7 @@ bx_init_hardware()
   return(0);
 }
 
-
-
-  void
-bx_init_bx_dbg (void)
+void bx_init_bx_dbg (void)
 {
   bx_dbg.floppy = 0;
   bx_dbg.keyboard = 0;
@@ -963,15 +961,12 @@ bx_init_bx_dbg (void)
 #if BX_MAGIC_BREAKPOINT
   bx_dbg.magic_break_enabled = 0;
 #endif
-#ifdef BX_GDBSTUB
+#if BX_GDBSTUB
   bx_dbg.gdbstub_enabled = 0;
 #endif
-
 }
 
-
-int
-bx_atexit(void)
+int bx_atexit(void)
 {
   static bx_bool been_here = 0;
   if (been_here) return 1;   // protect from reentry
@@ -993,9 +988,11 @@ bx_atexit(void)
 #endif
 
 #if BX_SUPPORT_PCI
-  if (bx_options.Oi440FXSupport->get ()) {
-    bx_devices.pluginPciBridge->print_i440fx_state();
+  if (SIM && SIM->get_init_done ()) {
+    if (bx_options.Oi440FXSupport->get ()) {
+      bx_devices.pluginPciBridge->print_i440fx_state();
     }
+  }
 #endif
 
   // restore signal handling to defaults
@@ -1009,11 +1006,11 @@ bx_atexit(void)
   signal(SIGALRM, SIG_DFL);
 #endif
 #endif
-        return 0;
+
+  return 0;
 }
 
-  void
-bx_signal_handler( int signum)
+void bx_signal_handler(int signum)
 {
   // in a multithreaded environment, a signal such as SIGINT can be sent to all
   // threads.  This function is only intended to handle signals in the
@@ -1036,17 +1033,22 @@ bx_signal_handler( int signum)
 #endif
 
 #if BX_SHOW_IPS
-  extern unsigned long ips_count;
+  static Bit64u ticks_count = 0;
 
-  if (signum == SIGALRM ) {
-    BX_INFO(("ips = %lu", ips_count));
-    ips_count = 0;
+  if (signum == SIGALRM)
+  {
+    // amount of system ticks passed from last time the handler was called
+    Bit64u ips_count = bx_pc_system.time_ticks() - ticks_count;
+    if (ips_count) {
+      BX_INFO(("ips = %lu", (unsigned long) ips_count));
+      ticks_count = bx_pc_system.time_ticks();
+    }
 #ifndef __MINGW32__
     signal(SIGALRM, bx_signal_handler);
     alarm( 1 );
 #endif
     return;
-    }
+  }
 #endif
 
 #if BX_GUI_SIGHANDLER
@@ -1057,6 +1059,6 @@ bx_signal_handler( int signum)
     }
   }
 #endif
+
   BX_PANIC(("SIGNAL %u caught", signum));
 }
-
