@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: memory.cc,v 1.35 2004/10/21 18:20:36 sshwarts Exp $
+// $Id: memory.cc,v 1.40 2005/04/10 19:42:48 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -67,27 +67,40 @@ BX_MEM_C::writePhysicalPage(BX_CPU_C *cpu, Bit32u addr, unsigned len, void *data
 
 #if BX_SUPPORT_ICACHE
   if (a20addr < BX_MEM_THIS len)
-    cpu->iCache.decWriteStamp(a20addr);
+    pageWriteStampTable.decWriteStamp(a20addr);
 #endif
 
-  if ( a20addr <= BX_MEM_THIS len ) {
+#if BX_SUPPORT_APIC
+    bx_generic_apic_c *local_apic = &cpu->local_apic;
+    bx_generic_apic_c *ioapic = bx_devices.ioapic;
+    if (local_apic->is_selected (a20addr, len)) {
+      local_apic->write (a20addr, (Bit32u *)data, len);
+      return;
+    }
+    if (ioapic->is_selected (a20addr, len)) {
+      ioapic->write (a20addr, (Bit32u *)data, len);
+      return;
+    }
+#endif
+
+  if ( (a20addr + len) <= BX_MEM_THIS len ) {
     // all of data is within limits of physical memory
     if ( (a20addr & 0xfff80000) != 0x00080000 ) {
       if (len == 4) {
         WriteHostDWordToLittleEndian(&vector[a20addr], *(Bit32u*)data);
         BX_DBG_DIRTY_PAGE(a20addr >> 12);
         return;
-        }
+      }
       if (len == 2) {
         WriteHostWordToLittleEndian(&vector[a20addr], *(Bit16u*)data);
         BX_DBG_DIRTY_PAGE(a20addr >> 12);
         return;
-        }
+      }
       if (len == 1) {
         * ((Bit8u *) (&vector[a20addr])) = * (Bit8u *) data;
         BX_DBG_DIRTY_PAGE(a20addr >> 12);
         return;
-        }
+      }
       // len == other, just fall thru to special cases handling
       }
 
@@ -112,7 +125,7 @@ inc_one:
       data_ptr--;
 #endif
       goto write_one;
-      }
+    }
 
     // addr in range 00080000 .. 000FFFFF
 
@@ -121,25 +134,27 @@ inc_one:
       vector[a20addr] = *data_ptr;
       BX_DBG_DIRTY_PAGE(a20addr >> 12);
       goto inc_one;
-      }
+    }
     // adapter ROM     C0000 .. DFFFF
     // ROM BIOS memory E0000 .. FFFFF
 #if BX_SUPPORT_PCI == 0
     // ignore write to ROM
 #else
     // Write Based on 440fx Programming
-    if (bx_options.Oi440FXSupport->get () &&
-        ((a20addr >= 0xC0000) && (a20addr <= 0xFFFFF))) {
+    if ( bx_options.Oi440FXSupport->get () &&
+          ((a20addr & 0xfffc0000) == 0x000c0000) )
+    {
       switch (DEV_pci_wr_memtype(a20addr & 0xFC000)) {
         case 0x1:   // Writes to ShadowRAM
           BX_DEBUG(("Writing to ShadowRAM: address %08x, data %02x", (unsigned) a20addr, *data_ptr));
-          shadow[a20addr - 0xc0000] = *data_ptr;
+          vector[a20addr] = *data_ptr;
           BX_DBG_DIRTY_PAGE(a20addr >> 12);
           goto inc_one;
 
         case 0x0:   // Writes to ROM, Inhibit
           BX_DEBUG(("Write to ROM ignored: address %08x, data %02x", (unsigned) a20addr, *data_ptr));
           goto inc_one;
+
         default:
           BX_PANIC(("writePhysicalPage: default case"));
           goto inc_one;
@@ -147,36 +162,21 @@ inc_one:
       }
 #endif
     goto inc_one;
-    }
-
+  }
   else {
     // some or all of data is outside limits of physical memory
-    unsigned i;
 
 #ifdef BX_LITTLE_ENDIAN
-  data_ptr = (Bit8u *) data;
+    data_ptr = (Bit8u *) data;
 #else // BX_BIG_ENDIAN
-  data_ptr = (Bit8u *) data + (len - 1);
+    data_ptr = (Bit8u *) data + (len - 1);
 #endif
 
-
-#if BX_SUPPORT_APIC
-    bx_generic_apic_c *local_apic = &cpu->local_apic;
-    bx_generic_apic_c *ioapic = bx_devices.ioapic;
-    if (local_apic->is_selected (a20addr, len)) {
-      local_apic->write (a20addr, (Bit32u *)data, len);
-      return;
-    } else if (ioapic->is_selected (a20addr, len)) {
-      ioapic->write (a20addr, (Bit32u *)data, len);
-      return;
-    }
-    else 
-#endif
-    for (i = 0; i < len; i++) {
+    for (unsigned i = 0; i < len; i++) {
       if (a20addr < BX_MEM_THIS len) {
         vector[a20addr] = *data_ptr;
         BX_DBG_DIRTY_PAGE(a20addr >> 12);
-        }
+      }
       // otherwise ignore byte, since it overruns memory
       addr++;
       a20addr = (addr);
@@ -185,9 +185,9 @@ inc_one:
 #else // BX_BIG_ENDIAN
       data_ptr--;
 #endif
-      }
-    return;
     }
+    return;
+  }
 }
 
 
@@ -224,23 +224,36 @@ BX_MEM_C::readPhysicalPage(BX_CPU_C *cpu, Bit32u addr, unsigned len, void *data)
         }
 #endif
 
+#if BX_SUPPORT_APIC
+  bx_generic_apic_c *local_apic = &cpu->local_apic;
+  bx_generic_apic_c *ioapic = bx_devices.ioapic;
+    if (local_apic->is_selected (addr, len)) {
+      local_apic->read (addr, data, len);
+      return;
+    }
+    if (ioapic->is_selected (addr, len)) {
+      ioapic->read (addr, data, len);
+      return;
+    }
+#endif
+
   if ( (a20addr + len) <= BX_MEM_THIS len ) {
     // all of data is within limits of physical memory
     if ( (a20addr & 0xfff80000) != 0x00080000 ) {
       if (len == 4) {
         ReadHostDWordFromLittleEndian(&vector[a20addr], * (Bit32u*) data);
         return;
-        }
+      }
       if (len == 2) {
         ReadHostWordFromLittleEndian(&vector[a20addr], * (Bit16u*) data);
         return;
-        }
+      }
       if (len == 1) {
         * (Bit8u *) data =  * ((Bit8u *) (&vector[a20addr]));
         return;
-        }
-      // len == 3 case can just fall thru to special cases handling
       }
+      // len == 3 case can just fall thru to special cases handling
+    }
 
 
 #ifdef BX_LITTLE_ENDIAN
@@ -249,8 +262,6 @@ BX_MEM_C::readPhysicalPage(BX_CPU_C *cpu, Bit32u addr, unsigned len, void *data)
     data_ptr = (Bit8u *) data + (len - 1);
 #endif
 
-
-
 read_one:
     if ( (a20addr & 0xfff80000) != 0x00080000 ) {
       // addr *not* in range 00080000 .. 000FFFFF
@@ -265,47 +276,40 @@ inc_one:
       data_ptr--;
 #endif
       goto read_one;
-      }
+    }
 
     // addr in range 00080000 .. 000FFFFF
-#if BX_SUPPORT_PCI == 0
-    if ((a20addr <= 0x0009ffff) || (a20addr >= 0x000c0000) ) {
-      // regular memory 80000 .. 9FFFF, C0000 .. F0000
-      *data_ptr = vector[a20addr];
-      goto inc_one;
+#if BX_SUPPORT_PCI
+    if ( bx_options.Oi440FXSupport->get () &&
+          ((a20addr & 0xfffc0000) == 0x000c0000) )
+    {
+      switch (DEV_pci_rd_memtype (a20addr)) {
+        case 0x0:  // Read from ROM
+          *data_ptr = rom[a20addr - 0xc0000];
+          goto inc_one;
+        case 0x1:  // Read from ShadowRAM
+          *data_ptr = vector[a20addr];
+          goto inc_one;
+        default:
+          BX_PANIC(("readPhysicalPage: default case"));
       }
-#else   // #if BX_SUPPORT_PCI == 0
-    if (a20addr <= 0x0009ffff) {
-      *data_ptr = vector[a20addr];
       goto inc_one;
-      }
-
-    if ((a20addr >= 0xC0000) && (a20addr <= 0xFFFFF)) {
-      if (!bx_options.Oi440FXSupport->get ()) {
-        *data_ptr = vector[a20addr];
-        goto inc_one;
-        }
-      else {
-        switch (DEV_pci_rd_memtype(a20addr & 0xFC000)) {
-          case 0x1:   // Read from ShadowRAM
-            *data_ptr = shadow[a20addr - 0xc0000];
-            BX_DEBUG(("Reading to ShadowRAM: address %08x, data %02x", (unsigned) a20addr, *data_ptr));
-            goto inc_one;
-
-          case 0x0:   // Read from ROM
-            *data_ptr = vector[a20addr];
-            goto inc_one;
-          default:
-            BX_PANIC(("::readPhysicalPage: default case"));
-          }
-        }
-      goto inc_one;
-      }
-#endif  // #if BX_SUPPORT_PCI == 0
     }
-  else {
-    // some or all of data is outside limits of physical memory
-    unsigned i;
+    else
+#endif  // #if BX_SUPPORT_PCI
+    {
+      if ( (a20addr & 0xfffc0000) == 0x000c0000 ) {
+        *data_ptr = rom[a20addr - 0xc0000];
+      }
+      else
+      {
+        *data_ptr = vector[a20addr];
+      }
+      goto inc_one;
+    }
+  }
+  else
+  {  // some or all of data is outside limits of physical memory
 
 #ifdef BX_LITTLE_ENDIAN
     data_ptr = (Bit8u *) data;
@@ -313,52 +317,13 @@ inc_one:
     data_ptr = (Bit8u *) data + (len - 1);
 #endif
 
-
-#if BX_SUPPORT_APIC
-    bx_generic_apic_c *local_apic = &cpu->local_apic;
-    bx_generic_apic_c *ioapic = bx_devices.ioapic;
-    if (local_apic->is_selected (addr, len)) {
-      local_apic->read (addr, data, len);
-      return;
-    } else if (ioapic->is_selected (addr, len)) {
-      ioapic->read (addr, data, len);
-      return;
-    }
-#endif
-    for (i = 0; i < len; i++) {
-#if BX_SUPPORT_PCI == 0
+    for (unsigned i = 0; i < len; i++) {
       if (a20addr < BX_MEM_THIS len)
         *data_ptr = vector[a20addr];
+      else if (a20addr >= 0xfffe0000)
+        *data_ptr = rom[a20addr & 0x3ffff];
       else
         *data_ptr = 0xff;
-#else   // BX_SUPPORT_PCI == 0
-      if (a20addr < BX_MEM_THIS len) {
-        if ((a20addr >= 0x000C0000) && (a20addr <= 0x000FFFFF)) {
-          if (!bx_options.Oi440FXSupport->get ())
-            *data_ptr = vector[a20addr];
-          else {
-            switch (DEV_pci_rd_memtype(a20addr & 0xFC000)) {
-              case 0x0:   // Read from ROM
-                *data_ptr = vector[a20addr];
-                break;
-
-              case 0x1:   // Read from Shadow RAM
-                *data_ptr = shadow[a20addr - 0xc0000];
-                BX_DEBUG(("Reading to ShadowRAM: address %08x, data %02x", (unsigned) a20addr, *data_ptr));
-                break;
-              default:
-                BX_PANIC(("readPhysicalPage: default case"));
-              } // Switch
-            }
-          }
-        else {
-          *data_ptr = vector[a20addr];
-          BX_INFO(("Reading from Norm %08x, Data %02x  ", (unsigned) a20addr, *data_ptr));
-          }
-        }
-      else 
-        *data_ptr = 0xff;
-#endif  // BX_SUPPORT_PCI == 0
       addr++;
       a20addr = (addr);
 #ifdef BX_LITTLE_ENDIAN
@@ -366,9 +331,9 @@ inc_one:
 #else // BX_BIG_ENDIAN
       data_ptr--;
 #endif
-      }
-    return;
     }
+    return;
+  }
 }
 
 #endif // #if BX_PROVIDE_CPU_MEMORY
