@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: memory.h,v 1.16 2003/08/05 13:19:35 cbothamy Exp $
+// $Id: memory.h,v 1.24 2004/11/16 18:50:21 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -10,6 +10,8 @@
 //    http://www.linux-mandrake.com/
 //    http://www.mandrakesoft.com/
 //
+//  I/O memory handlers API Copyright (C) 2003 by Frank Cornelis
+//
 //  This library is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU Lesser General Public
 //  License as published by the Free Software Foundation; either
@@ -24,7 +26,8 @@
 //  License along with this library; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
-
+#ifndef BX_MEM_H
+#  define BX_MEM_H 1
 
 #define BX_USE_MEM_SMF 0
 
@@ -40,16 +43,30 @@
 // alignment of memory vector, must be a power of 2
 #define BX_MEM_VECTOR_ALIGN 4096
 
-class BOCHSAPI BX_MEM_C : public logfunctions {
+typedef bx_bool (*memory_handler_t)(unsigned long addr, unsigned long len, void *data, void *param);
 
+struct memory_handler_struct {
+	struct memory_handler_struct *next;
+	unsigned long begin;
+	unsigned long end;
+	memory_handler_t read_handler;
+	memory_handler_t write_handler;
+	void *read_param;
+	void *write_param;
+};
+
+class BOCHSAPI BX_MEM_C : public logfunctions {
+private:
+  struct memory_handler_struct **memory_handlers;
+  bx_bool rom_present[65];
+  
 public:
   Bit8u   *actual_vector;
-  Bit8u   *vector;  // aligned correctly
+  Bit8u   *vector;   // aligned correctly
   size_t  len;
-  size_t  megabytes;  // (len in Megabytes)
-#if BX_PCI_SUPPORT
-  Bit8u   shadow[4*16*4096]; // 256k of memory
-#endif
+  size_t  megabytes; // (len in Megabytes)
+  Bit8u   *rom;      // 256k rom space
+  Bit8u   *bogus;    // 4k for unexisting memory
 #if BX_DEBUGGER
   unsigned char dbg_dirty_pages[(BX_MAX_DIRTY_PAGE_TABLE_MEGS * 1024 * 1024) / 4096];
   Bit32u dbg_count_dirty_pages () {
@@ -58,7 +75,6 @@ public:
 #endif
 
   BX_MEM_C(void);
-  //BX_MEM_C(size_t memsize);
   ~BX_MEM_C(void);
   BX_MEM_SMF void    alloc_vector_aligned (size_t bytes, size_t alignment) BX_CPP_AttrRegparmN(2);
   BX_MEM_SMF void    init_memory(int memsize);
@@ -68,15 +84,17 @@ public:
                                        unsigned len, void *data) BX_CPP_AttrRegparmN(3);
   BX_MEM_SMF void    load_ROM(const char *path, Bit32u romaddress, Bit8u type);
   BX_MEM_SMF Bit32u  get_memory_in_k(void);
-#if BX_PCI_SUPPORT
-  BX_MEM_SMF Bit8u*  pci_fetch_ptr(Bit32u addr) BX_CPP_AttrRegparmN(1);
-#endif
   BX_MEM_SMF bx_bool dbg_fetch_mem(Bit32u addr, unsigned len, Bit8u *buf);
   BX_MEM_SMF bx_bool dbg_set_mem(Bit32u addr, unsigned len, Bit8u *buf);
   BX_MEM_SMF bx_bool dbg_crc32(
     unsigned long (*f)(unsigned char *buf, int len),
     Bit32u addr1, Bit32u addr2, Bit32u *crc);
-  BX_MEM_SMF Bit8u * getHostMemAddr(BX_CPU_C *cpu, Bit32u a20Addr, unsigned op) BX_CPP_AttrRegparmN(3);
+  BX_MEM_SMF Bit8u* getHostMemAddr(BX_CPU_C *cpu, Bit32u a20Addr, unsigned op) BX_CPP_AttrRegparmN(3);
+  BX_MEM_SMF bx_bool registerMemoryHandlers(memory_handler_t read_handler, void *read_param, 
+		  memory_handler_t write_handler, void *write_param, 
+		  unsigned long begin_addr, unsigned long end_addr);
+  BX_MEM_SMF bx_bool unregisterMemoryHandlers(memory_handler_t read_handler, memory_handler_t write_handler, 
+		  unsigned long begin_addr, unsigned long end_addr);
   };
 
 #if BX_PROVIDE_CPU_MEMORY==1
@@ -94,3 +112,5 @@ BOCHSAPI extern BX_MEM_C    *bx_mem_array[BX_ADDRESS_SPACES];
 #else
 #  define BX_DBG_DIRTY_PAGE(page)
 #endif
+
+#endif
