@@ -1,6 +1,6 @@
 /*
  * misc/bximage.c
- * $Id: bxcommit.c,v 1.8 2004/08/19 19:42:22 vruppert Exp $
+ * $Id: bxcommit.c,v 1.9.2.1 2005/07/06 20:40:48 vruppert Exp $
  *
  * Commits a redolog file in a flat file for bochs images.
  *
@@ -43,17 +43,13 @@ int snprintf (char *s, size_t maxlen, const char *format, ...)
 }
 #endif  /* !BX_HAVE_SNPRINTF */
 
-#define uint8   Bit8u
-#define uint16  Bit16u
-#define uint32  Bit32u
-
 #include "../osdep.h"
 
 #define INCLUDE_ONLY_HD_HEADERS 1
 #include "../iodev/harddrv.h"
 
 char *EOF_ERR = "ERROR: End of input";
-char *rcsid = "$Id: bxcommit.c,v 1.8 2004/08/19 19:42:22 vruppert Exp $";
+char *rcsid = "$Id: bxcommit.c,v 1.9.2.1 2005/07/06 20:40:48 vruppert Exp $";
 char *divider = "========================================================================";
 
 void myexit (int code)
@@ -288,7 +284,7 @@ int commit_redolog (char *flatname, char *redologname )
   lseek(redologfd, dtoh32(header.standard.header), SEEK_SET);
 
   catalog_size = dtoh32(header.specific.catalog) * sizeof(Bit32u);
-  if (read(redologfd, catalog, catalog_size)!= catalog_size)
+  if ((Bit32u) read(redologfd, catalog, catalog_size) != catalog_size)
      fatal ("\nERROR: while reading redolog catalog!");
 
   printf ("] Done.");
@@ -315,7 +311,7 @@ int commit_redolog (char *flatname, char *redologname )
                   lseek(redologfd, bitmap_offset, SEEK_SET);
 
                   bitmap_size = dtoh32(header.specific.bitmap);
-                  if (read(redologfd, bitmap, bitmap_size) != bitmap_size)
+                  if ((Bit32u) read(redologfd, bitmap, bitmap_size) != bitmap_size)
                           fatal ("\nERROR: while reading bitmap from redolog !");
 
                   for(j=0; j<dtoh32(header.specific.bitmap); j++)
