@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pc_system.cc,v 1.34 2003/06/07 19:16:51 vruppert Exp $
+// $Id: pc_system.cc,v 1.40.2.1 2005/07/06 20:20:24 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -27,6 +27,7 @@
 
 
 #include "bochs.h"
+#include "iodev/iodev.h"
 #define LOG_THIS bx_pc_system.
 
 #ifdef WIN32
@@ -36,10 +37,6 @@
 #endif
 #endif
 
-#if BX_SHOW_IPS
-unsigned long ips_count=0;
-#endif
-
 #if defined(PROVIDE_M_IPS)
 double     m_ips; // Millions of Instructions Per Second
 #endif
@@ -50,11 +47,8 @@ double     m_ips; // Millions of Instructions Per Second
 #define SpewPeriodicTimerInfo 0
 #define MinAllowableTimerPeriod 1
 
-
-#if SpewPeriodicTimerInfo
-// If debugging, set the heartbeat to 5M cycles.  Each heartbeat
-// spews the active timer info.
-const Bit64u bx_pc_system_c::NullTimerInterval = 5000000;
+#if BX_SUPPORT_ICACHE
+const Bit64u bx_pc_system_c::NullTimerInterval = ICacheWriteStampMax;
 #else
 // This must be the maximum 32-bit unsigned int value, NOT (Bit64u) -1.
 const Bit64u bx_pc_system_c::NullTimerInterval = 0xffffffff;
@@ -78,12 +72,12 @@ bx_pc_system_c::bx_pc_system_c(void)
   currCountdown       = NullTimerInterval;
   currCountdownPeriod = NullTimerInterval;
   numTimers = 1; // So far, only the nullTimer.
+  triggeredTimer = 0;
   lastTimeUsec = 0;
   usecSinceLast = 0;
 }
 
-  void
-bx_pc_system_c::init_ips(Bit32u ips)
+void bx_pc_system_c::init_ips(Bit32u ips)
 {
   HRQ = 0;
 
@@ -104,8 +98,7 @@ bx_pc_system_c::init_ips(Bit32u ips)
   BX_DEBUG(("ips = %u", (unsigned) ips));
 }
 
-  void
-bx_pc_system_c::set_HRQ(bx_bool val)
+void bx_pc_system_c::set_HRQ(bx_bool val)
 {
   HRQ = val;
   if (val)
@@ -114,8 +107,7 @@ bx_pc_system_c::set_HRQ(bx_bool val)
 
 
 #if (BX_NUM_SIMULATORS < 2)
-  void
-bx_pc_system_c::set_INTR(bx_bool value)
+void bx_pc_system_c::set_INTR(bx_bool value)
 {
   if (bx_dbg.interrupts)
     BX_INFO(("pc_system: Setting INTR=%d on bootstrap processor %d", (int)value, BX_BOOTSTRAP_PROCESSOR));
@@ -166,11 +158,11 @@ bx_pc_system_c::set_enable_a20(Bit8u value)
 #else /* 386+ */
     a20_mask   = 0xffffffff; /* 386: enable all 32 address lines */
 #endif
-    }
+  }
   else {
     enable_a20 = 0;
     a20_mask   = 0xffefffff;   /* mask off A20 address line */
-    }
+  }
 
   BX_DBG_A20_REPORT(value);
 
@@ -190,8 +182,7 @@ bx_pc_system_c::set_enable_a20(Bit8u value)
 #endif
 }
 
-  bx_bool
-bx_pc_system_c::get_enable_a20(void)
+bx_bool bx_pc_system_c::get_enable_a20(void)
 {
 #if BX_SUPPORT_A20
   if (bx_dbg.a20)
@@ -205,28 +196,30 @@ bx_pc_system_c::get_enable_a20(void)
 #endif  // #if BX_SUPPORT_A20
 }
 
-  int
-bx_pc_system_c::ResetSignal( PCS_OP operation )
+int bx_pc_system_c::Reset( unsigned type )
 {
-  UNUSED( operation );
-  // Reset the processor.
+  // type is BX_RESET_HARDWARE or BX_RESET_SOFTWARE
+  BX_INFO(( "bx_pc_system_c::Reset(%s) called",type==BX_RESET_HARDWARE?"HARDWARE":"SOFTWARE" ));
+
+  // Always reset cpu
+  for (int i=0; i<BX_SMP_PROCESSORS; i++) {
+    BX_CPU(i)->reset(type);
+  }
+
+  // Reset devices only on Hardware resets
+  if (type==BX_RESET_HARDWARE) {
+    DEV_reset_devices(type);
+  }
 
-  BX_ERROR(( "# bx_pc_system_c::ResetSignal() called" ));
-  for (int i=0; i<BX_SMP_PROCESSORS; i++)
-    BX_CPU(i)->reset(BX_RESET_SOFTWARE);
-  DEV_reset_devices(BX_RESET_SOFTWARE);
   return(0);
 }
 
-
-  Bit8u
-bx_pc_system_c::IAC(void)
+Bit8u bx_pc_system_c::IAC(void)
 {
   return( DEV_pic_iac() );
 }
 
-  void
-bx_pc_system_c::exit(void)
+void bx_pc_system_c::exit(void)
 {
   if (DEV_hd_present())
     DEV_hd_close_harddrive();
@@ -241,8 +234,7 @@ bx_pc_system_c::exit(void)
 // Bochs internal timer delivery framework features
 // ================================================
 
-  int
-bx_pc_system_c::register_timer( void *this_ptr, void (*funct)(void *),
+int bx_pc_system_c::register_timer( void *this_ptr, void (*funct)(void *),
   Bit32u useconds, bx_bool continuous, bx_bool active, const char *id)
 {
   Bit64u ticks;
@@ -253,8 +245,7 @@ bx_pc_system_c::register_timer( void *this_ptr, void (*funct)(void *),
   return register_timer_ticks(this_ptr, funct, ticks, continuous, active, id);
 }
 
-  int
-bx_pc_system_c::register_timer_ticks(void* this_ptr, bx_timer_handler_t funct,
+int bx_pc_system_c::register_timer_ticks(void* this_ptr, bx_timer_handler_t funct,
     Bit64u ticks, bx_bool continuous, bx_bool active, const char *id)
 {
   unsigned i;
@@ -262,7 +253,7 @@ bx_pc_system_c::register_timer_ticks(void* this_ptr, bx_timer_handler_t funct,
 #if BX_TIMER_DEBUG
   if (numTimers >= BX_MAX_TIMERS) {
     BX_PANIC(("register_timer: too many registered timers."));
-    }
+  }
   if (this_ptr == NULL)
     BX_PANIC(("register_timer_ticks: this_ptr is NULL"));
   if (funct == NULL)
@@ -275,12 +266,12 @@ bx_pc_system_c::register_timer_ticks(void* this_ptr, bx_timer_handler_t funct,
     //BX_INFO(("register_timer_ticks: adjusting ticks of %llu to min of %u",
     //          ticks, MinAllowableTimerPeriod));
     ticks = MinAllowableTimerPeriod;
-    }
+  }
 
   for (i=0; i < numTimers; i++) {
     if (timer[i].inUse == 0)
       break;
-    }
+  }
 
   timer[i].inUse      = 1;
   timer[i].period     = ticks;
@@ -300,8 +291,8 @@ bx_pc_system_c::register_timer_ticks(void* this_ptr, bx_timer_handler_t funct,
       // by the delta.
       currCountdownPeriod -= (currCountdown - Bit32u(ticks));
       currCountdown = Bit32u(ticks);
-      }
     }
+  }
 
   // If we didn't find a free slot, increment the bound, numTimers.
   if (i==numTimers)
@@ -311,9 +302,7 @@ bx_pc_system_c::register_timer_ticks(void* this_ptr, bx_timer_handler_t funct,
   return(i);
 }
 
-
-  void
-bx_pc_system_c::countdownEvent(void)
+void bx_pc_system_c::countdownEvent(void)
 {
   unsigned i;
   Bit64u   minTimeToFire;
@@ -346,21 +335,21 @@ bx_pc_system_c::countdownEvent(void)
         if (timer[i].continuous==0) {
           // If triggered timer is one-shot, deactive.
           timer[i].active = 0;
-          }
+        }
         else {
           // Continuous timer, increment time-to-fire by period.
           timer[i].timeToFire += timer[i].period;
           if (timer[i].timeToFire < minTimeToFire)
             minTimeToFire = timer[i].timeToFire;
-          }
         }
+      }
       else {
         // This timer is not ready to fire yet.
         if (timer[i].timeToFire < minTimeToFire)
           minTimeToFire = timer[i].timeToFire;
-        }
       }
     }
+  }
 
   // Calculate next countdown period.  We need to do this before calling
   // any of the callbacks, as they may call timer features, which need
@@ -372,13 +361,14 @@ bx_pc_system_c::countdownEvent(void)
     // Call requested timer function.  It may request a different
     // timer period or deactivate etc.
     if (triggered[i]) {
+      triggeredTimer = i;
       timer[i].funct(timer[i].this_ptr);
-      }
+      triggeredTimer = 0;
     }
+  }
 }
 
-  void
-bx_pc_system_c::nullTimer(void* this_ptr)
+void bx_pc_system_c::nullTimer(void* this_ptr)
 {
   // This function is always inserted in timer[0].  It is sort of
   // a heartbeat timer.  It ensures that at least one timer is
@@ -399,14 +389,17 @@ bx_pc_system_c::nullTimer(void* this_ptr)
       BX_INFO(("BxTimer(%s): period=" FMT_LL "u, continuous=%u",
                bx_pc_system.timer[i].id, bx_pc_system.timer[i].period,
                bx_pc_system.timer[i].continuous));
-      }
     }
+  }
+#endif
+
+#if BX_SUPPORT_ICACHE
+  purgeICache();
 #endif
 }
 
 #if BX_DEBUGGER
-  void
-bx_pc_system_c::timebp_handler(void* this_ptr)
+void bx_pc_system_c::timebp_handler(void* this_ptr)
 {
       BX_CPU(0)->break_point = BREAK_POINT_TIME;
       BX_DEBUG(( "Time breakpoint triggered" ));
@@ -421,8 +414,8 @@ bx_pc_system_c::timebp_handler(void* this_ptr)
 }
 #endif // BX_DEBUGGER
 
-  Bit64u
-bx_pc_system_c::time_usec_sequential() {
+Bit64u bx_pc_system_c::time_usec_sequential()
+{
     Bit64u this_time_usec = time_usec();
     if(this_time_usec != lastTimeUsec) {
       Bit64u diff_usec = this_time_usec-lastTimeUsec;
@@ -436,18 +429,14 @@ bx_pc_system_c::time_usec_sequential() {
     usecSinceLast++;
     return (this_time_usec+usecSinceLast);
 }
-  Bit64u
-bx_pc_system_c::time_usec() {
+
+Bit64u bx_pc_system_c::time_usec() {
   return (Bit64u) (((double)(Bit64s)time_ticks()) / m_ips );
 }
 
-  void
-bx_pc_system_c::start_timers(void)
-{
-}
+void bx_pc_system_c::start_timers(void) { }
 
-  void
-bx_pc_system_c::activate_timer_ticks(unsigned i, Bit64u ticks, bx_bool continuous)
+void bx_pc_system_c::activate_timer_ticks(unsigned i, Bit64u ticks, bx_bool continuous)
 {
 #if BX_TIMER_DEBUG
   if (i >= numTimers)
@@ -463,7 +452,7 @@ bx_pc_system_c::activate_timer_ticks(unsigned i, Bit64u ticks, bx_bool continuou
     //BX_INFO(("activate_timer_ticks: adjusting ticks of %llu to min of %u",
     //          ticks, MinAllowableTimerPeriod));
     ticks = MinAllowableTimerPeriod;
-    }
+  }
 
   timer[i].period = ticks;
   timer[i].timeToFire = (ticksTotal + Bit64u(currCountdownPeriod-currCountdown)) +
@@ -477,11 +466,10 @@ bx_pc_system_c::activate_timer_ticks(unsigned i, Bit64u ticks, bx_bool continuou
     // by the delta.
     currCountdownPeriod -= (currCountdown - Bit32u(ticks));
     currCountdown = Bit32u(ticks);
-    }
+  }
 }
 
-  void
-bx_pc_system_c::activate_timer(unsigned i, Bit32u useconds, bx_bool continuous)
+void bx_pc_system_c::activate_timer(unsigned i, Bit32u useconds, bx_bool continuous)
 {
   Bit64u ticks;
 
@@ -494,7 +482,7 @@ bx_pc_system_c::activate_timer(unsigned i, Bit32u useconds, bx_bool continuous)
   // else set new period from useconds
   if (useconds==0) {
     ticks = timer[i].period;
-    }
+  }
   else {
     // convert useconds to number of ticks
     ticks = (Bit64u) (double(useconds) * m_ips);
@@ -505,16 +493,15 @@ bx_pc_system_c::activate_timer(unsigned i, Bit32u useconds, bx_bool continuous)
       //BX_INFO(("activate_timer: adjusting ticks of %llu to min of %u",
       //          ticks, MinAllowableTimerPeriod));
       ticks = MinAllowableTimerPeriod;
-      }
+    }
 
     timer[i].period = ticks;
-    }
+  }
 
   activate_timer_ticks(i, ticks, continuous);
 }
 
-  void
-bx_pc_system_c::deactivate_timer( unsigned i )
+void bx_pc_system_c::deactivate_timer( unsigned i )
 {
 #if BX_TIMER_DEBUG
   if (i >= numTimers)
@@ -524,8 +511,7 @@ bx_pc_system_c::deactivate_timer( unsigned i )
   timer[i].active = 0;
 }
 
-  unsigned
-bx_pc_system_c::unregisterTimer(int timerIndex)
+unsigned bx_pc_system_c::unregisterTimer(int timerIndex)
 {
   unsigned i = (unsigned) timerIndex;
 
@@ -541,7 +527,7 @@ bx_pc_system_c::unregisterTimer(int timerIndex)
   if (timer[i].active) {
     BX_PANIC(("unregisterTimer: timer '%s' is still active!", timer[i].id));
     return(0); // Fail.
-    }
+  }
 
   // Reset timer fields for good measure.
   timer[i].inUse      = 0; // No longer registered.
