@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pc_system.h,v 1.29 2004/07/06 19:59:10 vruppert Exp $
+// $Id: pc_system.h,v 1.32.2.1 2005/07/06 20:20:24 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2004  MandrakeSoft S.A.
@@ -26,16 +26,10 @@
 
 
 
-
 #define BX_MAX_TIMERS 64
 #define BX_NULL_TIMER_HANDLE 10000
 
 
-#if BX_SHOW_IPS
-extern unsigned long ips_count;
-#endif
-
-
 typedef void (*bx_timer_handler_t)(void *);
 
 
@@ -64,7 +58,7 @@ private:
                                //   has to be stored as well.
 #define BxMaxTimerIDLen 32
     char id[BxMaxTimerIDLen]; // String ID of timer.
-    } timer[BX_MAX_TIMERS];
+  } timer[BX_MAX_TIMERS];
 
   unsigned   numTimers;  // Number of currently allocated timers.
   unsigned   triggeredTimer;  // ID of the actually triggered timer.
@@ -110,33 +104,21 @@ public:
     return triggeredTimer;
   }
   static BX_CPP_INLINE void tick1(void) {
-#if BX_SHOW_IPS
-  {
-  extern unsigned long ips_count;
-  ips_count++;
-  }
-#endif
     if (--bx_pc_system.currCountdown == 0) {
       bx_pc_system.countdownEvent();
-      }
     }
-  static BX_CPP_INLINE void tickn(Bit64u n) {
-#if BX_SHOW_IPS
-  {
-  extern unsigned long ips_count;
-  ips_count += n;
   }
-#endif
+  static BX_CPP_INLINE void tickn(Bit64u n) {
     while (n >= Bit64u(bx_pc_system.currCountdown)) {
       n -= Bit64u(bx_pc_system.currCountdown);
       bx_pc_system.currCountdown = 0;
       bx_pc_system.countdownEvent();
       // bx_pc_system.currCountdown is adjusted to new value by countdownevent().
-      };
+    }
     // 'n' is not (or no longer) >= the countdown size.  We can just decrement
     // the remaining requested ticks and continue.
     bx_pc_system.currCountdown -= Bit32u(n);
-    }
+  }
 
   int register_timer_ticks(void* this_ptr, bx_timer_handler_t, Bit64u ticks,
                            bx_bool continuous, bx_bool active, const char *id);
@@ -147,14 +129,11 @@ public:
   static BX_CPP_INLINE Bit64u time_ticks() {
     return bx_pc_system.ticksTotal +
       Bit64u(bx_pc_system.currCountdownPeriod - bx_pc_system.currCountdown);
-    }
-  static BX_CPP_INLINE Bit64u getTicksTotal(void) {
-    return bx_pc_system.ticksTotal;
-    }
+  }
 
   static BX_CPP_INLINE Bit32u  getNumCpuTicksLeftNextEvent(void) {
     return bx_pc_system.currCountdown;
-    }
+  }
 #if BX_DEBUGGER
   static void timebp_handler(void* this_ptr);
 #endif
@@ -200,4 +179,4 @@ public:
   void    set_enable_a20(Bit8u value) BX_CPP_AttrRegparmN(1);
   bx_bool get_enable_a20(void);
   void    exit(void);
-  };
+};
