@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: plugin.h,v 1.32 2004/09/25 22:15:01 vruppert Exp $
+// $Id: plugin.h,v 1.40 2005/02/08 18:31:48 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 // This file provides macros and types needed for plugins.  It is based on
@@ -29,6 +29,7 @@ extern "C" {
 #define BX_PLUGIN_PARALLEL  "parallel"
 #define BX_PLUGIN_SERIAL    "serial"
 #define BX_PLUGIN_KEYBOARD  "keyboard"
+#define BX_PLUGIN_BUSMOUSE  "busmouse"
 #define BX_PLUGIN_HARDDRV   "harddrv"
 #define BX_PLUGIN_DMA       "dma"
 #define BX_PLUGIN_PIC       "pic"
@@ -103,7 +104,9 @@ extern "C" {
 
 ///////// keyboard macros
 #define DEV_mouse_motion(dx, dy, state) \
-    (bx_devices.pluginKeyboard->mouse_motion(dx, dy, state))
+    (bx_devices.pluginKeyboard->mouse_motion(dx, dy, 0, state))
+#define DEV_mouse_motion_ext(dx, dy, dz, state) \
+    (bx_devices.pluginKeyboard->mouse_motion(dx, dy, dz, state))
 #define DEV_kbd_gen_scancode(key) \
     (bx_devices.pluginKeyboard->gen_scancode(key))
 #define DEV_kbd_paste_bytes(bytes, count) \
@@ -128,6 +131,9 @@ extern "C" {
     (bx_devices.pluginHardDrive->set_cd_media_status(handle, status))
 #define DEV_hd_close_harddrive()  bx_devices.pluginHardDrive->close_harddrive()
 #define DEV_hd_present() (bx_devices.pluginHardDrive != &bx_devices.stubHardDrive)
+#define DEV_hd_bmdma_read_sector(a,b) bx_devices.pluginHardDrive->bmdma_read_sector(a,b)
+#define DEV_hd_bmdma_write_sector(a,b) bx_devices.pluginHardDrive->bmdma_write_sector(a,b)
+#define DEV_hd_bmdma_complete(a) bx_devices.pluginHardDrive->bmdma_complete(a)
 
 #define DEV_bulk_io_quantum_requested() (bx_devices.bulkIOQuantumsRequested)
 #define DEV_bulk_io_quantum_transferred() (bx_devices.bulkIOQuantumsTransferred)
@@ -185,15 +191,38 @@ extern "C" {
 #define DEV_pci_rd_memtype(addr) bx_devices.pluginPciBridge->rd_memType(addr)
 #define DEV_pci_wr_memtype(addr) bx_devices.pluginPciBridge->wr_memType(addr)
 #define DEV_pci_print_i440fx_state() bx_devices.pluginPciBridge->print_i440fx_state()
+#define DEV_ide_bmdma_present() bx_devices.pluginPciIdeController->bmdma_present()
 
 ///////// NE2000 macro
 #define DEV_ne2k_print_info(file,page,reg,brief) \
     bx_devices.pluginNE2kDevice->print_info(file,page,reg,brief)
 
-///////// Speaker macro
+///////// Speaker macros
 #define DEV_speaker_beep_on(frequency) bx_devices.pluginSpeaker->beep_on(frequency)
 #define DEV_speaker_beep_off() bx_devices.pluginSpeaker->beep_off()
 
+///////// Serial macro
+#define DEV_serial_mouse_enq(dx, dy, dz, state) \
+    (bx_devices.pluginSerialDevice->serial_mouse_enq(dx, dy, dz, state))
+
+///////// BUS mouse macro
+#define DEV_bus_mouse_enq(dx, dy, dz, state) \
+    (bx_devices.pluginBusMouse->bus_mouse_enq(dx, dy, 0, state))
+
+///////// USB device macros
+#if BX_SUPPORT_PCIUSB
+#define DEV_usb_mouse_enq(dx, dy, dz, state) \
+    (bx_devices.pluginPciUSBAdapter->usb_mouse_enq(dx, dy, dz, state))
+#define DEV_usb_mouse_enable(enable) \
+    (bx_devices.pluginPciUSBAdapter->usb_mouse_enable(enable))
+#define DEV_usb_key_enq(scan_code) \
+    (bx_devices.pluginPciUSBAdapter->usb_key_enq(scan_code))
+#define DEV_usb_keyboard_connected() \
+    (bx_devices.pluginPciUSBAdapter->usb_keyboard_connected())
+#define DEV_usb_mouse_connected() \
+    (bx_devices.pluginPciUSBAdapter->usb_mouse_connected())
+#endif
+
 //////// Memory macros
 #define DEV_register_memory_handlers(rh,rp,wh,wp,b,e) \
     bx_devices.mem->registerMemoryHandlers(rh,rp,wh,wp,b,e)
@@ -328,6 +357,7 @@ int plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[]);
   
 DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(harddrv)
 DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(keyboard)
+DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(busmouse)
 DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(serial)
 DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(unmapped)
 DECLARE_PLUGIN_INIT_FINI_FOR_MODULE(biosdev)
